#ifndef AG_MAP_BASE_H
#define AG_MAP_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <stdlib.h>
#include "agabt.h"
#include "agref.h"
#include "agcntnr.h"

template <class Key, class Value>
class AgKVPair {
  Key k;
  Value v;
public:
  AgKVPair(const Key &key = Key(), const Value &value = Value());
  AgKVPair(const AgKVPair<Key, Value> &);
  ~AgKVPair() {}

  const Key &key() const {return k;}
  Value &value() {return v;}
  const Value &value() const {return v;}

  int operator == (const AgKVPair<Key, Value> &p) const;
  int operator != (const AgKVPair<Key, Value> &p) const {return !(*this == p);}
};

template <class Key, class Value>
AgKVPair<Key, Value>::AgKVPair(const Key &key, const Value &value)
: k(key), v(value)
{
  // Nothing to do
}

template <class Key, class Value>
AgKVPair<Key, Value>::AgKVPair(const AgKVPair<Key, Value> &p)
: k(p.k), v(p.v)
{
  // Nothing to do
}

template <class Key, class Value>
int AgKVPair<Key, Value>::operator == (const AgKVPair<Key, Value> &p) const {
  return key() == p.key() && value() == p.value();
}


template <class Key, class Value>
int agABTHash(const AgKVPair<Key, Value> &p, int startValue) {
  return agABTHash(p.key(), agABTHash(p.value(), startValue));
}

template <class Key, class Value>
int agABTHash(const AgKVPair<Key, Value> &p) {
  return agABTHash(p, 0);
}

template <class Key, class Value>
class AgMapBase
: public AgAleatoryBinaryTree< AgKVPair<Key, Value> >,
  public AgReferenceCounter
{
  typedef AgKVPair<Key, Value> Pair;
  typedef AgAleatoryBinaryTree<AgKVPair<Key, Value> > Base;
  typedef AgMapBase<Key, Value> This;
  int keyHash(const Key &k) const;
  int equivalent(const Pair &x, const Pair &y) const
      {return x.key() == y.key();}
public:
  AgMapBase();
  AgMapBase(const AgContainerContents< AgKVPair<Key, Value> > &);
  AgMapBase(const This &m);
  virtual ~AgMapBase() {}
  Value *pValue(const Key &key) const;
  Value &value(const Key &key);
  int remove(const Key &key) {return findAndRemove(root, Pair(key), keyHash(key));}
  This &reset() {Base::reset(); return *this;}
  This &operator = (const This &s) {
    AgAleatoryBinaryTree<Pair>::operator = (s);
    return *this;
  }
  int size() const {return Base::size();}
  This &operator = (const AgContainerContents<Pair> &);
  Pair &operator [](int x) const;
  int operator == (const AgMapBase<Key, Value> &) const;
  int operator != (const AgMapBase<Key, Value> &x) const {return !(*this == x);}

};

template <class Key, class Value>
AgMapBase<Key, Value>::AgMapBase()
: Base(),
  AgReferenceCounter()
{
  // Nothing to do
}

template <class Key, class Value>
AgMapBase<Key, Value>::AgMapBase(const AgContainerContents< AgKVPair<Key, Value> > &c)
: Base(),
  AgReferenceCounter()
{
  int i = 0;
  while (i < c.size()) {
    const AgKVPair<Key, Value> &pair = c[i++];
    value(pair.key()) = pair.value();
  }
}

template <class Key, class Value>
AgMapBase<Key, Value>::AgMapBase(const This &m)
: Base(m),
  AgReferenceCounter()
{
  // Nothing to do
}
template <class Key, class Value>
int AgMapBase<Key, Value>::keyHash(const Key &k) const {
  return agABTHash(k);
}

template <class Key, class Value>
Value *AgMapBase<Key, Value>::pValue(const Key &key) const {
  Node *node = find(root, AgKVPair<Key, Value>(key), keyHash(key));
  return node ? &node->object.value() : NULL;
}

template <class Key, class Value>
Value &AgMapBase<Key, Value>::value(const Key &key) {
  return findOrCreate(root, AgKVPair<Key,Value>(key), keyHash(key))->object.value();
}

template <class Key, class Value>
AgKVPair<Key, Value> &AgMapBase<Key, Value>::operator [] (int x) const {
  assert((unsigned) x < (unsigned) size());
  return root->find(x);
}

template <class Key, class Value>
AgMapBase<Key, Value> &AgMapBase<Key, Value>::operator = (const AgContainerContents<Pair> &c) {
  reset();
  int i = 0;
  while (i < c.size()) {
    const AgKVPair<Key, Value> &pair = c[i++];
    value(pair.key()) = pair.value();
  }
  return *this;
}

template <class Key, class Value>
int AgMapBase<Key, Value>::operator == (const AgMapBase<Key, Value> &x) const {
  if (this == &x) return 1;
  int n = size();
  if (n != x.size()) return 0;
  while (n--) if (!find(root, AgKVPair<Key, Value>(x[n]), keyHash(x[n].key()))) return 0;
  n = size();
  while (n--) if (!x.find(root, AgKVPair<Key, Value>(operator [](n)), keyHash(operator[](n).key()))) return 0;
  return 1;
}

#endif
