/*
   dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays and subscripts

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dci.h and dci.cpp are created from
 dci.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   bcidefs.cpp             // Bytecode interpreter definitions
   dci.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348674
#include "dci.h"
#endif

#ifndef DCI_H_1027348674
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 456, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  argCode.append(CALL, idFunction(name, argCount));
  return argCode;
}


ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, CodeFragment &x) {
/* Line 129, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 133, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 134, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 152, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 154, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 155, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 158, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 162, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 164, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 167, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 168, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL) {
/* Line 171, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.code(PUSHC, Value(1));
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 172, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_14(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 185, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment::repeatLoop(s,x);
}

static inline CodeFragment ag_rp_15(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 186, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return lv.concat(x).append(SAP);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 187, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 189, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 190, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(RETURN);
}

static inline CodeFragment ag_rp_19(PCB_DECL, AgString &n, CodeFragment &x) {
/* Line 194, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.code(LOCATE, n).concat(x).append(ARRAY).append(SAP);
}

static inline CodeFragment ag_rp_20(PCB_DECL, CodeFragment &x) {
/* Line 206, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_21(PCB_DECL, CodeFragment &s) {
/* Line 209, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_22(PCB_DECL, AgString &n) {
/* Line 213, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_23(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 214, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_24(PCB_DECL, CodeFragment &code) {
/* Line 218, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_25(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 219, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_26(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 226, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 230, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_28(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 234, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_29(PCB_DECL, CodeFragment &x) {
/* Line 239, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_30(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 240, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_31(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 248, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_32(PCB_DECL, CodeFragment &x) {
/* Line 256, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_33(PCB_DECL, Constant &x) {
/* Line 257, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_34(PCB_DECL, CodeFragment &x) {
/* Line 258, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_35(PCB_DECL, CodeFragment &x) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_36(PCB_DECL, CodeFragment &x) {
/* Line 261, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_37(PCB_DECL, AgString &n) {
/* Line 264, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_38(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 265, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return lv.concat(x).append(SUBSCRIPT);
}

static inline CodeFragment ag_rp_39(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 268, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_40(PCB_DECL) {
/* Line 271, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_41(PCB_DECL, CodeFragment &code) {
/* Line 275, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_42(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_43(PCB_DECL, long x) {
/* Line 279, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_44(PCB_DECL, double x) {
/* Line 280, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_45(PCB_DECL, int x) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_46(PCB_DECL) {
/* Line 286, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_47(PCB_DECL) {
/* Line 287, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return NE;
}

static inline Opcode ag_rp_48(PCB_DECL) {
/* Line 288, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return LT;
}

static inline Opcode ag_rp_49(PCB_DECL) {
/* Line 289, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return LE;
}

static inline Opcode ag_rp_50(PCB_DECL) {
/* Line 290, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return GT;
}

static inline Opcode ag_rp_51(PCB_DECL) {
/* Line 291, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return GE;
}

static inline Opcode ag_rp_52(PCB_DECL) {
/* Line 294, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_53(PCB_DECL) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_54(PCB_DECL) {
/* Line 296, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return IOR;
}

static inline Opcode ag_rp_55(PCB_DECL) {
/* Line 297, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return XOR;
}

static inline Opcode ag_rp_56(PCB_DECL) {
/* Line 300, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 301, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return RDIV;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return IDIV;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 303, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 304, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return AND;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 305, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return LS;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 306, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return RS;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 309, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 310, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return NOT;
}

static inline AgString ag_rp_65(PCB_DECL, int c) {
/* Line 328, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_66(PCB_DECL, AgString &ns, int c) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_67(PCB_DECL, double x, long e) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_68(PCB_DECL, double x, long e) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_69(PCB_DECL, double i, double f) {
/* Line 339, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return i+f;
}

static inline double ag_rp_70(PCB_DECL, double i) {
/* Line 340, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return i;
}

static inline double ag_rp_71(PCB_DECL, double f) {
/* Line 341, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return f;
}

static inline double ag_rp_72(PCB_DECL, long x) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline double ag_rp_73(PCB_DECL, long x) {
/* Line 345, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x;
}

static inline double ag_rp_74(PCB_DECL, long x) {
/* Line 346, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_75(PCB_DECL, long x) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return (long)x;
}

static inline long ag_rp_76(PCB_DECL, long x) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return -(long)x;
}

static inline double ag_rp_77(PCB_DECL, int d) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_78(PCB_DECL, int d, double f) {
/* Line 354, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_79(PCB_DECL, int d) {
/* Line 362, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return d-'0';
}

static inline long ag_rp_80(PCB_DECL, long x, int d) {
/* Line 363, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_81(PCB_DECL, int d) {
/* Line 366, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return d-'0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 367, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL, long x, int d) {
/* Line 370, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_84(PCB_DECL, long x, int d) {
/* Line 371, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_85(PCB_DECL) {
/* Line 374, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 0;
}

static inline long ag_rp_86(PCB_DECL, long n, int d) {
/* Line 375, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_87(PCB_DECL) {
/* Line 378, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 0;
}

static inline long ag_rp_88(PCB_DECL, long n, int d) {
/* Line 379, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_89(PCB_DECL, int x) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return x - '0';
}

static inline int ag_rp_90(PCB_DECL, int d) {
/* Line 383, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_91(PCB_DECL, Value &s) {
/* Line 388, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_92(PCB_DECL, Constant &s, Value &e) {
/* Line 389, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_93(PCB_DECL, AgString &b) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_94(PCB_DECL) {
/* Line 396, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return AgString();
}

static inline AgString ag_rp_95(PCB_DECL, AgString &s, int c) {
/* Line 397, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return s.append(c);
}

static inline int ag_rp_96(PCB_DECL) {
/* Line 409, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\'';
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 410, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '"';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\?';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 412, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\\';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 413, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\a';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 414, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\b';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 415, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\f';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 416, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\n';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 417, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\r';
}

static inline int ag_rp_105(PCB_DECL) {
/* Line 418, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\t';
}

static inline int ag_rp_106(PCB_DECL) {
/* Line 419, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return '\v';
}

static inline int ag_rp_107(PCB_DECL, int d) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return d-'0';
}

static inline int ag_rp_108(PCB_DECL, int n, int d) {
/* Line 428, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_109(PCB_DECL, int n, int d) {
/* Line 431, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_110(PCB_DECL, int d) {
/* Line 434, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return d;
}

static inline int ag_rp_111(PCB_DECL, int n, int d) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_112(PCB_DECL, int d) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return d-'0';
}

static inline int ag_rp_113(PCB_DECL, int d) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_114(PCB_DECL, int c) {
/* Line 448, W:/agdev/agex/packages/xidek/pll/array/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  6, 6, 0, 6, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4,
  8, 0, 0, 0, 6, 0, 0, 6, 0, 0, 6, 6, 6, 4, 0, 0, 0, 6, 0, 0, 0, 0, 6,
  6, 6, 6, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 0, 0, 0, 6, 6, 6, 0, 6, 6, 0, 0, 0,
  0, 0, 0, 6, 0, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18,  0,  0, 19,  0,  0,  0, 20, 21, 22, 23, 24, 25,  0, 26,
    0, 27,  0, 28,  0, 29, 30,  0, 31, 32, 33, 34,  0, 35, 36, 37, 38, 39,
   40,  0, 41, 42, 43, 44,  0, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
   56, 57, 58, 59, 60, 61, 62, 63, 64,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 65, 66,  0, 67, 68, 69, 70, 71, 72, 73, 74,  0,  0, 75, 76, 77,
   78,  0,  0,  0, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92,
   93, 94, 95,  0,  0,  0,  0,  0, 96, 97, 98, 99,100,101,102,103,104,105,
  106,  0,  0,  0,107,108,109,110,111,112,113,114
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,151,  0,152,  0,153,  0,155,  0,157,  0,158,  0,162,  0,163,
  0,164,  0,165,  0,167,  0,166,  0,169,  0,168,  0,170,  0,172,
  0,177,  0,178,  0,189,  0,190,  0,193,  0,194,  0,195,  0,196,
  0,197,  0,198,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73, 80, 82, 87,
  255, 42,255, 42, 47,255, 61, 62,255, 78, 82,255, 69, 89,255, 85,255, 73,
   79, 85,255, 76, 78,255, 80, 84,255, 69,255, 76, 82,255, 72,255, 72, 79,
  255, 42, 47, 48, 58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80,
   82, 83, 84, 85, 87, 88,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 80,
   82, 87,255, 42, 47,255, 47, 48, 78,255, 42, 47,255, 47,255, 42, 47,255,
   80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,
  255, 61, 62,255, 42, 47, 58, 60, 62,255, 42, 47,255, 80, 84,255, 69,255,
   47, 65, 66, 68, 70, 73, 80, 82, 85, 87,255, 48, 78,255, 80, 84,255, 69,
  255, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,255, 76, 78,255, 80,
   84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 85, 87,255, 58,255,
   80, 84,255, 69,255, 65, 66, 68, 70, 73, 80, 82, 85, 87,255, 34, 39, 63,
   65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,255, 61, 62,255,
   73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 65, 66, 68,
   77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,255, 72,255, 72,
   79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,255, 73, 79,255,
   76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68, 77, 79, 83, 84,
   88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76, 78,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62, 65, 66, 68, 77,
   79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66, 68, 79, 84, 88,
  255, 84,255, 68,255, 76, 78,255, 80, 84,255, 69,255, 65, 66, 68, 69, 70,
   73, 80, 82, 85, 87,255, 42, 47,255, 61, 62,255, 73, 79,255, 76, 82,255,
   72,255, 72, 79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,
  255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61, 62,255,
   73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62, 65, 66,
   68, 77, 79, 83, 84, 88,255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,2,7,4,3,4,0,0,4,0,0,4,7,7,4,7,5,4,7,4,
  7,6,7,4,7,7,4,7,7,4,2,4,5,5,4,2,4,7,5,4,3,2,3,3,2,3,2,2,2,2,7,7,7,7,7,
  7,7,2,2,2,7,7,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,0,0,4,2,3,7,4,0,0,4,2,4,
  0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,7,2,7,4,0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,7,
  7,4,2,4,2,7,7,7,7,7,7,2,7,7,4,3,7,4,7,7,4,2,4,7,7,7,7,7,7,7,2,7,4,0,0,
  4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,7,4,3,4,7,7,4,2,4,7,7,7,7,7,7,2,
  7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,
  3,2,2,3,7,7,2,7,7,2,2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,
  7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,
  7,4,3,7,7,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,
  7,5,4,2,3,7,7,7,2,7,4,7,4,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,7,2,7,7,4,0,
  0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,7,5,4,
  7,7,7,2,7,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,
  7,4,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 83, 88,  0, 14, 10,  0,  0,  0,  0, 12, 24, 28, 18, 20, 30,  0, 16,
    0, 87,  0, 83, 88,  0,185,183,  0, 44, 12,  0, 24,  6,  0, 34,  0, 40,
    2, 28,  0,  0, 26,  0, 14, 10,  0,  0,  0, 46, 48,  0,  0,  0, 22,  4,
    0,174,  0,113,154,  0,187,  0,  0,  0,  0, 18, 20, 32, 42, 50, 36, 30,
    0,  0,  0,  8, 16, 38,  0, 14, 10,  0,  0,  0, 12, 24, 28, 18, 20, 30,
    0, 16,  0, 83, 88,  0,  0,113, 50,  0, 83, 88,  0,  0,  0, 83, 88,  0,
   14, 10,  0,  0,  0,  0, 12, 24, 28, 26, 18, 20, 30,  0, 16,  0, 83, 88,
    0,185,183,  0,174,  0,154,  0,187,  0, 83, 88,  0, 14, 10,  0,  0,  0,
    0, 12, 24, 28, 18, 20, 30,  0,  8, 16,  0,113, 50,  0, 14, 10,  0,  0,
    0, 12, 24, 28, 26, 18, 20, 30,  0, 16,  0, 83, 88,  0,  0, 26,  0, 14,
   10,  0,  0,  0,  0, 12, 24, 28,  0, 18, 20, 30,  0,  8, 16,  0,154,  0,
   14, 10,  0,  0,  0, 12, 24, 28, 18, 20, 30,  0,  8, 16,  0,125,124,126,
  128,129,130,131,132,133,134,139,127,  0,  0,  0, 83, 88,  0,185,183,  0,
   40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,174,  0,  0,187, 44,  6,  0,
   42, 36,  0,  0, 38,  0, 83, 88,  0,185,183,  0, 46, 48,  0,  0,  0, 22,
    4,  0,174,  0,  0,187, 42, 36,  0,  0, 38,  0,185,183,  0, 40,  2,  0,
   46, 48,  0,  0,  0, 22,  4,  0,174,  0,187, 44,  6,  0, 42, 36,  0,  0,
   38,  0, 83, 88,  0,  0,113, 34, 32, 50,  0,113, 34, 32, 50,  0,185,183,
    0, 40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,  0,187, 44,  6,  0, 42,
   36,  0,  0, 38,  0,185,183,  0, 22,  4,  0,  0,187,  6,  2, 36,  0, 38,
    0, 22,  0,  2,  0,  0, 26,  0, 14, 10,  0,  0,  0, 12, 24, 28,  0, 18,
   20, 30,  0,  8, 16,  0, 83, 88,  0,185,183,  0, 40,  2,  0, 46, 48,  0,
    0,  0, 22,  4,  0,174,  0,113,  0,187, 44,  6,  0, 42, 36,  0,  0, 38,
    0,113,  0, 22,  4,  0,  6,  2, 36,  0, 38,  0, 83, 88,  0,185,183,  0,
   40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,174,  0,154,  0,187, 44,  6,
    0, 42, 36,  0,  0, 38,  0,  6,  4,  0,  4,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 24, 28,  0,  4,  0,  1,  0,  5, 10, 14, 17, 19,  7, 32,
    0, 37,  0,  0,  0,  0,  0,  0,  0, 47, 49,  0, 53,  0,  0, 59,  0, 57,
   33, 63,  0, 66, 69,  0, 93, 97,  0, 42,  0,  0,  0,  0, 47,  0,101,  0,
    0, 39, 21, 41, 43, 24, 45, 27, 30, 35, 39, 71, 74, 76, 80, 83, 86, 88,
   45, 50, 52,104,109,114,  0,141,145,  0, 79,  0,117,122,127,131,134,136,
   82,149,  0,  0,  0,  0, 93,154,156,  0,  0,  0,  0,100,  0,  0,  0,  0,
  186,190,  0,108,  0,105,159,164,169,173,176,179,181,111,194,  0,  0,  0,
    0,  0,  0,  0,199,124,201,127,203,  0,  0,  0,  0,229,233,  0,139,  0,
  136,205,210,215,219,222,224,142,237,242,  0,247,249,  0,279,283,  0,158,
    0,252,257,262,266,269,272,274,161,287,  0,  0,  0,  0,306,309,  0,321,
  325,  0,179,  0,173,292,297,302,176,311,314,316,182,329,334,  0,339,  0,
  365,369,  0,198,  0,341,346,351,355,358,360,201,373,378,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,213,  0,  0,  0,  0,  0,  0,  0,
  392,  0,  0,  0,  0,  0,237,  0,399,  0,  0,383,228,231,385,387,390,234,
  394,397,240,242,402,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,264,  0,414,
    0,  0,405,258,261,407,409,412,267,269,417,  0,  0,  0,  0,429,  0,  0,
    0,  0,  0,288,  0,436,  0,  0,420,282,422,424,427,285,431,434,291,293,
  439,  0,  0,  0,  0,308,442,444,450,454,  0,457,459,465,469,  0,  0,  0,
    0,479,  0,  0,  0,  0,  0,328,  0,486,  0,  0,322,472,474,477,325,481,
  484,331,333,489,  0,  0,  0,  0,500,  0,  0,347,492,494,496,498,350,503,
    0,506,  0,510,  0,526,529,  0,541,545,  0,368,  0,512,517,522,365,531,
  534,536,371,549,554,  0,  0,  0,  0,  0,  0,  0,570,  0,  0,  0,  0,  0,
  393,  0,577,  0,  0,559,384,561,387,563,565,568,390,572,575,396,398,580,
    0,583,  0,591,  0,  0,585,587,589,417,594,  0,  0,  0,  0,  0,  0,  0,
  608,  0,  0,  0,  0,  0,435,  0,615,  0,  0,597,426,599,429,601,603,606,
  432,610,613,438,440,618,  0,621,623,  0,625,  0
};

static const unsigned short ag_key_index[] = {
    9,  0, 19, 55, 84,  0,  0, 19, 19, 84, 96,103,113,130,144,103,155,  0,
   96,163,184,103, 96, 96,  0,  0,  0,155,196,203,155,  0,155, 84,226,  0,
    0,226,245,  0,245,272,245,245,245,245,245,296,296,311, 96, 96,296,317,
   96,296,155,155,336,353,163,  0, 96,155,  0,  0, 96,155, 96,155,203,361,
  196,363,373,226,245,  0,  0,226,226,226,  0,245,  0,  0,155,103,  0,103,
    0,  0, 96,155, 96, 96, 96, 96, 96, 96, 96,155, 96, 96,155, 96, 96, 96,
   96, 96, 96,155,184,155,  0,  0, 96,155,  9,155,  9, 84,  9, 84,  0,  0,
    0,  0,401,415,415,336,420,  0,443,  0,457,245,245,155,  0, 96,155,460,
   96,155,363, 84
};

static const unsigned char ag_key_ends[] = {
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 47,0, 
42,0, 88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 79,78,71,0, 
79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
69,78,0, 78,84,73,76,0, 72,73,76,69,0, 79,82,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 88,0, 79,84,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
42,0, 61,0, 61,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 61,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 
89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 
79,78,71,0, 79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 61,0, 89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 
79,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 
82,0, 69,78,0, 79,82,0, 42,0, 61,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 89,0, 79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,145,145,145,145,145,145,145,145,144, 92,144,144,144,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,144,145,116,145,
  145,145,145,141,176,175,191,173,171,188, 99,192,111,146,146,146,146,146,
  146,146,110,110,145,156,184,182,186,145,145,147,147,147,147, 96,147,148,
  148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
  148,160,138,159,145,148,145,147,147,147,147, 96,147,148,148,148,148,148,
  148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
172,170,169,167,165,164,163,162,158,156,148,147,144,96,92,88,83,9,0,1,149,
  150,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  138,116,111,110,99,96,92,0,90,91,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  138,116,111,110,99,96,92,87,0,85,86,
144,92,88,83,0,1,
172,170,169,167,165,164,163,162,158,156,148,147,96,9,0,7,8,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  138,116,111,110,99,96,0,
92,0,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  138,116,111,110,99,96,92,0,
87,0,
172,170,169,167,165,164,163,162,158,156,148,147,96,9,0,4,10,11,12,13,15,18,
  19,23,25,27,28,29,30,31,32,35,36,37,38,40,42,44,161,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
148,147,144,96,92,88,83,0,1,149,150,
172,170,169,168,167,165,164,163,162,158,156,148,147,144,96,92,88,83,0,1,149,
  150,
192,191,188,187,186,185,184,183,182,176,175,174,173,171,160,159,156,154,148,
  147,146,144,111,110,96,92,88,83,0,1,149,150,
172,170,169,167,165,164,163,162,158,157,156,148,147,144,96,92,88,83,0,1,149,
  150,
148,147,144,96,92,88,83,0,1,149,150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
148,147,96,0,4,161,
198,188,176,173,156,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,
  149,150,
172,170,169,168,167,165,164,163,162,158,156,148,147,96,0,8,
172,170,169,168,167,165,164,163,162,158,157,156,151,148,147,144,96,92,88,83,
  9,0,1,149,150,
148,147,144,96,92,88,83,0,1,149,150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
148,147,96,0,4,161,
171,156,0,27,43,
171,156,0,27,43,
198,188,176,173,156,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,
  27,45,47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,
  180,181,199,
160,154,0,20,33,
172,170,169,167,165,164,163,162,158,157,156,148,147,96,0,8,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
148,147,96,0,4,19,161,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
172,170,169,167,165,164,163,162,158,156,148,147,96,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,30,31,32,35,36,37,38,40,42,44,161,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  139,138,134,133,132,131,130,129,128,127,126,125,124,116,111,110,99,96,0,
  117,
146,111,110,0,106,
146,111,110,0,100,106,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,139,
  138,134,133,132,131,130,129,128,127,126,125,124,116,111,110,99,96,0,120,
  121,122,123,135,136,137,142,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,144,116,92,88,83,0,1,149,150,
99,96,0,
96,0,
147,146,111,110,96,0,114,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,146,144,111,110,99,96,92,88,83,0,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,146,144,111,110,99,96,92,88,83,0,106,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,144,92,88,83,0,1,149,150,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,144,92,88,83,0,1,149,150,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,159,156,155,153,152,144,92,88,83,0,1,149,150,
176,0,56,
116,0,5,199,
198,188,178,177,176,175,173,148,147,146,144,141,116,113,111,110,99,96,92,88,
  83,0,1,149,150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
160,0,33,
198,188,178,177,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,
  16,19,45,47,49,51,52,53,54,56,58,59,60,61,64,71,81,95,98,101,102,103,
  107,161,179,180,181,199,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
174,0,55,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,49,51,
  52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,199,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,49,51,
  52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,199,
197,196,195,194,193,192,191,0,50,74,75,76,77,78,79,80,
190,189,188,187,186,185,184,183,182,173,0,46,48,52,65,66,67,68,69,70,71,72,
  73,
172,170,169,168,167,165,164,163,162,158,156,148,147,96,0,4,10,11,12,13,15,
  18,19,23,25,27,28,29,30,31,32,35,36,37,38,40,41,42,44,161,
160,0,33,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
148,147,96,0,4,161,
156,0,27,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
172,170,169,167,165,164,163,162,158,157,156,148,147,96,0,4,10,11,12,13,15,
  18,19,23,25,26,27,28,29,30,31,32,35,36,37,38,40,42,44,161,
166,0,39,
160,154,0,20,33,
152,0,17,
151,0,14,
192,191,188,186,184,182,176,175,173,171,160,159,156,148,147,146,145,144,141,
  139,138,134,133,132,131,130,129,128,127,126,125,124,116,111,110,99,96,0,
  118,120,121,122,123,135,136,137,
146,111,110,0,100,106,
146,111,0,
147,146,111,110,96,0,140,
146,111,0,
146,111,0,
147,146,111,110,96,0,140,
141,0,
146,111,110,0,100,106,
188,173,146,111,110,0,97,104,
188,173,146,111,110,0,97,104,
198,188,176,175,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,
  45,47,49,51,52,53,54,56,58,59,62,63,64,71,81,95,98,101,102,103,107,161,
  179,180,181,199,
175,144,92,88,83,0,1,149,150,
175,0,57,
175,144,92,88,83,0,1,149,150,
175,0,57,
175,0,57,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,49,51,
  52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,199,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,49,51,
  52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,199,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,47,49,
  51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,199,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,45,47,
  49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,181,
  199,
172,170,169,168,167,165,164,163,162,158,157,156,151,148,147,144,96,92,88,83,
  9,0,1,149,150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
159,0,34,
156,0,27,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
172,170,169,167,165,164,163,162,158,156,148,147,144,96,92,88,83,0,1,149,150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
172,170,169,167,165,164,163,162,158,156,148,147,144,96,92,88,83,0,1,149,150,
172,170,169,167,165,164,163,162,158,156,148,147,96,0,4,11,12,13,15,18,19,23,
  25,27,28,29,30,31,32,35,36,37,38,40,42,44,161,
172,170,169,167,165,164,163,162,158,156,148,147,144,96,92,88,83,0,1,149,150,
172,170,169,167,165,164,163,162,158,156,148,147,96,0,4,11,12,13,15,18,19,23,
  25,27,28,29,30,31,32,35,36,37,38,40,42,44,161,
146,111,110,0,105,
146,111,110,0,105,
171,0,43,
175,0,57,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,176,175,174,
  173,171,166,159,156,155,153,152,148,147,146,144,141,116,113,111,110,99,
  96,92,88,83,0,1,149,150,
176,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,54,56,58,59,64,95,
  98,101,102,103,107,161,179,180,181,199,
176,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,19,54,56,58,59,64,95,
  98,101,102,103,107,161,179,180,181,199,
197,196,195,194,193,192,191,0,50,74,75,76,77,78,79,80,
190,189,188,173,0,48,52,71,72,73,
159,0,34,
197,196,195,194,193,192,191,190,189,188,187,186,185,184,183,182,175,174,173,
  171,166,160,159,156,155,154,153,152,144,92,88,83,0,1,149,150,
156,0,27,
155,153,0,21,24,
146,111,110,0,106,
146,111,110,0,106,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
156,0,27,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
153,0,22,
198,188,176,173,148,147,146,144,141,116,113,111,110,99,96,92,88,83,0,1,149,
  150,
198,188,176,173,148,147,146,141,116,113,111,110,99,96,0,2,3,4,5,6,16,19,45,
  47,49,51,52,53,54,56,58,59,64,71,81,95,98,101,102,103,107,161,179,180,
  181,199,
152,0,17,
172,170,169,167,165,164,163,162,158,156,148,147,96,0,4,11,12,13,15,18,19,23,
  25,27,28,29,30,31,32,35,36,37,38,40,42,44,161,

};


static unsigned const char ag_astt[3089] = {
  8,8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,0,1,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,5,3,7,1,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,7,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,
  2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,
  2,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,
  5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,
  10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,
  10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,1,4,1,1,5,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,4,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,2,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,
  1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,1,1,
  1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,
  2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,1,1,1,1,
  1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,
  1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,7,2,1,1,7,1,1,1,7,1,1,5,1,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,
  2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,
  5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,
  1,4,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,2,2,1,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,
  1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,7,2,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,
  1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,
  1,2,7,2,2,1,2,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,2,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,
  2,1,1,1,1,4,1,2,2,2,2,1,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,1,1,1,7,1,1,3,1,7,2,1,4,7,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,7,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
277,277,1,2,279,277,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,
  90,
91,6,
83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,
  83,85,
86,8,
10,11,12,23,21,22,14,15,18,20,92,92,92,1,9,51,3,3,3,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,10,3,3,301,
278,278,3,278,3,1,2,11,3,3,299,
278,278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,12,3,3,298,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,93,
  93,93,3,93,93,93,3,1,2,13,3,3,290,
278,278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,14,3,3,292,
278,278,3,278,3,1,2,15,3,3,291,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,16,58,59,47,124,61,32,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
92,92,92,17,30,13,
278,278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,18,3,3,
  287,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,19,60,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,278,
  20,3,3,285,
278,278,3,278,3,1,2,21,3,3,294,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,22,3,3,293,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,23,3,3,296,
92,92,92,24,61,13,
62,20,25,23,63,
62,20,26,22,64,
50,51,49,54,20,92,92,112,37,34,120,118,112,36,92,27,58,59,47,124,61,65,52,
  20,59,58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,
  38,
66,68,28,69,67,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,29,70,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,30,58,59,47,124,61,71,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
92,92,92,31,51,72,13,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,32,58,59,47,124,61,73,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
10,11,12,23,21,22,14,15,18,20,92,92,92,33,51,6,6,74,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,
  127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,
  34,75,
117,117,117,101,117,
76,76,76,36,99,76,
82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,78,77,144,143,142,141,
  140,139,138,137,136,135,134,82,82,82,82,82,37,82,82,82,81,80,79,82,82,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,3,278,3,1,2,38,3,3,328,
83,84,39,
85,94,
123,122,122,122,123,111,121,
110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,
  110,110,110,110,110,110,110,119,110,119,116,102,102,110,110,110,42,
109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,
  109,109,109,109,109,109,109,113,109,113,113,100,100,109,109,109,43,113,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,3,3,1,2,44,3,3,310,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,3,3,1,2,45,3,3,309,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,3,3,1,2,46,3,3,308,
49,51,86,
34,60,125,38,
278,278,278,278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,
  49,3,3,305,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,50,3,3,327,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,51,3,3,317,
66,47,67,
50,51,87,89,49,54,92,92,112,37,34,120,118,112,36,92,53,58,59,47,124,61,91,
  52,59,58,58,58,57,56,55,53,46,55,90,88,48,79,80,40,39,43,35,42,41,13,46,
  45,44,38,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,54,3,3,302,
92,43,93,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,56,58,59,47,124,61,52,42,42,
  57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,57,58,59,47,124,61,52,41,41,
  57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
94,95,96,97,98,99,100,36,101,72,73,74,75,76,77,78,
102,103,51,105,106,107,108,109,110,54,34,111,104,68,62,63,64,65,66,67,69,70,
  71,
10,11,12,112,23,21,22,14,15,18,20,92,92,92,60,51,3,3,3,33,32,31,28,13,29,18,
  13,27,26,25,24,29,32,31,30,19,29,17,16,13,
66,61,113,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,62,3,3,300,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,63,58,59,47,124,61,33,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
92,92,92,64,31,13,
20,65,21,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,66,3,3,289,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,67,58,59,47,124,61,114,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,68,3,3,283,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,69,58,59,47,124,61,115,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
10,11,12,23,21,22,14,15,18,116,20,92,92,92,70,51,3,3,3,33,32,31,28,13,29,
  117,18,13,27,26,25,24,29,32,31,30,19,17,16,13,
118,71,28,
66,68,72,119,67,
120,73,121,
122,5,123,
128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
  78,77,144,143,142,141,140,139,138,137,136,135,134,126,128,128,128,128,
  75,128,128,128,128,81,80,79,128,
76,76,76,107,108,76,
148,148,77,
154,153,153,153,154,78,151,
150,150,146,
149,149,145,
154,153,153,153,154,133,152,
155,82,
76,76,76,98,97,76,
124,125,125,125,125,84,96,125,
124,125,125,125,125,85,95,125,
50,51,49,54,54,92,92,112,37,34,120,118,112,36,92,86,58,59,47,124,61,56,52,
  59,58,58,58,57,56,55,53,46,55,127,126,48,79,80,40,39,43,35,42,41,13,46,
  45,44,38,
278,3,3,1,2,87,3,3,307,
128,88,129,
278,3,3,1,2,89,3,3,306,
128,90,130,
128,91,45,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,92,3,3,303,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,93,58,59,47,124,61,52,44,44,
  57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,94,3,3,326,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,95,3,3,325,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,96,3,3,324,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,97,3,3,323,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,98,3,3,322,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,99,3,3,321,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,100,3,3,320,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,101,58,59,47,124,61,52,39,39,
  57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,102,3,3,319,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,103,3,3,318,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,104,58,59,47,124,61,52,131,
  131,131,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,105,3,3,316,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,106,3,3,315,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,107,3,3,314,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,108,3,3,313,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,109,3,3,312,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,110,3,3,311,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,111,58,59,47,124,61,52,132,58,
  58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,278,
  112,3,3,297,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,113,58,59,47,124,61,133,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
134,114,52,
20,115,17,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,116,3,3,286,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,117,58,59,47,124,61,135,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,118,3,3,295,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,119,58,59,47,124,61,136,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,120,3,3,281,
10,11,12,23,21,22,14,15,18,20,92,92,92,121,51,8,11,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
278,278,278,278,278,278,278,278,278,278,278,278,3,278,3,1,2,122,3,3,280,
10,11,12,23,21,22,14,15,18,20,92,92,92,123,51,7,10,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
114,114,114,124,137,
114,114,114,125,138,
62,55,139,
128,127,53,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,278,278,278,278,3,278,278,278,278,278,278,
  278,3,1,2,128,3,3,304,
49,92,92,112,37,34,120,118,112,36,92,129,58,59,47,124,61,52,50,53,46,50,48,
  40,39,43,35,42,41,13,46,45,44,38,
49,92,92,112,37,34,120,118,112,36,92,130,58,59,47,124,61,52,49,53,46,49,48,
  40,39,43,35,42,41,13,46,45,44,38,
94,95,96,97,98,99,100,37,101,72,73,74,75,76,77,78,
102,103,51,54,35,104,68,69,70,71,
134,133,140,
278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,278,
  278,278,278,278,278,278,278,278,278,3,3,1,2,134,3,3,288,
20,135,16,
141,14,136,143,142,
115,115,115,106,115,
115,115,115,105,115,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,139,58,59,47,124,61,57,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
20,140,24,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,141,3,3,284,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,142,58,59,47,124,61,15,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
144,143,145,
278,278,278,278,278,278,278,3,278,278,278,278,278,278,278,3,1,2,144,3,3,282,
50,51,49,54,92,92,112,37,34,120,118,112,36,92,145,58,59,47,124,61,146,52,59,
  58,58,58,57,56,55,53,46,55,48,79,80,40,39,43,35,42,41,13,46,45,44,38,
120,146,147,
10,11,12,23,21,22,14,15,18,20,92,92,92,147,51,9,12,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,

};


static const unsigned short ag_sbt[] = {
     0,  22,  51,  81,  87, 104, 130, 132, 159, 161, 200, 222, 233, 255,
   287, 309, 320, 366, 372, 395, 411, 436, 447, 469, 491, 497, 502, 507,
   555, 560, 576, 622, 629, 675, 713, 752, 757, 763, 808, 843, 846, 848,
   855, 891, 928, 962, 996,1030,1033,1037,1062,1084,1106,1109,1159,1181,
  1184,1227,1270,1286,1309,1349,1352,1374,1420,1426,1429,1451,1497,1519,
  1565,1605,1608,1613,1616,1619,1665,1671,1674,1681,1684,1687,1694,1696,
  1702,1710,1718,1767,1776,1779,1788,1791,1794,1816,1859,1881,1903,1925,
  1947,1969,1991,2013,2056,2078,2100,2144,2166,2188,2210,2232,2254,2276,
  2321,2346,2392,2395,2398,2420,2466,2487,2533,2554,2591,2612,2649,2654,
  2659,2662,2665,2710,2744,2778,2794,2804,2807,2843,2846,2851,2856,2861,
  2907,2910,2932,2978,2981,3003,3049,3052,3089
};


static const unsigned short ag_sbe[] = {
    18,  48,  78,  85, 101, 129, 131, 158, 160, 175, 218, 229, 251, 283,
   305, 316, 334, 369, 391, 409, 432, 443, 465, 487, 494, 499, 504, 522,
   557, 574, 590, 625, 643, 688, 750, 755, 760, 799, 839, 845, 847, 853,
   890, 926, 958, 992,1026,1031,1034,1058,1080,1102,1107,1125,1177,1182,
  1198,1241,1277,1296,1323,1350,1370,1388,1423,1427,1447,1465,1515,1533,
  1579,1606,1610,1614,1617,1656,1668,1673,1679,1683,1686,1692,1695,1699,
  1707,1715,1733,1772,1777,1784,1789,1792,1812,1830,1877,1899,1921,1943,
  1965,1987,2009,2027,2074,2096,2114,2162,2184,2206,2228,2250,2272,2290,
  2342,2360,2393,2396,2416,2434,2483,2501,2550,2567,2608,2625,2652,2657,
  2660,2663,2706,2721,2755,2785,2798,2805,2839,2844,2848,2854,2859,2875,
  2908,2928,2946,2979,2999,3017,3050,3065,3089
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,2,3,2,2,6,1,1,1,3,3,2,3,2,3,1,
  3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,
  2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 21, 21, 23, 23,
   23, 23, 23, 23, 23, 23, 23, 25, 15, 18, 13, 28, 30, 30, 31, 31, 16, 16,
   45, 45, 47, 47, 49, 49, 49, 51, 51, 54, 54, 54, 54, 54, 54, 19, 19, 59,
   62, 62, 63, 63, 58, 58, 58, 58, 46, 46, 46, 46, 46, 46, 48, 48, 48, 48,
   50, 50, 50, 50, 50, 50, 50, 53, 53,  1, 85, 85, 86, 86,  1, 90, 90, 91,
   91,  1,161,161,180,180,180, 95, 95, 95, 98, 98, 98,104,104, 97, 97,100,
  100,179,179,179,101,101,105,105,102,102,103,103,107,107,114,114, 64, 64,
  199,117,117,118,118,120,120,120,121,121,121,121,121,121,121,121,121,121,
  121,122,122,122,135,136,137,123,123,140,140,181,142,142,106, 82, 82, 84,
   84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84,
   84, 84, 84, 84, 84, 84, 84, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89,
   89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 93, 93, 93, 94,
   94, 94, 94, 94, 94,108,108,109,109,109,112,112,115,115,119,119,119,119,
  119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,119,
  119,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
  143,143,143,143,143,143,149,149,150,150, 14, 17, 22, 20, 24, 27, 26, 29,
   34, 33,  4, 32, 35, 36, 37, 39, 38, 41, 40, 42, 43, 44, 52, 55, 57, 56,
   60, 61,  2,  3,  6, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77,
   78, 79, 80, 81,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_6 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER)); break;
    case 13: VNO AG_WRAP_6(ag_rp_13(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 15: VRO(AG_WRAP_6 *, ag_rp_15(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: VNO AG_WRAP_6(ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_6 *); break;
    case 20: VNO AG_WRAP_6(ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 21: VNO AG_WRAP_6(ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 22: VNO AG_WRAP_6(ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 23: VRO(AG_WRAP_6 *, ag_rp_23(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 28: VRO(AG_WRAP_6 *, ag_rp_28(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 29: VNO AG_WRAP_6(ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 30: VNO AG_WRAP_6(ag_rp_30(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 31: VRO(AG_WRAP_6 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 32: VNO AG_WRAP_6(ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 33: VRO(AG_WRAP_8 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 34: VRO(AG_WRAP_6 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 35: VNO AG_WRAP_6(ag_rp_35(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 36: VNO AG_WRAP_6(ag_rp_36(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 37: VRO(AG_WRAP_4 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 38: VRO(AG_WRAP_6 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VRO(AG_WRAP_4 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 40: VNO AG_WRAP_7(ag_rp_40(PCB_POINTER)); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 42: VRO(AG_WRAP_7 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 43: VNO AG_WRAP_8(ag_rp_43(PCB_POINTER, V(0,(long *)))); break;
    case 44: VNO AG_WRAP_8(ag_rp_44(PCB_POINTER, V(0,(double *)))); break;
    case 45: VNO AG_WRAP_8(ag_rp_45(PCB_POINTER, V(0,(int *)))); break;
    case 46: V(0,(Opcode *)) = ag_rp_46(PCB_POINTER); break;
    case 47: V(0,(Opcode *)) = ag_rp_47(PCB_POINTER); break;
    case 48: V(0,(Opcode *)) = ag_rp_48(PCB_POINTER); break;
    case 49: V(0,(Opcode *)) = ag_rp_49(PCB_POINTER); break;
    case 50: V(0,(Opcode *)) = ag_rp_50(PCB_POINTER); break;
    case 51: V(0,(Opcode *)) = ag_rp_51(PCB_POINTER); break;
    case 52: V(0,(Opcode *)) = ag_rp_52(PCB_POINTER); break;
    case 53: V(0,(Opcode *)) = ag_rp_53(PCB_POINTER); break;
    case 54: V(0,(Opcode *)) = ag_rp_54(PCB_POINTER); break;
    case 55: V(0,(Opcode *)) = ag_rp_55(PCB_POINTER); break;
    case 56: V(0,(Opcode *)) = ag_rp_56(PCB_POINTER); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: VNO AG_WRAP_4(ag_rp_65(PCB_POINTER, V(0,(int *)))); break;
    case 66: VRO(AG_WRAP_4 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 67: V(0,(double *)) = ag_rp_67(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 68: V(0,(double *)) = ag_rp_68(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 69: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 70: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(double *))); break;
    case 71: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(1,(double *))); break;
    case 72: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 73: V(0,(double *)) = ag_rp_73(PCB_POINTER, V(0,(long *))); break;
    case 74: V(0,(double *)) = ag_rp_74(PCB_POINTER, V(0,(long *))); break;
    case 75: V(0,(long *)) = ag_rp_75(PCB_POINTER, V(1,(long *))); break;
    case 76: V(0,(long *)) = ag_rp_76(PCB_POINTER, V(1,(long *))); break;
    case 77: V(0,(double *)) = ag_rp_77(PCB_POINTER, V(0,(int *))); break;
    case 78: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 79: V(0,(long *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 80: V(0,(long *)) = ag_rp_80(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 81: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 83: V(0,(long *)) = ag_rp_83(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 84: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 85: V(0,(long *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 87: V(0,(long *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 89: V(0,(int *)) = ag_rp_89(PCB_POINTER, V(0,(int *))); break;
    case 90: V(0,(int *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 91: VRO(AG_WRAP_5 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 92: VRO(AG_WRAP_8 *, ag_rp_92(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 93: VNO AG_WRAP_5(ag_rp_93(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 94: VNO AG_WRAP_4(ag_rp_94(PCB_POINTER)); break;
    case 95: VRO(AG_WRAP_4 *, ag_rp_95(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 96: V(0,(int *)) = ag_rp_96(PCB_POINTER); break;
    case 97: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_105(PCB_POINTER); break;
    case 106: V(0,(int *)) = ag_rp_106(PCB_POINTER); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER, V(1,(int *))); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(1,(int *))); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(0,(int *))); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(0,(int *))); break;
    case 114: V(0,(int *)) = ag_rp_114(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[200] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "\"UNTIL\"",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "']'",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"UNTIL\"",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'+'",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


