/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays and subscripts

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>                                     // to declare tolower()
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

// struct to describe a for loop
struct ForControl {
  Value lvalue;
  long begin;
  long increment;
  long end;
  ForControl() : lvalue(), begin(0), increment(1), end(0) {}
  ForControl(const Value &lv,
      const Value &b, const Value &i,
      const Value &e)
  : lvalue(lv),
    begin(b.getLong()),
    increment(i.getLong()),
    end(e.getLong())
  {
    // Nothing to do
  }
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146392
#include "dxi.h"
#endif

#ifndef DXI_H_1031146392
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 548, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

ForControl dxi_pcb_struct::forControl(const Value &lv, const Value &b, const Value &i, const Value &e) {
  if (mode == executeMode) return ForControl(lv, b, i, e);
  return ForControl();
}

void dxi_pcb_struct::forLoop(ForControl &fc) {
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  if (fc.increment == 0) reportError("Loop increment is zero");
  // Calculate number of times through the loop
  int count = (fc.end + fc.increment - fc.begin)/fc.increment;
  int index = fc.begin;
  // Set mode for parsing loop
  loopPCB.mode = (mode == executeMode && count > 0) ? executeMode : breakMode;
  if (loopPCB.mode == executeMode) fc.lvalue = index;
  while (count--) {
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    index += fc.increment;
    if (loopPCB.mode == executeMode) fc.lvalue = index;
    if (loopPCB.mode == breakMode) return;
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  return pcb.returnValue;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 201, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return Value();
}

static inline void ag_rp_5(PCB_DECL) {
/* Line 208, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 211, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 214, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 217, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 240, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 242, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 246, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.simpleLoop("repeatLoop");
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 261, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_14(PCB_DECL, ForControl &fc) {
/* Line 262, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.forLoop(fc);
}

static inline void ag_rp_15(PCB_DECL, Value &x) {
/* Line 266, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_16(PCB_DECL, AgString &n, Value &x) {
/* Line 270, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  VALUE(PCB.dataset.value(n) = Value(x, 0));
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static void ag_rp_18(PCB_DECL, Value &x) {
/* Line 287, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static void ag_rp_19(PCB_DECL, Value &x) {
/* Line 291, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
if (PCB.mode != executeMode || x.isTrue()) PCB.mode = breakMode;
}

static inline ForControl ag_rp_20(PCB_DECL, Value &lv, Value &b, Value &i, Value &e) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return PCB.forControl(lv, b, i, e);
}

static inline Value ag_rp_21(PCB_DECL) {
/* Line 298, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(1);
}

static inline Value ag_rp_22(PCB_DECL, Value &x) {
/* Line 299, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x;
}

static void ag_rp_23(PCB_DECL, Value &x) {
/* Line 304, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_24(PCB_DECL) {
/* Line 308, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_25(PCB_DECL, Value &lv, Value &x) {
/* Line 311, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  VALUE(lv = x);
}

static inline void ag_rp_26(PCB_DECL, AgString &n) {
/* Line 318, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_27(PCB_DECL, AgString &n) {
/* Line 319, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_28(PCB_DECL, Value &x) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_29(PCB_DECL, Value &x) {
/* Line 324, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  PCB.print(x);
}

static inline Value ag_rp_30(PCB_DECL) {
/* Line 327, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return Value();
}

static inline Value ag_rp_31(PCB_DECL, Value &x, Value &y) {
/* Line 333, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_32(PCB_DECL, Value &x, Value &y) {
/* Line 334, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_33(PCB_DECL, Value &x, Value &y) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_34(PCB_DECL, Value &x, Value &y) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_35(PCB_DECL, Value &x, Value &y) {
/* Line 337, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_36(PCB_DECL, Value &x, Value &y) {
/* Line 338, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_37(PCB_DECL, Value &x, Value &y) {
/* Line 342, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x + y);
}

static inline Value ag_rp_38(PCB_DECL, Value &x, Value &y) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x - y);
}

static inline Value ag_rp_39(PCB_DECL, Value &x, Value &y) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x | y);
}

static inline Value ag_rp_40(PCB_DECL, Value &x, Value &y) {
/* Line 345, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x ^ y);
}

static inline Value ag_rp_41(PCB_DECL, Value &x, Value &y) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x * y);
}

static inline Value ag_rp_42(PCB_DECL, Value &x, Value &y) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x.rdiv(y));
}

static inline Value ag_rp_43(PCB_DECL, Value &x, Value &y) {
/* Line 351, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x.idiv(y));
}

static inline Value ag_rp_44(PCB_DECL, Value &x, Value &y) {
/* Line 352, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x % y);
}

static inline Value ag_rp_45(PCB_DECL, Value &x, Value &y) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x & y);
}

static inline Value ag_rp_46(PCB_DECL, Value &x, Value &y) {
/* Line 354, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x << y);
}

static inline Value ag_rp_47(PCB_DECL, Value &x, Value &y) {
/* Line 355, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x >> y);
}

static inline Value ag_rp_48(PCB_DECL, Value &x) {
/* Line 359, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_49(PCB_DECL, Value &x) {
/* Line 360, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_50(PCB_DECL, Value &x) {
/* Line 361, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_51(PCB_DECL, Value &x, Value &y) {
/* Line 369, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_52(PCB_DECL, Value &x) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x;
}

static inline Value ag_rp_53(PCB_DECL, Value &lv) {
/* Line 379, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_54(PCB_DECL, Value &x) {
/* Line 381, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_55(PCB_DECL, Value &x) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_56(PCB_DECL, AgString &n) {
/* Line 385, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_57(PCB_DECL, Value &lv, Value &x) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(lv[x]);
}

static inline Value ag_rp_58(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 389, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(callFunction(n, args));
}

static inline AgStack<Value> ag_rp_59(PCB_DECL) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_60(PCB_DECL, Value &x) {
/* Line 396, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_61(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 397, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_62(PCB_DECL, long x) {
/* Line 400, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_63(PCB_DECL, double x) {
/* Line 401, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_64(PCB_DECL, int x) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_65(PCB_DECL, int c) {
/* Line 422, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_66(PCB_DECL, AgString &ns, int c) {
/* Line 423, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_67(PCB_DECL, double x, long e) {
/* Line 428, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_68(PCB_DECL, double x, long e) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_69(PCB_DECL, double i, double f) {
/* Line 432, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return i+f;
}

static inline double ag_rp_70(PCB_DECL, double i) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return i;
}

static inline double ag_rp_71(PCB_DECL, double f) {
/* Line 434, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return f;
}

static inline double ag_rp_72(PCB_DECL, long x) {
/* Line 437, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x;
}

static inline double ag_rp_73(PCB_DECL, long x) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x;
}

static inline double ag_rp_74(PCB_DECL, long x) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_75(PCB_DECL, long x) {
/* Line 442, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return (long)x;
}

static inline long ag_rp_76(PCB_DECL, long x) {
/* Line 443, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_77(PCB_DECL, int d) {
/* Line 446, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_78(PCB_DECL, int d, double f) {
/* Line 447, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_79(PCB_DECL, int d) {
/* Line 455, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return d-'0';
}

static inline long ag_rp_80(PCB_DECL, long x, int d) {
/* Line 456, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_81(PCB_DECL, int d) {
/* Line 459, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return d-'0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 460, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL, long x, int d) {
/* Line 463, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_84(PCB_DECL, long x, int d) {
/* Line 464, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_85(PCB_DECL) {
/* Line 467, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 0;
}

static inline long ag_rp_86(PCB_DECL, long n, int d) {
/* Line 468, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_87(PCB_DECL) {
/* Line 471, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 0;
}

static inline long ag_rp_88(PCB_DECL, long n, int d) {
/* Line 472, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_89(PCB_DECL, int x) {
/* Line 475, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return x - '0';
}

static inline int ag_rp_90(PCB_DECL, int d) {
/* Line 476, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_91(PCB_DECL, Value &s, Value &e) {
/* Line 482, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_92(PCB_DECL, AgString &b) {
/* Line 485, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_93(PCB_DECL) {
/* Line 488, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_94(PCB_DECL, AgString &s, int c) {
/* Line 489, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_95(PCB_DECL) {
/* Line 501, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\'';
}

static inline int ag_rp_96(PCB_DECL) {
/* Line 502, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '"';
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 503, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\?';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 504, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\\';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 505, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\a';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 506, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\b';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 507, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\f';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 508, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\n';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 509, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\r';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 510, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\t';
}

static inline int ag_rp_105(PCB_DECL) {
/* Line 511, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return '\v';
}

static inline int ag_rp_106(PCB_DECL, int d) {
/* Line 517, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return d-'0';
}

static inline int ag_rp_107(PCB_DECL, int n, int d) {
/* Line 520, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_108(PCB_DECL, int n, int d) {
/* Line 523, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_109(PCB_DECL, int d) {
/* Line 526, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return d;
}

static inline int ag_rp_110(PCB_DECL, int n, int d) {
/* Line 527, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_111(PCB_DECL, int d) {
/* Line 530, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return d-'0';
}

static inline int ag_rp_112(PCB_DECL, int d) {
/* Line 531, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_113(PCB_DECL, int c) {
/* Line 540, W:/agdev/agex/packages/xidek/pll/array/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 4, 5, 0, 5, 0, 0, 0, 0, 5, 0, 0, 0, 5, 5, 0, 0,
  0, 0, 0, 0, 0, 5, 4, 0, 0, 0, 5, 0, 5, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  5, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5,
  5, 0, 5, 5, 0, 0, 0, 0, 0, 5, 0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<ForControl >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  0,  0,  0,  0,  6,  7,  8,  0,  9,
   10, 11,  0,  0,  0, 12,  0,  0, 13,  0, 14,  0,  0, 15, 16, 17, 18, 19,
   20, 21, 22,  0, 23, 24, 25, 26, 27,  0,  0, 28, 29, 30, 31, 32, 33,  0,
   34, 35, 36, 37,  0, 38, 39, 40, 41, 42, 43, 44,  0, 45, 46, 47,  0, 48,
   49,  0, 50,  0, 51, 52, 53, 54, 55, 56,  0, 57, 58, 59, 60,  0, 61,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 62, 63,  0, 64, 65, 66, 67, 68,
   69, 70, 71,  0,  0, 72, 73, 74, 75,  0,  0,  0, 76, 77, 78, 79, 80, 81,
   82, 83, 84, 85, 86, 87,  0, 88, 89, 90, 91,  0,  0,  0,  0,  0, 92, 93,
   94, 95, 96, 97, 98, 99,100,101,102,  0,  0,  0,103,104,105,106,107,108,
  109,110
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,163,  0, 20,  0, 21,  0, 22,  0, 23,  0,168,  0,172,  0,173,
  0,174,  0,175,  0,176,  0,177,  0,179,  0,181,  0,180,  0,182,
  0,184,  0,183,  0,185,  0,187,  0,196,  0,197,  0,200,  0,201,
  0,202,  0,203,  0,204,  0,205,  0,209,  0,210,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 67, 84,255, 47, 82, 83, 87,255, 42,255, 42, 47,255, 61,
   62,255, 78, 82,255, 69, 89,255, 85,255, 73, 79, 85,255, 76, 78,255, 76,
  255, 84,255, 65,255, 69,255, 80, 84,255, 69,255, 76, 82,255, 67, 72, 84,
  255, 72, 79,255, 76,255, 69,255, 76,255, 73,255, 72,255, 42, 47, 48, 58,
   60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85, 87,
   88,255, 67, 84,255, 82, 83, 87,255, 42, 47,255, 80, 84,255, 69,255, 47,
   65, 66, 68, 70, 73, 80, 82, 87,255, 42, 47,255, 80, 84,255, 69,255, 47,
   65, 66, 68, 70, 73, 80, 82, 85, 87,255, 42, 47,255, 47, 48, 78,255, 80,
   84,255, 69,255, 65, 66, 68, 70, 73, 80, 82, 87,255, 80, 84,255, 69,255,
   65, 66, 68, 70, 73, 80, 82, 85, 87,255, 48, 78,255, 42, 47,255, 61, 62,
  255, 42, 47, 58, 60, 62,255, 42, 47,255, 47,255, 42, 47,255, 80, 84,255,
   69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,255, 76, 78,
  255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 85, 87,255,
   80, 84,255, 69,255, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 58,255, 68,
  255, 34, 39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,
  255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,
  255, 72,255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68,
   77, 79, 83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76,
   78,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62,
   65, 66, 68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66,
   68, 79, 84, 88,255, 84,255, 76, 78,255, 80, 84,255, 69,255, 65, 66, 68,
   69, 70, 73, 80, 82, 85, 87,255, 42, 47,255, 61, 62,255, 73, 79,255, 76,
   82,255, 72,255, 72, 79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83,
   84, 88,255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61,
   62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62,
   65, 66, 68, 77, 79, 83, 84, 88,255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,2,7,4,3,4,0,0,4,0,0,4,7,7,4,7,5,4,7,4,7,6,7,4,7,7,4,
  7,4,6,4,2,4,2,4,2,7,4,2,4,5,5,4,7,2,7,4,7,5,4,7,4,6,4,2,4,2,4,2,4,3,2,
  3,3,2,3,2,2,2,2,7,7,7,7,7,7,7,2,2,2,7,2,7,4,7,7,4,7,2,7,4,0,0,4,7,7,4,
  2,4,2,7,7,7,7,7,7,2,7,4,0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,2,7,7,4,0,0,4,2,
  3,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,7,4,3,7,4,
  0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,2,4,0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,7,2,7,
  4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,7,4,7,7,4,2,4,7,7,7,7,7,7,
  7,2,7,4,3,4,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,
  2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,
  3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,
  4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,
  7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,4,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,7,2,7,
  7,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,
  7,5,4,7,7,7,2,7,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,
  7,2,2,7,4,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 94, 99,  0,  2,  8,  0,  0,  6,  0,  4,  0, 98,  0, 94, 99,  0,189,
  193,  0, 48, 12,  0, 32, 24,  0, 58,  0, 44,  0, 36,  0, 30, 34,  0,  6,
    0, 14,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0, 50, 52,  0,  2,  0,  8,
    0, 28, 22,  0,  4,  0, 18,  0,  0,  0,  0,  0,  0,  0,206,  0,124,178,
    0,191,  0,  0,  0,  0, 20, 26, 56, 46, 54, 40, 38,  0,  0,  0, 16,  0,
   42,  0,  2,  8,  0,  6,  0,  4,  0, 94, 99,  0, 14, 10,  0,  0,  0,  0,
   12, 32, 36, 20, 26, 38,  0, 18,  0, 94, 99,  0, 14, 10,  0,  0,  0,  0,
   12, 32, 36, 20, 26, 38,  0, 16, 18,  0, 94, 99,  0,  0,124, 54,  0, 14,
   10,  0,  0,  0, 12, 32, 36, 20, 26, 38,  0, 18,  0, 14, 10,  0,  0,  0,
   12, 32, 36, 20, 26, 38,  0, 16, 18,  0,124, 54,  0, 94, 99,  0,189,193,
    0,206,  0,178,  0,191,  0, 94, 99,  0,  0,  0, 94, 99,  0, 14, 10,  0,
    0,  0,  0, 12, 32, 36, 34, 20, 26, 38,  0, 18,  0, 94, 99,  0, 30, 34,
    0, 14, 10,  0,  0,  0,  0, 12, 32, 36,  0, 20, 26, 38,  0, 16, 18,  0,
   14, 10,  0,  0,  0, 12, 32, 36, 34, 20, 26, 38,  0, 18,  0,178,  0,  0,
    0,136,135,137,139,140,141,142,143,144,145,150,138,  0,  0,  0, 94, 99,
    0,189,193,  0, 44,  0,  0, 50, 52,  0,  0,  0, 28, 22,  0,206,  0,  0,
  191, 48, 24,  0, 46, 40,  0,  0, 42,  0, 94, 99,  0,189,193,  0, 50, 52,
    0,  0,  0, 28, 22,  0,206,  0,  0,191, 46, 40,  0,  0, 42,  0,189,193,
    0, 44,  0,  0, 50, 52,  0,  0,  0, 28, 22,  0,206,  0,191, 48, 24,  0,
   46, 40,  0,  0, 42,  0, 94, 99,  0,  0,124, 58, 56, 54,  0,124, 58, 56,
   54,  0,189,193,  0, 44,  0,  0, 50, 52,  0,  0,  0, 28, 22,  0,  0,191,
   48, 24,  0, 46, 40,  0,  0, 42,  0,189,193,  0, 28, 22,  0,  0,191, 24,
    0, 40,  0, 42,  0, 28,  0, 30, 34,  0, 14, 10,  0,  0,  0, 12, 32, 36,
    0, 20, 26, 38,  0, 16, 18,  0, 94, 99,  0,189,193,  0, 44,  0,  0, 50,
   52,  0,  0,  0, 28, 22,  0,206,  0,124,  0,191, 48, 24,  0, 46, 40,  0,
    0, 42,  0,124,  0, 28, 22,  0, 24,  0, 40,  0, 42,  0, 94, 99,  0,189,
  193,  0, 44,  0,  0, 50, 52,  0,  0,  0, 28, 22,  0,206,  0,178,  0,191,
   48, 24,  0, 46, 40,  0,  0, 42,  0, 24, 22,  0, 22,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 10, 15,  0,  1,  0,  4, 23,  0, 32,  0,  0,  0,  0,  0,
    0,  0, 42, 44,  0, 48,  0,  0, 54,  0, 52, 26, 58,  0, 61, 64,  0, 88,
    0, 35,  0, 37,  0, 39,  0, 41, 92,  0, 43,  0,  0,  0,  0, 96, 48,101,
    0,109,  0,  0,117,  0, 58,  0, 60,  0, 62,  0, 64,  0, 34, 14, 36, 38,
   17, 40, 20, 23, 28, 32, 66, 69, 71, 75, 78, 81, 83, 46, 51, 55,112, 66,
  121,  0,134,139,  0,124, 92,147,  0,  0,  0,  0,180,184,  0,102,  0, 99,
  156,161,166,170,173,175,105,188,  0,  0,  0,  0,217,221,  0,120,  0,117,
  193,198,203,207,210,212,123,225,230,  0,  0,  0,  0,136,235,237,  0,264,
  268,  0,143,  0,240,245,250,254,257,259,146,272,  0,301,305,  0,157,  0,
  277,282,287,291,294,296,160,309,314,  0,319,321,  0,  0,  0,  0,  0,  0,
    0,324,175,326,178,328,  0,  0,  0,  0,187,  0,  0,  0,  0,357,361,  0,
  195,  0,192,330,335,340,344,347,350,352,198,365,  0,  0,  0,  0,384,387,
    0,399,403,  0,217,  0,211,370,375,380,214,389,392,394,220,407,412,  0,
  444,448,  0,234,  0,417,422,427,431,434,437,439,237,452,  0,457,  0,459,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,253,  0,  0,  0,
    0,  0,  0,  0,470,  0,  0,  0,  0,  0,277,  0,477,  0,  0,461,268,271,
  463,465,468,274,472,475,280,282,480,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,304,  0,492,  0,  0,483,298,301,485,487,490,307,309,495,  0,  0,  0,
    0,507,  0,  0,  0,  0,  0,328,  0,514,  0,  0,498,322,500,502,505,325,
  509,512,331,333,517,  0,  0,  0,  0,348,520,522,528,532,  0,535,537,543,
  547,  0,  0,  0,  0,557,  0,  0,  0,  0,  0,368,  0,564,  0,  0,362,550,
  552,555,365,559,562,371,373,567,  0,  0,  0,  0,578,  0,  0,387,570,572,
  574,576,390,581,  0,584,  0,602,605,  0,617,621,  0,406,  0,588,593,598,
  403,607,610,612,409,625,630,  0,  0,  0,  0,  0,  0,  0,646,  0,  0,  0,
    0,  0,431,  0,653,  0,  0,635,422,637,425,639,641,644,428,648,651,434,
  436,656,  0,659,  0,667,  0,  0,661,663,665,455,670,  0,  0,  0,  0,  0,
    0,  0,684,  0,  0,  0,  0,  0,473,  0,691,  0,  0,673,464,675,467,677,
  679,682,470,686,689,476,478,694,  0,697,699,  0,701,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 12, 68, 95,  0,  0, 12, 12,107,125,139,107,148,162,172,148,181,
  190,139,190,200,222,222,190,172,  0,139,239,222,249,  0,139,  0,  0,  0,
  172,  0,251,172,148,162,266,  0,  0,266,285,  0,285,312,285,285,285,285,
  285,336,336,351,336,357,139,139,139,336,172,172,172,376,393,251,148,239,
  139,172,139,172,249,  0,139,172,  0,  0,222,401,411,139,172,266,285,  0,
    0,266,266,266,  0,285,  0,  0,172,190,  0,190,  0,  0,139,172,139,172,
  139,172,139,172,139,172,139,172,139,172,139,172,139,172,139,172,172,172,
  139,172,139,172,139,172,139,172,139,172,139,172,148,222,  0,172,172,107,
  107,148,  0,  0,  0,  0,  0,439,453,453,376,376,376,376,458,458,458,458,
  458,458,481,495,  0,285,285,172,139,172,498,  0,139,172
};

static const unsigned char ag_key_ends[] = {
69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 47,0, 42,0, 
88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 79,78,71,0, 
79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 79,79,80,0, 85,82,78,0, 
82,73,80,84,0, 65,84,69,77,69,78,84,0, 69,78,0, 78,84,73,76,0, 
79,79,80,0, 79,82,0, 69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 78,84,73,76,0, 72,73,76,69,0, 88,0, 
79,84,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 42,0, 61,0, 61,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 78,68,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 72,73,76,69,0, 61,0, 79,0, 42,0, 61,0, 78,68,0, 
89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 
79,78,71,0, 79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 61,0, 89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 
82,0, 69,78,0, 79,82,0, 42,0, 61,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 89,0, 79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   13,156,156,156,156,156,156,156,156,155,103,155,155,155,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,155,156,127,156,
  156,156,156,152,208,207,198,194,186,195,110,199,122,157,157,157,157,157,
  157,157,121,121,156,165,188,192,190,156,156,158,158,158,158,107,158,159,
  159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,
  159,170,149,169,156,159,156,158,158,158,158,107,158,159,159,159,159,159,
  159,159,159,159,159,159,159,159,159,159,159,159,159,159,159,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
155,103,99,94,23,22,21,20,0,1,160,161,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  149,127,122,121,110,107,103,0,101,102,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  149,127,122,121,110,107,103,98,0,96,97,
155,103,99,94,0,1,
23,22,21,20,0,7,8,9,10,11,162,164,166,167,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  149,127,122,121,110,107,0,
103,0,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  149,127,122,121,110,107,103,0,
98,0,
187,185,184,181,176,175,173,172,168,165,159,158,155,107,103,99,94,0,1,160,
  161,
187,185,184,181,176,175,174,173,172,168,165,159,158,155,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
187,185,184,181,176,175,173,172,168,165,159,158,155,107,103,99,94,13,0,1,
  160,161,
187,185,184,181,176,175,173,172,168,165,159,158,107,0,4,16,19,24,25,26,28,
  29,30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,57,59,171,
187,185,184,181,176,175,174,173,172,168,165,159,158,107,0,12,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,14,40,
  47,60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,
  212,213,214,
187,185,184,181,176,175,173,172,168,165,159,158,107,13,0,12,
208,207,206,199,198,195,194,193,192,191,190,189,188,186,178,170,169,165,159,
  158,157,155,122,121,107,103,99,94,0,1,160,161,
159,158,155,107,103,99,94,0,1,160,161,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
159,158,155,107,103,99,94,0,1,160,161,
187,185,184,183,181,176,175,173,172,168,165,159,158,155,107,103,99,94,0,1,
  160,161,
187,185,184,183,182,181,176,175,174,173,172,168,165,159,158,155,107,103,99,
  94,13,0,1,160,161,
187,185,184,183,182,181,176,175,174,173,172,168,165,159,158,155,107,103,99,
  94,13,0,1,160,161,
159,158,155,107,103,99,94,0,1,160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
159,158,107,0,4,171,
208,205,195,194,165,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,
  1,160,161,
187,185,184,183,181,176,175,173,172,168,165,159,158,107,0,12,
187,185,184,183,182,181,176,175,174,173,172,168,165,159,158,155,107,103,99,
  94,13,0,1,160,161,
178,170,0,39,48,
159,158,107,0,4,47,171,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
159,158,107,0,4,171,
186,165,0,19,58,
186,165,0,19,58,
208,205,195,194,165,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,35,
  40,47,60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,
  211,212,213,214,
165,0,19,
163,0,15,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
187,185,184,181,176,175,173,172,168,165,159,158,107,0,4,16,19,24,25,26,28,
  29,30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,57,59,171,
187,185,184,181,176,175,174,173,172,168,165,159,158,107,0,4,16,17,19,24,25,
  26,28,29,30,31,32,33,34,36,37,38,42,43,44,45,46,47,52,55,57,59,171,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  150,149,145,144,143,142,141,140,139,138,137,136,135,127,122,121,110,107,
  0,128,
157,122,121,0,117,
157,122,121,0,111,117,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,150,
  149,145,144,143,142,141,140,139,138,137,136,135,127,122,121,110,107,0,
  131,132,133,134,146,147,148,153,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,155,127,103,99,94,0,1,160,161,
110,107,0,
107,0,
158,157,122,121,107,0,125,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,157,155,122,121,110,107,103,99,94,0,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,157,155,122,121,110,107,103,99,94,0,117,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,155,103,99,94,0,1,160,161,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,155,103,99,94,0,1,160,161,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,177,169,165,163,155,103,99,94,0,1,160,161,
208,0,84,
127,0,5,214,
210,209,208,207,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,
  99,94,0,1,160,161,
170,0,39,
210,209,208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,
  40,47,60,67,68,69,72,80,81,82,84,86,87,88,89,92,106,109,112,113,114,118,
  171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
206,0,83,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
204,203,202,201,200,199,198,0,73,74,75,76,77,78,79,
197,196,195,194,193,192,191,190,189,188,0,61,62,63,64,65,66,68,69,70,71,
163,0,15,
187,185,184,181,176,175,173,172,168,165,159,158,107,13,0,4,16,19,24,25,26,
  28,29,30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,57,59,171,
187,185,184,183,181,176,175,173,172,168,165,159,158,107,0,4,16,19,24,25,26,
  28,29,30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,56,57,59,171,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
178,170,0,39,48,
170,0,39,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
159,158,107,0,4,171,
165,0,19,
187,185,184,183,182,181,176,175,174,173,172,168,165,159,158,155,107,103,99,
  94,13,0,1,160,161,
180,0,53,
182,0,27,54,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,18,40,
  47,60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,
  212,213,214,
208,207,199,198,195,194,192,190,188,186,170,169,165,159,158,157,156,155,152,
  150,149,145,144,143,142,141,140,139,138,137,136,135,127,122,121,110,107,
  0,129,131,132,133,134,146,147,148,
157,122,121,0,111,117,
157,122,0,
158,157,122,121,107,0,151,
157,122,0,
157,122,0,
158,157,122,121,107,0,151,
152,0,
157,122,121,0,111,117,
195,194,157,122,121,0,108,115,
195,194,157,122,121,0,108,115,
208,207,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,
  47,60,67,68,69,72,80,81,82,84,86,87,90,91,92,106,109,112,113,114,118,
  171,211,212,213,214,
207,155,103,99,94,0,1,160,161,
207,0,85,
207,155,103,99,94,0,1,160,161,
207,0,85,
207,0,85,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,68,
  69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,67,
  68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,
  214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,67,
  68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,
  214,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,67,
  68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,
  214,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,67,
  68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,213,
  214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,60,
  67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
187,185,184,181,176,175,173,172,168,165,159,158,107,0,4,16,19,24,25,26,28,
  29,30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,57,59,171,
187,185,184,183,182,181,176,175,174,173,172,168,165,159,158,155,107,103,99,
  94,13,0,1,160,161,
169,0,41,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
187,185,184,181,176,175,173,172,168,165,159,158,155,107,103,99,94,0,1,160,
  161,
187,185,184,181,176,175,173,172,168,165,159,158,155,107,103,99,94,0,1,160,
  161,
187,185,184,181,176,175,173,172,168,165,159,158,107,0,4,19,24,25,26,28,29,
  30,31,32,33,34,36,37,38,42,44,45,46,47,52,55,57,59,171,
165,0,19,
157,122,121,0,116,
157,122,121,0,116,
186,0,58,
207,0,85,
208,207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,
  188,186,180,179,177,169,165,163,159,158,157,155,152,127,124,122,121,110,
  107,103,99,94,0,1,160,161,
208,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,82,84,86,87,92,
  106,109,112,113,114,118,171,211,212,213,214,
208,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,47,82,84,86,87,92,
  106,109,112,113,114,118,171,211,212,213,214,
204,203,202,201,200,199,198,0,73,74,75,76,77,78,79,
204,203,202,201,200,199,198,0,73,74,75,76,77,78,79,
204,203,202,201,200,199,198,0,73,74,75,76,77,78,79,
204,203,202,201,200,199,198,0,73,74,75,76,77,78,79,
197,196,195,194,0,68,69,70,71,
197,196,195,194,0,68,69,70,71,
197,196,195,194,0,68,69,70,71,
197,196,195,194,0,68,69,70,71,
197,196,195,194,0,68,69,70,71,
197,196,195,194,0,68,69,70,71,
207,206,204,203,202,201,200,199,198,197,196,195,194,193,192,191,190,189,188,
  186,180,179,178,177,170,169,165,163,155,103,99,94,0,1,160,161,
179,177,0,49,51,
169,0,41,
157,122,121,0,117,
157,122,121,0,117,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,
177,0,50,
165,0,19,
208,205,195,194,159,158,157,155,152,127,124,122,121,110,107,103,99,94,0,1,
  160,161,
208,205,195,194,159,158,157,152,127,124,122,121,110,107,0,2,3,4,5,6,40,47,
  60,67,68,69,72,80,81,82,84,86,87,92,106,109,112,113,114,118,171,211,212,
  213,214,

};


static unsigned const char ag_astt[3916] = {
  1,1,1,1,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,
  1,5,3,2,2,2,2,7,1,1,1,1,0,1,1,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,
  5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,2,1,2,2,2,1,
  1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,
  5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,
  3,1,1,7,1,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,
  2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,4,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,3,1,7,2,1,1,1,1,2,2,2,
  1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,1,2,1,1,1,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,3,1,3,3,3,1,3,2,2,1,1,3,1,1,1,1,2,
  2,2,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,
  2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,
  4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,
  4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,
  1,5,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,4,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  5,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,3,3,
  3,3,1,3,2,2,1,1,3,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  7,2,3,3,3,3,1,3,2,2,1,1,3,1,1,1,1,2,2,1,1,1,1,1,3,1,1,1,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,1,5,1,1,1,5,7,1,1,3,1,7,2,1,5,1,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,
  2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,
  8,7,2,1,1,1,8,8,8,7,2,1,1,4,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,
  1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,
  1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,
  1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,2,7,2,2,2,2,2,1,2,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,7,2,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,
  2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,2,2,2,1,1,2,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,7,2,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,1,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,
  1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,7,1,1,1,7,
  1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
285,285,1,2,287,285,
8,7,6,5,4,16,15,14,13,0,12,11,10,9,
96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,
  98,
99,6,
91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,91,
  91,93,
94,8,
286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,9,3,3,293,
286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,10,3,3,292,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,11,3,3,290,
286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,12,3,3,288,
19,20,21,32,18,22,23,24,27,29,100,100,100,13,78,4,4,4,4,40,4,17,18,38,37,4,
  36,35,34,33,17,18,31,31,30,39,28,26,25,17,
41,41,41,41,41,41,41,41,41,41,41,41,41,41,14,41,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,15,85,86,55,56,88,69,31,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
70,70,70,70,70,70,70,70,70,70,70,70,70,70,16,70,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,101,
  101,101,3,101,101,101,3,1,2,17,3,3,297,
286,286,3,286,3,1,2,18,3,3,302,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,19,3,3,313,
286,286,3,286,3,1,2,20,3,3,311,
286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,21,3,3,310,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,
  22,3,3,301,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,
  23,3,3,299,
286,286,3,286,3,1,2,24,3,3,298,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,25,85,86,55,56,88,42,58,68,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
100,100,100,26,40,17,
286,286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,27,3,3,
  294,
71,71,71,71,71,71,71,71,71,71,71,71,71,71,28,71,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,
  29,3,3,291,
74,72,30,73,75,
100,100,100,31,78,76,17,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,32,3,3,307,
100,100,100,33,77,17,
78,29,34,25,79,
78,29,35,24,80,
57,60,61,62,44,100,100,120,45,42,128,126,120,44,100,36,85,86,55,56,88,81,81,
  58,68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
29,37,20,
82,38,19,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,39,85,86,55,56,88,83,58,68,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
19,20,21,32,18,22,23,24,27,29,100,100,100,40,78,13,84,13,84,40,84,17,18,38,
  37,84,36,35,34,33,17,18,31,31,30,39,28,26,25,17,
19,20,21,32,18,22,85,23,24,27,29,100,100,100,41,78,10,86,10,10,10,40,10,17,
  18,38,37,10,36,35,34,33,17,28,18,31,31,30,39,28,26,25,17,
135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
  135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
  42,87,
125,125,125,109,125,
88,88,88,44,107,88,
94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,94,90,89,152,151,150,149,
  148,147,146,145,144,143,142,94,94,94,94,94,45,94,94,94,93,92,91,94,94,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,3,286,3,1,2,46,3,3,340,
95,96,47,
97,102,
131,130,130,130,131,119,129,
118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,
  118,118,118,118,118,118,118,127,118,127,124,110,110,118,118,118,50,
117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,
  117,117,117,117,117,117,117,121,117,121,121,108,108,117,117,117,51,121,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,3,3,1,2,52,3,3,339,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,3,3,1,2,53,3,3,338,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,3,3,1,2,54,3,3,337,
57,78,98,
42,87,133,46,
286,286,286,286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,
  57,3,3,334,
72,74,73,
99,101,57,60,61,62,100,100,120,45,42,128,126,120,44,100,59,85,86,55,56,88,
  103,58,68,67,66,65,67,67,64,63,59,63,63,102,100,56,48,47,51,43,50,49,17,
  54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,60,3,3,331,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,61,3,3,321,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,62,3,3,320,
104,70,105,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,64,85,86,55,56,88,58,66,65,
  69,69,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,65,85,86,55,56,88,58,66,65,
  68,68,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,66,85,86,55,56,88,58,66,65,
  67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
106,108,110,112,114,116,118,53,119,117,115,113,111,109,107,
120,122,61,62,126,128,130,132,134,136,46,137,135,133,131,129,127,125,124,
  123,121,
82,69,138,
19,20,21,32,18,22,23,24,27,29,100,100,100,1,70,78,10,10,10,10,40,10,17,18,
  38,37,10,36,35,34,33,17,18,31,31,30,39,28,26,25,17,
19,20,21,139,32,18,22,23,24,27,29,100,100,100,71,78,10,10,10,10,40,10,17,18,
  38,37,10,36,35,34,33,17,18,31,31,30,39,28,39,26,25,17,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,72,3,3,296,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,73,85,86,55,56,88,140,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,74,3,3,304,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,75,85,86,55,56,88,38,58,68,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
74,72,76,73,141,
72,77,142,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,78,3,3,312,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,79,85,86,55,56,88,43,58,68,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
100,100,100,80,41,17,
29,81,23,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,
  82,3,3,289,
143,83,36,
144,12,145,37,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,85,3,3,300,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,86,85,86,55,56,88,146,32,
  58,68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,136,
  90,89,152,151,150,149,148,147,146,145,144,143,142,134,136,136,136,136,
  87,136,136,136,136,93,92,91,136,
88,88,88,115,116,88,
156,156,89,
162,161,161,161,162,90,159,
158,158,154,
157,157,153,
162,161,161,161,162,141,160,
163,94,
88,88,88,106,105,88,
147,148,148,148,148,96,104,148,
147,148,148,148,148,97,103,148,
57,81,60,61,62,100,100,120,45,42,128,126,120,44,100,98,85,86,55,56,88,83,58,
  68,67,66,65,67,67,64,63,59,63,63,150,149,56,48,47,51,43,50,49,17,54,53,
  52,46,
286,3,3,1,2,99,3,3,336,
151,100,152,
286,3,3,1,2,101,3,3,335,
151,102,153,
151,103,72,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,104,3,3,332,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,105,85,86,55,56,88,58,66,
  65,71,71,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,106,3,3,330,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,107,85,86,55,56,88,58,66,
  65,65,65,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,108,3,3,329,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,109,85,86,55,56,88,58,66,
  65,64,64,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,110,3,3,328,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,111,85,86,55,56,88,58,66,
  65,63,63,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,112,3,3,327,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,113,85,86,55,56,88,58,66,
  65,62,62,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,114,3,3,326,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,115,85,86,55,56,88,58,66,
  65,61,61,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,116,3,3,325,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,117,85,86,55,56,88,58,66,
  65,60,60,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,118,3,3,324,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,119,85,86,55,56,88,58,66,
  65,59,59,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,120,3,3,323,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,121,85,86,55,56,88,58,154,
  66,65,154,154,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,122,3,3,322,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,123,85,86,55,56,88,58,155,
  66,65,155,155,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,124,85,86,55,56,88,58,156,
  66,65,156,156,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,125,85,86,55,56,88,58,157,
  66,65,157,157,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,126,3,3,319,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,127,85,86,55,56,88,58,158,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,128,3,3,318,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,129,85,86,55,56,88,58,159,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,130,3,3,317,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,131,85,86,55,56,88,58,160,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,132,3,3,316,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,133,85,86,55,56,88,58,161,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,134,3,3,315,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,135,85,86,55,56,88,58,162,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,136,3,3,314,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,137,85,86,55,56,88,58,163,
  67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
19,20,21,32,18,22,23,24,27,29,100,100,100,138,78,2,2,2,2,40,2,17,18,38,37,2,
  36,35,34,33,17,18,31,31,30,39,28,26,25,17,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,286,
  139,3,3,309,
164,140,79,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,141,85,86,55,56,88,165,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,142,85,86,55,56,88,166,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,143,3,3,306,
286,286,286,286,286,286,286,286,286,286,286,286,3,286,3,1,2,144,3,3,308,
19,20,21,32,18,22,23,24,27,29,100,100,100,145,78,15,14,15,40,15,17,18,38,37,
  15,36,35,34,33,17,18,31,31,30,39,28,26,25,17,
29,146,3,
122,122,122,147,167,
122,122,122,148,168,
78,82,169,
151,150,80,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,286,286,286,286,3,286,286,286,286,286,286,
  286,3,1,2,151,3,3,333,
57,100,100,120,45,42,128,126,120,44,100,152,85,86,55,56,88,58,77,59,77,77,
  56,48,47,51,43,50,49,17,54,53,52,46,
57,100,100,120,45,42,128,126,120,44,100,153,85,86,55,56,88,58,76,59,76,76,
  56,48,47,51,43,50,49,17,54,53,52,46,
106,108,110,112,114,116,118,57,119,117,115,113,111,109,107,
106,108,110,112,114,116,118,56,119,117,115,113,111,109,107,
106,108,110,112,114,116,118,55,119,117,115,113,111,109,107,
106,108,110,112,114,116,118,54,119,117,115,113,111,109,107,
120,122,61,62,52,125,124,123,121,
120,122,61,62,51,125,124,123,121,
120,122,61,62,50,125,124,123,121,
120,122,61,62,49,125,124,123,121,
120,122,61,62,48,125,124,123,121,
120,122,61,62,47,125,124,123,121,
286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,
  286,286,286,286,286,286,286,286,286,3,3,1,2,164,3,3,295,
170,34,165,172,171,
164,166,173,
123,123,123,114,123,
123,123,123,113,123,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,169,85,86,55,56,88,84,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,170,3,3,305,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,171,85,86,55,56,88,35,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,
174,172,175,
29,173,26,
286,286,286,286,286,286,286,3,286,286,286,286,286,286,286,3,1,2,174,3,3,303,
57,60,61,62,100,100,120,45,42,128,126,120,44,100,175,85,86,55,56,88,33,58,
  68,67,66,65,67,67,64,63,59,63,63,56,48,47,51,43,50,49,17,54,53,52,46,

};


static const unsigned short ag_sbt[] = {
     0,  12,  41,  71,  77,  91, 117, 119, 146, 148, 169, 191, 213, 235,
   275, 291, 337, 353, 385, 396, 418, 429, 451, 476, 501, 512, 557, 563,
   586, 602, 627, 632, 639, 661, 667, 672, 677, 724, 727, 730, 775, 815,
   858, 897, 902, 908, 953, 988, 991, 993,1000,1036,1073,1107,1141,1175,
  1178,1182,1207,1210,1259,1281,1303,1325,1328,1370,1412,1454,1469,1490,
  1493,1534,1576,1598,1643,1665,1710,1715,1718,1740,1785,1791,1794,1819,
  1822,1826,1848,1894,1940,1946,1949,1956,1959,1962,1969,1971,1977,1985,
  1993,2041,2050,2053,2062,2065,2068,2090,2132,2154,2196,2218,2260,2282,
  2324,2346,2388,2410,2452,2474,2516,2538,2580,2602,2645,2667,2710,2753,
  2796,2818,2862,2884,2928,2950,2994,3016,3060,3082,3126,3148,3192,3232,
  3257,3260,3305,3350,3371,3392,3431,3434,3439,3444,3447,3450,3495,3529,
  3563,3578,3593,3608,3623,3632,3641,3650,3659,3668,3677,3713,3718,3721,
  3726,3731,3776,3798,3843,3846,3849,3871,3916
};


static const unsigned short ag_sbe[] = {
     8,  38,  68,  75,  81, 116, 118, 145, 147, 165, 187, 209, 231, 248,
   289, 305, 351, 381, 392, 414, 425, 447, 472, 497, 508, 526, 560, 582,
   600, 623, 629, 635, 657, 664, 669, 674, 692, 725, 728, 744, 788, 829,
   895, 900, 905, 944, 984, 990, 992, 998,1035,1071,1103,1137,1171,1176,
  1179,1203,1208,1226,1277,1299,1321,1326,1342,1384,1426,1461,1479,1491,
  1507,1548,1594,1612,1661,1679,1712,1716,1736,1754,1788,1792,1815,1820,
  1823,1844,1862,1931,1943,1948,1954,1958,1961,1967,1970,1974,1982,1990,
  2008,2046,2051,2058,2063,2066,2086,2104,2150,2168,2214,2232,2278,2296,
  2342,2360,2406,2424,2470,2488,2534,2552,2598,2616,2663,2681,2724,2767,
  2814,2832,2880,2898,2946,2964,3012,3030,3078,3096,3144,3162,3205,3253,
  3258,3274,3319,3367,3388,3405,3432,3437,3442,3445,3448,3491,3506,3540,
  3570,3585,3600,3615,3627,3636,3645,3654,3663,3672,3709,3715,3719,3724,
  3729,3745,3794,3812,3844,3847,3867,3885,3916
};


static const unsigned char ag_fl[] = {
  2,3,4,5,2,1,1,1,1,0,2,1,1,2,4,4,1,1,1,2,2,1,1,3,2,2,6,1,1,1,1,1,1,7,0,
  2,3,1,3,3,2,3,2,3,0,1,1,3,3,3,3,3,3,1,3,3,3,3,1,3,3,3,3,3,3,3,1,2,2,2,
  1,3,3,1,1,1,4,4,1,4,4,0,1,1,3,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,
  3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0, 11, 11, 11, 11,162,164,166,167, 12, 12, 16, 16, 24, 24, 25, 25, 28,
   28, 28, 28, 28, 28, 28, 28, 28, 28, 29, 17, 30, 45, 14, 18, 31, 49, 49,
   26, 27, 32, 33, 36, 36, 37, 37, 35, 35, 40, 40, 40, 40, 40, 40, 40, 60,
   60, 60, 60, 60, 67, 67, 67, 67, 67, 67, 67, 67, 72, 72, 72, 72, 80, 80,
   82, 82, 82, 82, 82, 82, 47, 47, 87, 90, 90, 91, 91, 86, 86, 86, 86,  1,
   96, 96, 97, 97,  1,101,101,102,102,  1,171,171,212,212,212,106,106,106,
  109,109,109,115,115,108,108,111,111,211,211,211,112,112,116,116,113,113,
  114,114,118,118,125,125, 92, 92,214,128,128,129,129,131,131,131,132,132,
  132,132,132,132,132,132,132,132,132,133,133,133,146,147,148,134,134,151,
  151,213,153,153,117, 93, 93, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95,
   95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95,100,100,100,
  100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,
  100,100,100,100,104,104,104,105,105,105,105,105,105,119,119,120,120,120,
  123,123,126,126,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
  130,130,130,130,130,130,130,130,130,154,154,154,154,154,154,154,154,154,
  154,154,154,154,154,154,154,154,154,154,154,154,154,154,160,160,161,161,
    7, 15,  8, 19,  9, 10, 34, 41, 39,  4, 38, 42, 43, 44, 46, 50, 48, 51,
   53, 52, 54, 56, 55, 57, 58, 59, 61, 62, 63, 64, 65, 66, 68, 69, 70, 71,
   73, 74, 75, 76, 77, 78, 79, 81, 83, 85, 84, 88, 89,  2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: ag_rp_5(PCB_POINTER); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER, VW(0, AG_WRAP_7 *)); 
            VWD(0, AG_WRAP_7 *); break;
    case 12: ag_rp_15(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 13: ag_rp_16(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_5 *); break;
    case 14: ag_rp_17(PCB_POINTER); break;
    case 15: ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 16: ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 17: VNO AG_WRAP_7(ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(4, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); VWD(3, AG_WRAP_5 *); VWD(4, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 18: VNO AG_WRAP_5(ag_rp_21(PCB_POINTER)); break;
    case 19: VNO AG_WRAP_5(ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 20: ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 21: ag_rp_24(PCB_POINTER); break;
    case 22: ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); VWD(2, AG_WRAP_5 *); break;
    case 23: ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 24: ag_rp_27(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 25: ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 26: ag_rp_29(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 27: VNO AG_WRAP_5(ag_rp_30(PCB_POINTER)); break;
    case 28: VRO(AG_WRAP_5 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 29: VRO(AG_WRAP_5 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 30: VRO(AG_WRAP_5 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 31: VRO(AG_WRAP_5 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 32: VRO(AG_WRAP_5 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 33: VRO(AG_WRAP_5 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 34: VRO(AG_WRAP_5 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 35: VRO(AG_WRAP_5 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 36: VRO(AG_WRAP_5 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 37: VRO(AG_WRAP_5 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 44: VRO(AG_WRAP_5 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 45: VNO AG_WRAP_5(ag_rp_48(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 46: VNO AG_WRAP_5(ag_rp_49(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 47: VNO AG_WRAP_5(ag_rp_50(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 48: VRO(AG_WRAP_5 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 49: VNO AG_WRAP_5(ag_rp_52(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 51: VNO AG_WRAP_5(ag_rp_54(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 52: VNO AG_WRAP_5(ag_rp_55(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 53: VRO(AG_WRAP_4 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 54: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 55: VRO(AG_WRAP_4 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 56: VNO AG_WRAP_6(ag_rp_59(PCB_POINTER)); break;
    case 57: VRO(AG_WRAP_5 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 58: VRO(AG_WRAP_6 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 59: VNO AG_WRAP_5(ag_rp_62(PCB_POINTER, V(0,(long *)))); break;
    case 60: VNO AG_WRAP_5(ag_rp_63(PCB_POINTER, V(0,(double *)))); break;
    case 61: VNO AG_WRAP_5(ag_rp_64(PCB_POINTER, V(0,(int *)))); break;
    case 62: VNO AG_WRAP_4(ag_rp_65(PCB_POINTER, V(0,(int *)))); break;
    case 63: VRO(AG_WRAP_4 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 64: V(0,(double *)) = ag_rp_67(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 65: V(0,(double *)) = ag_rp_68(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 66: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 67: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(double *))); break;
    case 68: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(1,(double *))); break;
    case 69: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 70: V(0,(double *)) = ag_rp_73(PCB_POINTER, V(0,(long *))); break;
    case 71: V(0,(double *)) = ag_rp_74(PCB_POINTER, V(0,(long *))); break;
    case 72: V(0,(long *)) = ag_rp_75(PCB_POINTER, V(1,(long *))); break;
    case 73: V(0,(long *)) = ag_rp_76(PCB_POINTER, V(1,(long *))); break;
    case 74: V(0,(double *)) = ag_rp_77(PCB_POINTER, V(0,(int *))); break;
    case 75: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 76: V(0,(long *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 77: V(0,(long *)) = ag_rp_80(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 78: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(int *))); break;
    case 79: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 80: V(0,(long *)) = ag_rp_83(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 81: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_85(PCB_POINTER); break;
    case 83: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 84: V(0,(long *)) = ag_rp_87(PCB_POINTER); break;
    case 85: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 86: V(0,(int *)) = ag_rp_89(PCB_POINTER, V(0,(int *))); break;
    case 87: V(0,(int *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 88: VRO(AG_WRAP_5 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 89: VNO AG_WRAP_5(ag_rp_92(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 90: VNO AG_WRAP_4(ag_rp_93(PCB_POINTER)); break;
    case 91: VRO(AG_WRAP_4 *, ag_rp_94(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 92: V(0,(int *)) = ag_rp_95(PCB_POINTER); break;
    case 93: V(0,(int *)) = ag_rp_96(PCB_POINTER); break;
    case 94: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 95: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 96: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 97: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_105(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_106(PCB_POINTER, V(1,(int *))); break;
    case 104: V(0,(int *)) = ag_rp_107(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 105: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 106: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(1,(int *))); break;
    case 107: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 108: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(0,(int *))); break;
    case 109: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(0,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[215] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "REPEAT LOOP",
  "STATEMENT",
  "syntax selection",
  "statement list",
  "eof",
  "loop condition",
  "\"DO\"",
  "statement",
  "UNTIL",
  "until condition",
  "';'",
  "\"SCRIPT\"",
  "\"WHILELOOP\"",
  "\"REPEATLOOP\"",
  "\"STATEMENT\"",
  "open statement",
  "closed statement",
  "if condition",
  "ELSE",
  "simple statement",
  "REPEAT",
  "WHILE",
  "for initialization",
  "assignment statement",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "expression",
  "']'",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "FOR",
  "\"FOR\"",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "\"BY\"",
  "\"IF\"",
  "\"THEN\"",
  "\"ELSE\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "term",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "unary expression",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "factor",
  "\"NOT\"",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "\"DO\"",
  "WHILE LOOP",
  "';'",
  "REPEAT LOOP",
  "STATEMENT",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "\"THEN\"",
  "\"IF\"",
  "\"ELSE\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


