/*
 ast.syn -- Create an Abstract Syntax Tree
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

  Modified to support arrays, structs and functions.

The grammar contained in this file describes the same language as the
 grammar in cll.syn. Reduction procedures have been added to enable it
 to create an abstract syntax tree as it parses. Other than the addition of
 reduction procedures, there are no changes to the syntax.

 .syn files are compiled with the AnaGram parser generator, which creates a
 corresponding parser file and header file.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 This parser analyzes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree

// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348501
#include "ast.h"
#endif

#ifndef AST_H_1027348501
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 590, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AstNode * list, AgStack<AstNode *> &sl) {
/* Line 141, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  AST.root = AST.node(typeScript, list, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 145, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 146, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 164, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 166, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 168, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * init, AstNode * cond, AstNode * inc, AstNode * s) {
/* Line 171, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeForStatement, init, cond, inc, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 175, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 177, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * init, AstNode * cond, AstNode * inc, AstNode * s) {
/* Line 180, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeForStatement,init, cond, inc, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 181, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL, AstNode * s, AstNode * x) {
/* Line 190, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeDoStatement,s,x);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 191, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeExpressionStatement,x);
}

static inline AstNode * ag_rp_14(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_15(PCB_DECL) {
/* Line 194, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.checkLoop(), AST.node(typeBreak);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 195, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return PCB.checkLoop(), AST.node(typeContinue);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 197, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 198, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline AstNode * ag_rp_20(PCB_DECL, AgString &n, AstNode * x) {
/* Line 200, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeArray, AST.variableNode(n), x);
}

static inline AstNode * ag_rp_21(PCB_DECL) {
/* Line 205, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.nameListNode(AgStack<AgString>());
}

static inline AstNode * ag_rp_22(PCB_DECL, AgStack<AgString> &list) {
/* Line 206, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.nameListNode(list);
}

static inline void ag_rp_23(PCB_DECL) {
/* Line 209, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_24(PCB_DECL) {
/* Line 212, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_25(PCB_DECL) {
/* Line 215, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  PCB.loopDepth++;
}

static inline AstNode * ag_rp_26(PCB_DECL, AstNode * x) {
/* Line 218, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_27(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 221, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_28(PCB_DECL, AgString &k) {
/* Line 225, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_29(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 226, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_30(PCB_DECL, AstNode * x) {
/* Line 230, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_31(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 231, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return s.push(x);
}

static AstNode * ag_rp_32(PCB_DECL, AgString &n, AgStack<AgString> &fl, AgStack<AgString> &il) {
/* Line 235, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(fl),
                     AST.nameListNode(il));
 
}

static AstNode * ag_rp_33(PCB_DECL, AgString &n, AgStack<AgString> &fl) {
/* Line 241, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(fl),
                     AST.nameListNode(AgStack<AgString>()));
 
}

static AstNode * ag_rp_34(PCB_DECL, AgStack<AgString> &fl, AgStack<AgString> &nl) {
/* Line 247, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.node(typeNull),
                     AST.nameListNode(fl),
                     AST.nameListNode(nl));
 
}

static AstNode * ag_rp_35(PCB_DECL, AgString &n, AgStack<AgString> &nl) {
/* Line 254, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(AgStack<AgString>()),
                     AST.nameListNode(nl));
 
}

static AstNode * ag_rp_36(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 263, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
      return AST.node(typeFunction,
                      AST.variableNode(n),
                      AST.nameListNode(list),
                      AST.constantNode(text));
    
}

static AgString ag_rp_37(PCB_DECL) {
/* Line 270, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_38(PCB_DECL, AgString &n) {
/* Line 275, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_39(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 276, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return stack.push(n);
}

static inline AstNode * ag_rp_40(PCB_DECL) {
/* Line 279, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_41(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 286, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeCommaExpression,x,y);
}

static inline AstNode * ag_rp_42(PCB_DECL, AstNode * lv, Opcode op, AstNode * x) {
/* Line 292, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.assignmentNode(lv, op, x);
}

static inline AstNode * ag_rp_43(PCB_DECL, AstNode * c, AstNode * x, AstNode * y) {
/* Line 298, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeConditionalExpression,c,x,y);
}

static inline AstNode * ag_rp_44(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 304, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeLogicalOrExpression, x, y);
}

static inline AstNode * ag_rp_45(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 309, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeLogicalAndExpression, x, y);
}

static inline AstNode * ag_rp_46(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 315, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(IOR, x, y);
}

static inline AstNode * ag_rp_47(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(XOR, x, y);
}

static inline AstNode * ag_rp_48(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(AND, x, y);
}

static inline AstNode * ag_rp_49(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 329, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_50(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 334, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_51(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 339, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_52(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_53(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(op, x, y);
}

static inline AstNode * ag_rp_54(PCB_DECL, AstNode * x) {
/* Line 353, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_55(PCB_DECL, Opcode op, AstNode * x) {
/* Line 354, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_56(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 362, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_57(PCB_DECL, AstNode * x) {
/* Line 370, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_58(PCB_DECL, AstNode * x) {
/* Line 372, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_59(PCB_DECL, AstNode * x) {
/* Line 373, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(I_FETCH, x);
}

static inline AstNode * ag_rp_60(PCB_DECL, AstNode * x) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(D_FETCH, x);
}

static inline AstNode * ag_rp_61(PCB_DECL, AstNode * x) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(FETCH_I, x);
}

static inline AstNode * ag_rp_62(PCB_DECL, AstNode * x) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(FETCH_D, x);
}

static inline AstNode * ag_rp_63(PCB_DECL, AstNode * x) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_64(PCB_DECL, AstNode * x) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_65(PCB_DECL, AgString &n) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_66(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 383, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeSubscript, lv, x);
}

static inline AstNode * ag_rp_67(PCB_DECL, AstNode * lv, AgString &n) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.node(typeMember, lv, AST.nameNode(n));
}

static inline AstNode * ag_rp_68(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_69(PCB_DECL) {
/* Line 390, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_70(PCB_DECL, AstNode * x) {
/* Line 394, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_71(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 395, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_72(PCB_DECL, long x) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_73(PCB_DECL, double x) {
/* Line 399, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_74(PCB_DECL, Value &x) {
/* Line 400, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_75(PCB_DECL, int x) {
/* Line 401, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 405, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return STORE;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 406, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return ADDM;
}

static inline Opcode ag_rp_78(PCB_DECL) {
/* Line 407, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return SUBM;
}

static inline Opcode ag_rp_79(PCB_DECL) {
/* Line 408, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return MULM;
}

static inline Opcode ag_rp_80(PCB_DECL) {
/* Line 409, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return DIVM;
}

static inline Opcode ag_rp_81(PCB_DECL) {
/* Line 410, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return MODM;
}

static inline Opcode ag_rp_82(PCB_DECL) {
/* Line 411, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return IORM;
}

static inline Opcode ag_rp_83(PCB_DECL) {
/* Line 412, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return ANDM;
}

static inline Opcode ag_rp_84(PCB_DECL) {
/* Line 413, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return XORM;
}

static inline Opcode ag_rp_85(PCB_DECL) {
/* Line 414, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return LSM;
}

static inline Opcode ag_rp_86(PCB_DECL) {
/* Line 415, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return RSM;
}

static inline Opcode ag_rp_87(PCB_DECL) {
/* Line 416, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return POWM;
}

static inline Opcode ag_rp_88(PCB_DECL) {
/* Line 419, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_89(PCB_DECL) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return NE;
}

static inline Opcode ag_rp_90(PCB_DECL) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return LT;
}

static inline Opcode ag_rp_91(PCB_DECL) {
/* Line 424, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return LE;
}

static inline Opcode ag_rp_92(PCB_DECL) {
/* Line 425, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return GT;
}

static inline Opcode ag_rp_93(PCB_DECL) {
/* Line 426, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return GE;
}

static inline Opcode ag_rp_94(PCB_DECL) {
/* Line 429, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return LS;
}

static inline Opcode ag_rp_95(PCB_DECL) {
/* Line 430, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return RS;
}

static inline Opcode ag_rp_96(PCB_DECL) {
/* Line 433, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_97(PCB_DECL) {
/* Line 434, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_98(PCB_DECL) {
/* Line 437, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_99(PCB_DECL) {
/* Line 438, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return DIV;
}

static inline Opcode ag_rp_100(PCB_DECL) {
/* Line 439, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_101(PCB_DECL) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_102(PCB_DECL) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return NOT;
}

static inline Opcode ag_rp_103(PCB_DECL) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return COM;
}

static inline AgString ag_rp_104(PCB_DECL, int c) {
/* Line 462, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_105(PCB_DECL, AgString &ns, int c) {
/* Line 463, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return ns.append(c);
}

static inline double ag_rp_106(PCB_DECL, double x, long e) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_107(PCB_DECL, double x, long e) {
/* Line 470, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_108(PCB_DECL, double i, double f) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return i+f;
}

static inline double ag_rp_109(PCB_DECL, double i) {
/* Line 474, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return i;
}

static inline double ag_rp_110(PCB_DECL, double f) {
/* Line 475, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return f;
}

static inline double ag_rp_111(PCB_DECL, long x) {
/* Line 478, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline double ag_rp_112(PCB_DECL, long x) {
/* Line 479, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x;
}

static inline double ag_rp_113(PCB_DECL, long x) {
/* Line 480, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_114(PCB_DECL, long x) {
/* Line 483, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return (long)x;
}

static inline long ag_rp_115(PCB_DECL, long x) {
/* Line 484, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return -(long)x;
}

static inline double ag_rp_116(PCB_DECL, int d) {
/* Line 487, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_117(PCB_DECL, int d, double f) {
/* Line 488, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_118(PCB_DECL, int d) {
/* Line 496, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return d-'0';
}

static inline long ag_rp_119(PCB_DECL, long x, int d) {
/* Line 497, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_120(PCB_DECL, int d) {
/* Line 500, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return d-'0';
}

static inline long ag_rp_121(PCB_DECL, long x, int d) {
/* Line 501, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_122(PCB_DECL, long x, int d) {
/* Line 504, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_123(PCB_DECL, long x, int d) {
/* Line 505, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_124(PCB_DECL) {
/* Line 508, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 0;
}

static inline long ag_rp_125(PCB_DECL, long n, int d) {
/* Line 509, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_126(PCB_DECL) {
/* Line 512, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 0;
}

static inline long ag_rp_127(PCB_DECL, long n, int d) {
/* Line 513, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_128(PCB_DECL, int x) {
/* Line 516, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return x - '0';
}

static inline int ag_rp_129(PCB_DECL, int d) {
/* Line 517, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_130(PCB_DECL, Value &s, Value &e) {
/* Line 523, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return s+=e;
}

static inline Value ag_rp_131(PCB_DECL, AgString &b) {
/* Line 526, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_132(PCB_DECL) {
/* Line 530, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return AgString();
}

static inline AgString ag_rp_133(PCB_DECL, AgString &s, int c) {
/* Line 531, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return s.append(c);
}

static inline int ag_rp_134(PCB_DECL) {
/* Line 543, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\'';
}

static inline int ag_rp_135(PCB_DECL) {
/* Line 544, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '"';
}

static inline int ag_rp_136(PCB_DECL) {
/* Line 545, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\?';
}

static inline int ag_rp_137(PCB_DECL) {
/* Line 546, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\\';
}

static inline int ag_rp_138(PCB_DECL) {
/* Line 547, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\a';
}

static inline int ag_rp_139(PCB_DECL) {
/* Line 548, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\b';
}

static inline int ag_rp_140(PCB_DECL) {
/* Line 549, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\f';
}

static inline int ag_rp_141(PCB_DECL) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\n';
}

static inline int ag_rp_142(PCB_DECL) {
/* Line 551, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\r';
}

static inline int ag_rp_143(PCB_DECL) {
/* Line 552, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\t';
}

static inline int ag_rp_144(PCB_DECL) {
/* Line 553, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return '\v';
}

static inline int ag_rp_145(PCB_DECL, int d) {
/* Line 559, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return d-'0';
}

static inline int ag_rp_146(PCB_DECL, int n, int d) {
/* Line 562, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_147(PCB_DECL, int n, int d) {
/* Line 565, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_148(PCB_DECL, int d) {
/* Line 568, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return d;
}

static inline int ag_rp_149(PCB_DECL, int n, int d) {
/* Line 569, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_150(PCB_DECL, int d) {
/* Line 572, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return d-'0';
}

static inline int ag_rp_151(PCB_DECL, int d) {
/* Line 573, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_152(PCB_DECL, int c) {
/* Line 582, W:/agdev/agex/packages/xidek/cll/combine/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 7, 6, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 4, 6, 4, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0,
  0, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 7, 0, 0,
  0, 0, 0, 7, 0, 0, 0, 0, 0, 7, 0, 0, 0, 4, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,  0,
   15, 16, 17, 18, 19, 20,  0,  0, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
   31, 32, 33, 34, 35, 36, 37, 38, 39, 40,  0,  0, 41,  0, 42,  0, 43,  0,
   44,  0, 45,  0, 46,  0, 47,  0, 48,  0, 49,  0, 50,  0, 51,  0, 52,  0,
   53,  0, 54, 55,  0, 56, 57,  0, 58, 59, 60, 61, 62,  0, 63, 64, 65, 66,
   67, 68, 69,  0, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83,
   84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,
  102,103,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,104,105,  0,106,107,
  108,109,110,111,112,113,  0,  0,114,115,116,117,  0,  0,  0,118,119,120,
  121,122,123,124,125,  0,  0,126,127,128,129,  0,130,131,132,133,  0,  0,
    0,  0,  0,134,135,136,137,138,139,140,141,142,143,144,  0,  0,  0,145,
  146,147,148,149,150,151,152
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,188,  0,192,  0,193,  0,194,  0,195,  0,199,  0,200,  0,201,
  0,202,  0,203,  0,206,  0,208,  0,209,  0,210,  0,222,  0,223,
0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 88,120,255,111,117,255,111,117,255, 43, 45, 47, 48, 97,
   98, 99,100,102,105,108,112,114,115,119,255, 42,255, 38, 61,255, 61,255,
   42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255, 61,255,
   60, 61,255, 61,255, 61, 62,255,117,255,111,117,255,111,117,255, 99,110,
  255,111,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 48, 60, 61, 62, 94,
   97, 98, 99,100,101,102,105,108,112,114,115,119,124,255, 88,120,255,111,
  117,255,111,117,255, 43, 45, 48, 97, 98, 99,100,102,105,108,112,114,115,
  119,255, 42, 47,255, 47,255, 88,120,255,111,117,255,111,117,255, 43, 45,
   48, 97, 98, 99,100,102,105,112,114,115,119,255, 38, 61,255, 61,255, 42,
   61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 61,255, 60, 61,255, 61,
  255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,
  124,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,255, 42, 47,255, 88,120,
  255,111,117,255,111,117,255, 43, 45, 47, 48, 97, 98, 99,100,101,102,105,
  112,114,115,119,255, 34, 39, 63, 92, 97, 98,102,110,114,116,118,120,255,
   92,255, 42, 47,255, 60, 61,255, 61, 62,255, 33, 38, 42, 47, 60, 61, 62,
  124,255, 60, 61,255, 61, 62,255, 33, 38, 42, 60, 61, 62,124,255, 42, 47,
  255, 88,120,255, 43, 45, 47, 48,100,108,255, 88,120,255, 43, 45, 48,100,
  108,255, 88,120,255, 43, 45, 48,255, 60, 61,255, 61, 62,255, 33, 38, 60,
   61, 62,124,255, 33, 38, 60, 61, 62,124,255, 33, 38, 61,124,255, 38,124,
  255,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255, 61,
  255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45,
   47, 60, 61, 62, 94,124,255, 42, 47,255, 88,120,255,111,117,255,111,117,
  255, 43, 45, 47, 48, 97, 98, 99,100,102,105,112,114,115,119,255, 60, 61,
  255, 61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,124,255,119,255, 88,120,
  255,111,117,255,111,117,255, 43, 45, 48, 97, 98, 99,100,101,102,105,112,
  114,115,119,255, 42, 47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,
  255,111,117,255, 33, 38, 42, 43, 45, 47, 48, 60, 61, 62, 97, 98, 99,100,
  102,105,112,114,115,119,124,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,2,7,7,7,2,2,7,7,7,7,7,7,4,3,4,0,0,4,0,
  4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,7,4,6,7,4,7,7,4,
  7,7,4,2,4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,7,2,7,2,7,2,7,7,7,7,2,4,0,
  0,4,5,7,4,7,7,4,3,3,2,7,7,7,2,2,7,7,7,7,7,7,4,0,0,4,2,4,0,0,4,5,7,4,7,
  7,4,3,3,2,7,7,7,2,2,7,7,7,7,7,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,
  4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,0,0,4,0,0,4,3,3,2,2,
  4,0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,2,7,7,7,2,7,2,7,7,7,7,7,4,0,0,0,0,0,0,
  0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,4,0,0,4,3,3,
  3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,0,4,3,3,2,4,
  0,0,4,0,0,4,3,3,2,3,2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,3,4,0,0,4,0,4,
  1,0,4,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,4,
  0,0,4,0,0,4,5,7,4,7,7,4,3,3,2,2,7,7,7,2,2,7,7,7,7,7,4,0,0,4,0,0,4,3,3,
  3,3,3,2,3,2,3,4,7,4,0,0,4,5,7,4,7,7,4,3,3,2,7,7,7,2,7,2,7,7,7,7,7,4,0,
  0,4,0,0,4,0,0,4,0,0,4,5,7,4,7,7,4,3,3,3,3,3,2,2,2,3,2,7,7,7,2,2,7,7,7,
  7,7,3,4
};

static const unsigned char ag_key_parm[] = {
    0,119,124,  0,149,148,  0, 14, 20,  0, 16, 26,  0,220,221,  0,  0, 10,
    4,  6,  0,  0, 18, 12, 22,  8, 24,  2,  0,123,  0,214,235,  0,239,  0,
  219,231,  0,220,229,  0,221,230,  0,119,124,232,  0,149,148,  0,237,  0,
  246,243,  0,238,  0,245,247,  0, 30,  0, 14, 20,  0, 16, 26,  0, 12, 28,
    0,  0,  0,234,213,  0,241,233,  0,  0,  0,  0,  0,  0,  0,240,  0,236,
   10,  4,  6,  0,  0,  0, 18,  0, 22,  8, 24,  2,  0,  0,149,148,  0, 14,
   20,  0, 16, 26,  0,220,221,  0, 10,  4,  6,  0,  0, 18, 12, 22,  8, 24,
    2,  0,119,124,  0,  0,  0,149,148,  0, 14, 20,  0, 16, 26,  0,220,221,
    0, 10,  4,  6,  0,  0, 18, 22,  8, 24,  2,  0,214,235,  0,239,  0,219,
  231,  0,220,229,  0,221,230,  0,119,124,232,  0,237,  0,246,243,  0,238,
    0,245,247,  0,234,213,  0,241,233,  0,  0,  0,  0,  0,  0,240,  0,236,
    0,  0,119,124,  0,149,148,  0,220,221,  0,  0,  0,119,124,  0,149,148,
    0, 14, 20,  0, 16, 26,  0,220,221,  0,  0, 10,  4,  6,  0,  0,  0, 18,
   22,  8, 24,  2,  0,162,161,163,164,165,166,167,168,169,170,171,176,  0,
    0,  0,119,124,  0,246,243,  0,245,247,  0,241,214,219,  0,  0,240,  0,
  213,  0,246,243,  0,245,247,  0,241,214,219,  0,240,  0,213,  0,119,124,
    0,149,148,  0,220,221,  0,  0, 30, 28,  0,149,148,  0,220,221,  0, 30,
   28,  0,149,148,  0,220,221,  0,  0,246,243,  0,245,247,  0,241,214,  0,
  240,  0,213,  0,241,214,243,240,245,213,  0,241,214,240,213,  0,214,213,
    0,213,  0,214,235,  0,239,  0,219,231,  0,220,229,  0,221,230,  0,237,
    0,246,243,  0,238,  0,245,247,  0,234,213,  0,241,233,  0,  0,  0,  0,
  232,  0,240,  0,236,  0,  0,119,124,  0,149,148,  0, 14, 20,  0, 16, 26,
    0,220,221,  0,  0, 10,  4,  6,  0,  0, 18, 22,  8, 24,  2,  0,246,243,
    0,245,247,  0,241,214,219,220,221,  0,240,  0,213,  0,  2,  0,149,148,
    0, 14, 20,  0, 16, 26,  0,220,221,  0, 10,  4,  6,  0,  0,  0, 18, 22,
    8, 24,  2,  0,119,124,  0,149,148,  0,246,243,  0,245,247,  0, 14, 20,
    0, 16, 26,  0,241,214,219,220,221,  0,  0,  0,240,  0, 10,  4,  6,  0,
    0, 18, 22,  8, 24,  2,213,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  0,  0,  0,  0, 22,  0, 25, 27,  0,  0,  2,  1,  4,  4,
    9, 14,  7, 10, 34, 36, 41, 46, 52, 58,  0, 63,  0,  0,  0,  0,  0,  0,
   34,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   52,  0,  0,  0,  0,  0, 57,  0, 91,  0, 62, 95,  0,102,104,  0,113,116,
    0, 70,  0,  0,  0,  0, 65, 67, 31, 36, 39, 42, 45, 49, 54, 69, 59, 71,
   73, 78, 83, 64, 98, 67,111, 73,118,123,129,135, 75,  0,  0,  0,  0,  0,
  162,  0,165,167,  0,140,142,104,144,149,154,107,110,174,176,181,186,192,
  198,  0,  0,  0,  0,128,  0,  0,  0,  0,  0,225,  0,228,230,  0,203,205,
  133,207,212,217,136,139,237,239,244,250,256,  0,  0,  0,  0,  0,  0,159,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,174,  0,  0,  0,
    0,  0,179,  0,  0,  0,  0,261,263,156,161,164,167,170,176,265,181,267,
  184,  0,  0,  0,  0,  0,  0,  0,269,271,200,203,  0,  0,  0,  0,  0,  0,
    0,  0,295,  0,302,304,  0,273,275,211,214,277,282,287,217,298,220,311,
  313,318,324,330,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  239,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,335,337,339,254,257,341,260,
  343,  0,  0,  0,  0,  0,  0,  0,345,347,349,272,351,275,353,  0,  0,  0,
    0,  0,  0,  0,355,357,286,289,359,365,  0,  0,  0,  0,369,371,299,373,
  379,  0,  0,  0,  0,383,385,308,  0,  0,  0,  0,  0,  0,  0,387,389,315,
  391,318,393,  0,395,397,399,401,403,405,  0,407,409,411,413,  0,415,417,
    0,419,  0,  0,  0,  0,  0,  0,348,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,359,  0,  0,  0,  0,  0,364,  0,  0,  0,  0,421,423,345,350,353,356,
  425,361,427,366,429,369,  0,  0,  0,  0,  0,  0,  0,  0,453,  0,456,458,
    0,431,433,385,388,435,440,445,391,394,465,467,472,478,484,  0,  0,  0,
    0,  0,  0,  0,489,491,493,495,497,412,499,415,501,  0,503,  0,  0,  0,
    0,  0,530,  0,537,539,  0,508,510,430,512,517,522,433,533,436,546,548,
  553,559,565,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,600,
    0,603,605,  0,570,572,574,576,578,454,457,460,580,463,582,587,592,466,
  469,612,614,619,625,631,636,  0
};

static const unsigned short ag_key_index[] = {
   13,  0, 29, 78,113,  0,  0, 29, 29,131,  0,142,187,  0,142,206,  0,223,
  252,  0,  0,252,263,  0,263,263,263,263,263,263,263,263,263,278,292,206,
  206,206,  0,  0,302,206,278,311,311,321,321,321,328,335,340,340,340,343,
  372,  0,131,131,206,131, 13,372,397,  0,  0,131,311,  0,206,131,131,142,
  131,131,131,  0,  0,  0,  0,311,  0,  0,  0,142,  0,  0,  0,142,252,263,
    0,  0,252,252,252,  0,263,  0,  0,278,278,131,  0,131,  0,  0,206,311,
  418,206,206,206,311,311,206,206,311,206,206,206,206,311,206,206,311,206,
  311,206,311,206,311,206,311,206,311,311,206,311,131,  0,206,311,206,206,
  206,206,206,206,206,206,206,206,206,206,311,  0,  0,  0,142,  0,311,  0,
    0,  0,311,428,311,311,311,439,  0,  0,472,311,311,321,321,321,328,335,
  340,340,  0,  0,  0,  0,  0,  0,  0,  0,223,311,  0,  0,  0,  0,397,142,
  263,263,311,206,311,187,  0,  0,  0,  0,311,311,142,  0,  0,  0,  0,  0,
    0,113,  0,  0,311,142,  0,  0,142,142
};

static const unsigned char ag_key_ends[] = {
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
47,0, 61,0, 61,0, 61,0, 61,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 98,108,101,0, 109,112,0, 
108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 97,108,0, 
103,0, 114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 111,99,97,108,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
61,0, 61,0, 61,0, 61,0, 43,0, 45,0, 43,0, 45,0, 
114,114,97,121,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 108,115,101,0, 114,0, 110,99,116,105,111,110,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 111,117,98,108,101,0, 111,110,103,0, 
43,0, 45,0, 61,0, 38,0, 61,0, 124,0, 61,0, 38,0, 61,0, 61,0, 
61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 38,0, 124,0, 124,0, 
61,0, 61,0, 61,0, 61,0, 61,0, 43,0, 45,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 104,105,108,101,0, 
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 114,0, 
110,99,116,105,111,110,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
124,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   10,182,182,182,182,182,182,182,182,181,128,181,181,181,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,181,252,153,182,
  182,251,217,178,190,189,249,218,207,248,224,250,146,183,183,183,183,183,
  183,183,145,145,211,191,242,228,244,212,182,184,184,184,184,132,184,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,
  185,197,175,196,216,185,182,184,184,184,184,132,184,185,185,185,185,185,
  185,185,185,185,185,185,185,185,185,185,185,185,185,185,185,205,215,204,
  253,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,182,
  182,182,182,182
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,200,199,195,194,
  193,192,191,190,185,184,183,181,178,153,149,148,146,145,132,128,124,119,
  10,0,1,186,187,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,175,153,146,145,132,128,0,
  126,127,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,175,153,146,145,132,128,123,
  0,121,122,
181,128,124,119,0,1,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,200,199,195,194,
  193,192,191,190,185,184,183,178,153,149,148,146,145,132,10,0,7,8,37,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,175,153,146,145,132,0,
128,0,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,175,153,146,145,132,128,0,
123,0,
185,184,181,132,128,124,119,0,1,186,187,
185,184,132,0,4,38,198,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,10,0,9,
251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,230,229,228,224,221,220,219,218,217,216,215,214,213,212,211,207,
  205,204,197,196,191,190,189,185,184,183,181,146,145,132,128,124,119,0,1,
  186,187,
207,191,0,22,45,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,10,0,2,3,4,5,6,11,
  12,13,14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,
  44,46,47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
185,184,132,0,4,198,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,199,195,194,
  193,192,191,190,189,188,185,184,183,181,178,153,149,148,146,145,132,128,
  124,119,10,0,1,186,187,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,176,175,171,170,169,168,167,
  166,165,164,163,162,161,153,146,145,132,0,154,
183,146,145,0,141,
183,146,145,0,135,141,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,176,175,171,170,169,168,167,166,
  165,164,163,162,161,153,146,145,132,0,157,158,159,160,172,173,174,179,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,181,153,128,124,119,0,1,186,187,
224,132,0,
132,0,
184,183,146,145,132,0,151,
251,250,249,248,247,246,245,244,243,242,241,240,224,219,218,217,216,215,214,
  213,212,211,207,196,191,189,183,181,146,145,132,128,124,119,0,
251,250,249,248,247,246,245,244,243,242,241,240,224,219,218,217,216,215,214,
  213,212,211,207,196,191,189,183,181,146,145,132,128,124,119,0,141,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,181,128,124,119,0,1,186,187,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,181,128,124,119,0,1,186,187,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,181,128,124,119,0,1,186,187,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,185,184,181,132,128,124,119,0,1,186,187,
251,250,249,248,247,246,245,244,243,242,241,240,219,218,217,216,215,214,213,
  212,211,207,196,191,189,185,184,181,132,128,124,119,0,1,186,187,
153,0,5,254,
253,252,248,224,223,222,221,220,218,191,190,189,185,184,183,181,178,153,149,
  148,146,145,132,128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
185,184,132,0,4,52,198,
185,184,132,0,4,52,198,
253,252,248,224,223,222,221,220,218,190,185,184,183,178,153,149,148,146,145,
  132,0,2,3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,
  79,80,82,83,84,85,86,87,91,112,116,117,131,134,136,137,138,142,150,198,
  225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
219,0,81,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,136,137,
  138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,136,137,
  138,142,150,198,225,226,227,254,
251,250,249,0,76,113,114,115,
248,218,0,74,78,112,
247,246,0,72,110,111,
245,244,243,242,0,70,106,107,108,109,
241,240,0,68,104,105,
217,0,66,
216,0,64,
215,214,0,60,62,
213,0,58,
190,0,17,
212,0,55,
185,184,181,132,128,124,119,0,1,186,187,
205,185,184,181,132,128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
185,184,181,132,128,124,119,0,1,186,187,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,200,199,195,
  194,193,192,191,190,185,184,183,181,178,153,149,148,146,145,132,128,124,
  119,0,1,186,187,
239,238,237,236,235,234,233,232,231,230,229,228,224,221,220,197,0,33,53,83,
  84,88,92,93,94,95,96,97,98,99,100,101,102,103,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,181,178,153,149,148,146,145,132,128,124,119,0,1,
  186,187,
185,184,132,0,4,198,
205,185,184,132,0,4,42,198,
185,184,181,132,128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
185,184,132,0,4,198,
253,252,248,224,221,220,218,191,190,185,184,183,181,178,153,149,148,146,145,
  132,128,124,119,0,1,186,187,
191,181,128,124,119,0,1,186,187,
191,181,128,124,119,0,1,186,187,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,199,195,194,
  193,192,191,190,185,184,183,178,153,149,148,146,145,132,0,9,
190,181,128,124,119,0,1,186,187,
190,181,128,124,119,0,1,186,187,
190,181,128,124,119,0,1,186,187,
191,0,22,
185,184,132,0,4,198,
207,191,0,22,45,
207,191,0,22,45,
253,252,248,224,221,220,218,191,190,185,184,183,178,153,149,148,146,145,132,
  0,2,3,4,5,6,17,18,21,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,
  80,82,83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,
  227,254,
191,0,22,
191,0,22,
207,191,0,22,45,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,11,12,
  13,14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,
  46,47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,
  85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
190,0,17,
190,0,17,
190,0,17,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,11,12,
  13,14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,
  46,47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,
  85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,251,250,249,248,244,242,228,224,218,217,216,215,212,211,207,205,204,
  197,196,191,190,189,185,184,183,182,181,178,176,175,171,170,169,168,167,
  166,165,164,163,162,161,153,146,145,132,0,155,157,158,159,160,172,173,
  174,
183,146,145,0,135,141,
183,146,0,
184,183,146,145,132,0,177,
183,146,0,
183,146,0,
184,183,146,145,132,0,177,
178,0,
183,146,145,0,135,141,
248,218,183,146,145,0,133,139,
248,218,183,146,145,0,133,139,
224,197,0,33,88,
224,197,0,33,88,
189,181,128,124,119,0,1,186,187,
189,0,19,
189,181,128,124,119,0,1,186,187,
189,0,19,
207,189,0,19,45,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,136,137,
  138,142,150,198,225,226,227,254,
224,221,220,197,0,33,83,84,88,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,136,137,
  138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,73,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,136,
  137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,71,73,75,77,78,79,80,82,83,84,85,91,112,116,117,131,134,
  136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,69,71,73,75,77,78,79,80,82,83,84,85,91,112,116,117,131,
  134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,67,69,71,73,75,77,78,79,80,82,83,84,85,91,112,116,117,131,
  134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,65,67,69,71,73,75,77,78,79,80,82,83,84,85,91,112,116,117,
  131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,91,112,116,
  117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,91,112,
  116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,91,
  112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,
  91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,189,185,184,183,178,153,149,148,146,145,132,
  0,2,3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,89,90,91,112,116,117,131,134,136,137,138,142,150,198,225,226,
  227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
185,184,181,132,128,124,119,0,1,186,187,
185,184,132,0,4,198,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
190,0,17,
185,184,132,0,4,38,198,
205,185,184,132,0,4,38,42,198,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,199,195,194,
  193,192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,11,
  12,13,14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,
  43,44,46,47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
197,0,33,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
185,184,132,0,4,198,
207,0,45,
191,0,22,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
192,0,25,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,191,190,185,184,183,178,153,149,148,146,145,132,
  0,2,3,4,5,6,17,18,21,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,
  80,82,83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,
  227,254,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
188,0,15,
183,146,145,0,140,
183,146,145,0,140,
253,252,251,250,249,248,247,246,245,244,243,242,241,240,224,221,220,219,218,
  217,216,215,214,213,212,211,210,209,208,207,206,205,203,202,201,199,196,
  195,194,193,192,191,190,189,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
224,221,220,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,17,52,
  80,82,83,84,85,91,131,134,136,137,138,142,150,198,225,226,227,254,
224,221,220,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,17,52,
  80,82,83,84,85,91,131,134,136,137,138,142,150,198,225,226,227,254,
251,250,249,0,76,113,114,115,
248,218,0,74,78,112,
247,246,0,72,110,111,
245,244,243,242,0,70,106,107,108,109,
241,240,0,68,104,105,
217,0,66,
216,0,64,
207,0,45,
189,0,19,
211,207,0,45,56,
207,196,0,34,45,
185,184,132,0,4,38,198,
207,204,0,43,45,
207,0,45,
185,184,132,0,4,38,198,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,199,195,194,
  193,192,191,190,188,185,184,183,181,178,153,149,148,146,145,132,128,124,
  119,10,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
190,0,17,
207,189,0,19,45,
191,0,22,
207,189,0,19,45,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,181,178,153,149,148,146,145,132,128,124,119,0,1,
  186,187,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,12,13,
  14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,46,
  47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,
  91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
183,146,145,0,141,
183,146,145,0,141,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,
  84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,190,185,184,183,181,178,153,149,148,146,145,132,
  128,124,119,0,1,186,187,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,
  85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,
  232,231,230,229,228,224,221,220,219,218,217,216,215,214,213,212,211,207,
  197,196,191,189,181,128,124,119,0,1,186,187,
207,189,0,19,45,
185,184,132,0,4,38,198,
207,204,0,43,45,
207,196,0,34,45,
253,252,248,224,221,220,218,190,185,184,183,178,153,149,148,146,145,132,0,2,
  3,4,5,6,17,18,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,
  83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,191,190,185,184,183,178,153,149,148,146,145,132,
  0,2,3,4,5,6,17,18,21,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,
  80,82,83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,
  227,254,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,12,13,
  14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,46,
  47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,
  91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
205,0,42,
207,0,45,
185,184,132,0,4,38,198,
191,0,22,
207,189,0,19,45,
191,0,22,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,200,199,195,
  194,193,192,191,190,185,184,183,178,153,149,148,146,145,132,0,8,37,50,
207,0,45,
191,0,22,
253,252,248,224,221,220,218,190,189,185,184,183,178,153,149,148,146,145,132,
  0,2,3,4,5,6,17,18,21,47,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,
  80,82,83,84,85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,
  227,254,
253,252,248,224,221,220,218,210,209,208,206,205,204,203,202,201,199,195,194,
  193,192,191,190,185,184,183,178,153,149,148,146,145,132,0,9,
204,0,43,
189,0,19,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,11,12,
  13,14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,
  46,47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,
  85,91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,
253,252,248,224,221,220,218,210,209,208,206,205,203,202,201,199,195,194,193,
  192,191,190,185,184,183,178,153,149,148,146,145,132,0,2,3,4,5,6,12,13,
  14,16,17,18,20,22,23,24,25,26,27,28,29,30,31,32,35,36,39,40,41,42,44,46,
  47,48,49,51,52,54,57,59,61,63,65,67,69,71,73,75,77,78,79,80,82,83,84,85,
  91,112,116,117,131,134,136,137,138,142,150,198,225,226,227,254,

};


static unsigned const char ag_astt[5677] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,1,1,1,
  8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,0,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,
  1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,1,7,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,2,7,2,2,1,1,2,2,2,2,1,1,1,1,
  1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,
  10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,
  5,2,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,5,5,5,
  7,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,5,5,5,7,
  2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,4,2,1,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,2,2,2,7,2,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,
  1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,2,2,2,1,1,5,1,2,2,1,1,5,1,2,2,1,
  1,1,1,5,1,2,2,2,2,1,1,5,1,2,2,1,5,1,1,5,1,1,1,5,1,1,1,5,1,1,4,1,1,5,1,5,5,
  1,5,1,1,1,7,1,1,3,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,4,1,1,2,2,1,2,2,2,2,2,2,2,2,2,2,2,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,1,1,1,2,2,2,7,
  1,1,1,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,
  1,7,1,1,3,1,7,3,2,2,2,7,1,1,1,1,7,2,1,1,1,7,2,1,1,1,1,1,1,1,1,4,1,2,2,2,1,
  1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,1,7,2,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,2,
  2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,7,1,1,7,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,1,1,1,1,
  1,1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,
  2,2,2,2,2,2,2,2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,
  2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,
  1,1,4,1,1,1,1,4,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,
  2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,
  1,1,1,1,1,1,1,1,1,4,1,2,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,
  2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,
  1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,
  1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,1,5,
  1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,
  2,2,2,7,2,1,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,
  1,2,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,
  1,1,1,1,2,2,2,7,2,1,1,5,1,1,7,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,
  1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,
  1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,2,2,7,1,2,2,2,7,1,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,
  2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,4,1,2,
  2,1,1,4,1,2,2,1,1,1,1,4,1,2,2,2,2,1,1,4,1,2,2,1,4,1,1,4,1,1,5,1,1,7,2,1,1,
  7,1,1,1,1,7,2,1,2,2,2,7,2,1,1,1,1,7,1,1,1,4,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,7,2,1,1,7,1,
  1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,2,7,2,2,1,1,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,1,7,1,1,2,2,2,
  7,2,1,1,1,1,7,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,1,
  1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,4,1,2,2,2,4,2,
  1,1,1,7,2,1,1,7,1,1,1,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,4,1,1,7,2,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,
  2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,1,1,1,1,2,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,7,1,1,7,2,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,2,4,2,2,1,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,1,
  2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,2,7,2,2,1,1,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,4,3,1,2,4,
  0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,
  6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,
  7,8,
366,366,1,2,368,366,
26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,9,26,26,26,26,26,26,26,26,26,
  26,26,26,26,26,26,26,26,26,4,0,11,10,
135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,
  135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,137,
138,6,
130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
  130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,132,
133,8,
367,367,3,367,3,1,2,9,3,3,381,
139,139,139,10,43,13,12,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,11,14,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,140,140,140,3,140,140,140,3,1,2,12,3,3,379,
15,17,13,27,16,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,1,14,96,97,54,33,99,3,3,3,87,86,40,82,85,
  16,13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,
  55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,
  24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,15,3,3,388,
139,139,139,16,44,12,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,1,2,
  367,17,3,3,372,
176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,
  176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,176,
  176,176,176,176,176,176,176,176,176,176,18,88,
164,164,164,148,164,
89,89,89,20,146,89,
95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,
  95,95,95,95,91,90,193,192,191,190,189,188,187,186,185,184,183,95,95,95,
  95,21,95,95,95,94,93,92,95,95,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,367,3,1,2,22,3,3,435,
96,97,23,
98,141,
172,171,171,171,172,158,170,
157,157,157,157,157,157,157,157,157,157,157,157,149,157,157,157,157,157,157,
  157,157,157,157,157,157,157,166,157,166,163,149,157,157,157,26,
156,156,156,156,156,156,156,156,156,156,156,156,147,156,156,156,156,156,156,
  156,156,156,156,156,156,156,160,156,160,160,147,156,156,156,27,160,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,28,3,3,408,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,29,3,3,407,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,3,1,2,30,3,3,406,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,3,1,2,31,3,3,402,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,3,1,2,32,3,3,401,
18,98,174,22,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,
  367,367,367,367,3,1,2,34,3,3,371,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,35,3,3,434,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,36,3,3,433,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,37,3,3,429,
139,139,139,38,88,99,12,
139,139,139,39,88,100,12,
35,36,37,20,101,103,31,32,41,34,139,139,159,21,18,169,169,165,159,139,40,96,
  97,54,33,99,40,105,105,105,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,
  42,42,39,38,42,104,102,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,
  22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,41,3,3,399,
106,76,107,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,43,96,97,54,
  33,99,40,108,75,75,44,43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,
  25,169,12,30,29,28,22,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,44,96,97,54,
  33,99,40,108,74,74,44,43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,
  25,169,12,30,29,28,22,
109,110,111,69,112,122,123,124,
37,41,67,113,120,121,
114,115,65,116,118,119,
117,118,119,120,63,121,114,115,116,117,
122,123,61,124,112,113,
125,59,126,
127,57,128,
129,131,55,132,130,
133,53,134,
34,88,135,
136,51,137,
367,367,3,367,3,1,2,56,3,3,391,
367,367,367,3,367,3,1,2,57,3,3,390,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,58,3,3,389,
367,367,3,367,3,1,2,59,3,3,387,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,1,2,60,
  3,3,386,
142,143,144,145,146,147,148,149,150,151,152,153,138,31,32,140,80,141,154,83,
  84,139,100,101,102,103,104,105,106,107,108,109,110,111,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,1,2,62,3,3,382,
139,139,139,63,155,12,
60,139,139,139,64,157,156,12,
367,367,3,367,3,1,2,65,3,3,380,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,66,96,97,54,
  33,99,40,35,35,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
139,139,139,67,33,12,
367,367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,
  367,3,1,2,68,3,3,376,
367,3,3,1,2,69,3,3,375,
367,3,3,1,2,70,3,3,374,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,71,158,
367,3,3,1,2,72,3,3,383,
367,3,3,1,2,73,3,3,373,
367,3,3,1,2,74,3,3,384,
17,75,24,
139,139,139,76,159,12,
15,17,77,22,160,
15,17,78,21,161,
35,36,37,20,31,32,41,45,34,139,139,159,21,18,169,169,165,159,139,79,96,97,
  54,33,99,40,162,163,162,162,61,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
17,80,19,
17,81,18,
15,17,82,15,164,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,83,96,97,54,33,99,165,165,165,87,86,40,82,
  85,16,13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,
  61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,
  127,24,23,27,19,26,25,169,12,30,29,28,22,
34,84,166,
34,85,167,
34,86,168,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,87,96,97,54,33,99,6,6,169,87,86,40,82,85,
  16,13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,
  55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,
  24,23,27,19,26,25,169,12,30,29,28,22,
177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,91,90,193,192,191,190,189,
  188,187,186,185,184,183,175,177,177,177,88,177,177,177,177,94,93,92,177,
89,89,89,154,155,89,
197,197,90,
203,202,202,202,203,91,200,
199,199,195,
198,198,194,
203,202,202,202,203,182,201,
204,95,
89,89,89,145,144,89,
170,171,171,171,171,97,143,171,
170,171,171,171,171,98,142,171,
138,140,82,141,139,
138,140,81,141,139,
367,3,3,1,2,101,3,3,404,
172,102,173,
367,3,3,1,2,103,3,3,403,
172,104,174,
15,172,105,78,164,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,106,3,3,400,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,107,96,97,54,
  33,99,40,108,77,77,44,43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,
  25,169,12,30,29,28,22,
138,31,32,140,80,141,83,84,139,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,109,3,3,432,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,110,3,3,431,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,111,3,3,430,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,112,96,97,54,
  33,99,40,108,72,72,44,43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,
  25,169,12,30,29,28,22,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,113,96,97,54,
  33,99,40,108,175,175,175,44,43,42,42,39,38,42,33,125,126,127,24,23,27,
  19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,114,3,3,428,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,115,3,3,427,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,116,96,97,54,
  33,99,40,108,176,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,23,27,
  19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,117,3,3,426,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,118,3,3,425,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,119,3,3,424,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,120,3,3,423,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,121,96,97,54,
  33,99,40,108,177,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,23,
  27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,122,3,3,422,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,123,3,3,421,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,124,96,97,54,
  33,99,40,108,178,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,
  23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,125,3,3,398,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,126,96,97,54,
  33,99,40,108,179,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,
  24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,127,3,3,397,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,128,96,97,54,
  33,99,40,108,180,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,
  127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,129,3,3,396,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,130,96,97,54,
  33,99,40,108,181,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,
  126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,131,3,3,395,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,132,96,97,54,
  33,99,40,108,56,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,
  125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,133,3,3,394,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,134,96,97,54,
  33,99,40,108,54,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,
  33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,34,92,139,139,159,21,18,169,169,165,159,139,135,96,97,
  54,33,99,40,94,94,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,
  38,42,183,182,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,136,3,3,393,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,137,96,97,54,
  33,99,40,184,184,184,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,3,367,3,1,2,138,3,3,405,
139,139,139,139,90,12,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,140,3,3,378,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,141,96,97,54,
  33,99,40,185,185,185,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,142,3,3,420,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,143,3,3,419,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,144,3,3,418,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,145,3,3,417,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,146,3,3,416,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,147,3,3,415,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,148,3,3,414,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,149,3,3,413,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,150,3,3,412,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,151,3,3,411,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,152,3,3,410,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,153,3,3,409,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,154,96,97,54,
  33,99,40,50,50,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
34,155,186,
139,139,139,156,43,187,12,
60,139,139,139,157,43,188,189,12,
35,36,37,20,31,32,41,56,57,58,59,60,190,74,72,62,65,68,69,70,73,17,34,139,
  139,159,21,18,169,169,165,159,139,158,96,97,54,33,99,3,3,3,87,86,40,82,
  85,16,13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,32,67,66,82,64,63,
  82,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,
  126,127,24,23,27,19,26,25,169,12,30,29,28,22,
140,159,191,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,160,96,97,54,
  33,99,40,36,36,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
139,139,139,161,34,12,
15,46,164,
17,163,20,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,164,96,97,54,
  33,99,40,48,48,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
73,165,192,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,166,96,97,54,
  33,99,40,193,193,193,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,45,34,139,139,159,21,18,169,169,165,159,139,167,96,97,
  54,33,99,40,162,194,162,162,61,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,168,96,97,54,
  33,99,40,195,195,195,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
196,5,197,
161,161,161,170,198,
161,161,161,171,199,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,172,3,3,370,
20,31,32,34,139,139,159,21,18,169,169,165,159,139,173,96,97,54,33,99,40,108,
  87,87,39,38,87,33,24,23,27,19,26,25,169,12,30,29,28,22,
20,31,32,34,139,139,159,21,18,169,169,165,159,139,174,96,97,54,33,99,40,108,
  86,86,39,38,86,33,24,23,27,19,26,25,169,12,30,29,28,22,
109,110,111,70,112,122,123,124,
37,41,68,113,120,121,
114,115,66,116,118,119,
117,118,119,120,64,121,114,115,116,117,
122,123,62,124,112,113,
125,60,126,
127,58,128,
15,93,200,
172,183,91,
201,15,184,164,202,
15,203,185,89,164,
139,139,139,186,43,204,12,
15,190,187,205,16,
15,40,16,
139,139,139,189,43,206,12,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,1,2,367,
  190,3,3,385,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,191,96,97,54,
  33,99,40,207,207,207,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
34,192,208,
15,172,193,31,164,
17,194,209,
15,172,195,210,164,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,1,2,196,3,3,369,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,197,96,97,54,33,99,7,10,87,86,40,82,85,16,
  13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,55,
  53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,
  23,27,19,26,25,169,12,30,29,28,22,
162,162,162,153,162,
162,162,162,152,162,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,200,96,97,54,
  33,99,40,95,95,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,3,367,367,367,367,367,367,367,3,
  1,2,201,3,3,392,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,202,96,97,54,
  33,99,40,52,108,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,
  42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,367,
  367,367,367,367,3,3,1,2,203,3,3,377,
15,172,204,211,16,
139,139,139,205,43,212,12,
15,190,206,213,16,
15,203,207,214,164,
35,36,37,20,31,32,41,34,139,139,159,21,18,169,169,165,159,139,208,96,97,54,
  33,99,40,215,215,215,61,55,53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,
  39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,45,34,139,139,159,21,18,169,169,165,159,139,209,96,97,
  54,33,99,40,162,216,162,162,61,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,210,96,97,54,33,99,8,11,87,86,40,82,85,16,
  13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,55,
  53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,
  23,27,19,26,25,169,12,30,29,28,22,
60,211,217,
15,39,16,
139,139,139,38,43,218,12,
17,214,23,
15,172,215,219,164,
17,216,220,
26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,26,9,26,26,26,26,26,26,26,26,
  26,26,26,26,26,26,26,26,26,217,221,10,222,
15,37,16,
17,219,14,
35,36,37,20,31,32,41,34,45,139,139,159,21,18,169,169,165,159,139,220,96,97,
  54,33,99,40,162,223,162,162,61,55,53,52,51,50,49,48,47,46,45,45,45,44,
  43,42,42,39,38,42,33,125,126,127,24,23,27,19,26,25,169,12,30,29,28,22,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,221,224,
190,222,41,
172,223,225,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,42,96,97,54,33,99,3,3,3,87,86,40,82,85,16,
  13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,55,
  53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,
  23,27,19,26,25,169,12,30,29,28,22,
35,36,37,20,31,32,41,56,57,58,59,60,74,72,62,65,68,69,70,73,17,34,139,139,
  159,21,18,169,169,165,159,139,225,96,97,54,33,99,9,12,87,86,40,82,85,16,
  13,83,29,13,81,80,79,78,77,76,75,13,28,30,84,71,67,66,82,64,63,82,61,55,
  53,52,51,50,49,48,47,46,45,45,45,44,43,42,42,39,38,42,33,125,126,127,24,
  23,27,19,26,25,169,12,30,29,28,22,

};


static const unsigned short ag_sbt[] = {
     0,  42,  81, 121, 127, 165, 201, 203, 240, 242, 253, 260, 295, 353,
   358, 464, 490, 496, 540, 589, 594, 600, 655, 689, 692, 694, 701, 736,
   772, 805, 838, 871, 907, 943, 947, 977,1003,1029,1055,1062,1069,1137,
  1163,1166,1217,1268,1276,1282,1288,1298,1304,1307,1310,1315,1318,1321,
  1324,1335,1347,1373,1384,1426,1460,1500,1506,1514,1525,1588,1594,1621,
  1630,1639,1674,1683,1692,1701,1704,1710,1715,1720,1786,1789,1792,1797,
  1902,1905,1908,1911,2016,2072,2078,2081,2088,2091,2094,2101,2103,2109,
  2117,2125,2130,2135,2144,2147,2156,2159,2164,2190,2241,2250,2276,2302,
  2328,2379,2431,2457,2483,2536,2562,2588,2614,2640,2694,2720,2746,2801,
  2827,2883,2909,2966,2992,3050,3076,3136,3162,3223,3289,3315,3379,3390,
  3396,3422,3486,3512,3538,3564,3590,3616,3642,3668,3694,3720,3746,3772,
  3798,3861,3864,3871,3880,3987,3990,4053,4059,4062,4065,4128,4131,4195,
  4261,4325,4328,4333,4338,4400,4440,4480,4488,4494,4500,4510,4516,4519,
  4522,4525,4528,4533,4538,4545,4550,4553,4560,4603,4667,4670,4675,4678,
  4683,4723,4827,4832,4837,4900,4926,4988,5037,5042,5049,5054,5059,5123,
  5189,5293,5296,5299,5306,5309,5314,5317,5355,5358,5361,5427,5462,5465,
  5468,5573,5677
};


static const unsigned short ag_sbe[] = {
    38,  78, 118, 125, 161, 200, 202, 239, 241, 249, 256, 293, 349, 355,
   391, 486, 493, 536, 587, 592, 597, 646, 685, 691, 693, 699, 735, 770,
   801, 834, 867, 903, 939, 944, 973, 999,1025,1051,1058,1065,1089,1159,
  1164,1184,1235,1271,1278,1284,1292,1300,1305,1308,1312,1316,1319,1322,
  1331,1343,1369,1380,1422,1442,1496,1503,1510,1521,1543,1591,1617,1626,
  1635,1672,1679,1688,1697,1702,1707,1712,1717,1739,1787,1790,1794,1829,
  1903,1906,1909,1943,2063,2075,2080,2086,2090,2093,2099,2102,2106,2114,
  2122,2127,2132,2140,2145,2152,2157,2161,2186,2208,2245,2272,2298,2324,
  2346,2397,2453,2479,2501,2558,2584,2610,2636,2658,2716,2742,2764,2823,
  2845,2905,2927,2988,3010,3072,3094,3158,3180,3242,3311,3333,3386,3393,
  3418,3440,3508,3534,3560,3586,3612,3638,3664,3690,3716,3742,3768,3794,
  3816,3862,3867,3875,3913,3988,4008,4056,4060,4063,4083,4129,4149,4214,
  4279,4326,4331,4336,4396,4414,4454,4483,4490,4496,4504,4512,4517,4520,
  4523,4526,4530,4535,4541,4547,4551,4556,4599,4621,4668,4672,4676,4680,
  4719,4755,4830,4835,4855,4922,4944,5033,5039,5045,5051,5056,5077,5142,
  5221,5294,5297,5302,5307,5311,5315,5351,5356,5359,5380,5460,5463,5466,
  5500,5605,5677
};


static const unsigned char ag_fl[] = {
  2,3,0,2,1,1,2,4,5,9,4,5,9,1,7,2,1,1,2,2,3,2,2,6,2,1,0,3,1,1,1,4,3,2,3,
  2,3,6,5,5,3,8,2,1,3,0,1,1,3,1,3,1,5,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,
  3,1,3,1,2,2,1,3,3,1,1,2,2,2,2,1,4,4,1,4,3,4,0,1,1,3,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,
  2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,
  3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  9,  9, 11, 11, 12, 12, 12, 12, 13, 13, 13, 13, 23, 23, 23, 23,
   23, 23, 23, 23, 23, 23, 23, 23,  8,  8, 24, 16, 20, 14, 26, 30, 30, 31,
   31, 35, 35, 35, 35, 36, 50, 38, 38, 21, 21, 18, 18, 47, 47, 51, 51, 54,
   54, 57, 57, 59, 59, 61, 61, 63, 63, 65, 65, 67, 67, 69, 69, 71, 71, 73,
   73, 75, 75, 75, 77, 77, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 52, 52,
   52, 85, 89, 89, 90, 90, 82, 82, 82, 82, 53, 53, 53, 53, 53, 53, 53, 53,
   53, 53, 53, 53, 68, 68, 70, 70, 70, 70, 72, 72, 74, 74, 76, 76, 76, 79,
   79, 79,  1,121,121,122,122,  1,126,126,127,127,  1,198,198,226,226,226,
  131,131,131,134,134,134,139,139,133,133,135,135,225,225,225,136,136,140,
  140,137,137,138,138,150,150,142,142,151,151, 91, 91,254,154,154,155,155,
  157,157,157,158,158,158,158,158,158,158,158,158,158,158,159,159,159,172,
  173,174,160,160,177,177,227,179,179,141,118,118,120,120,120,120,120,120,
  120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,120,
  120,120,120,120,120,120,120,120,120,120,120,120,125,125,125,125,125,125,
  125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,
  125,125,125,125,125,125,125,125,125,125,125,129,129,129,130,130,130,130,
  130,130,143,143,144,144,144,147,147,152,152,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,156,
  156,156,156,156,156,156,156,156,180,180,180,180,180,180,180,180,180,180,
  180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
  180,180,180,180,180,186,186,187,187, 15, 19, 17, 22, 25, 27, 28, 29, 34,
   33,  4, 32, 37, 39, 40, 41, 43, 42, 44, 45, 46, 48, 49, 56, 55, 58, 60,
   62, 64, 66, 78, 81, 83, 84, 86, 87, 88,  2,  3,  6, 92, 93, 94, 95, 96,
   97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,
  115,116,117,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, V(0,(AstNode * *)), VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(2,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER, V(1,(AstNode * *)), V(4,(AstNode * *))); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(0,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: V(0,(AstNode * *)) = ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_4 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 21: V(0,(AstNode * *)) = ag_rp_21(PCB_POINTER); break;
    case 22: V(0,(AstNode * *)) = ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_7 *); break;
    case 23: ag_rp_23(PCB_POINTER); break;
    case 24: ag_rp_24(PCB_POINTER); break;
    case 25: ag_rp_25(PCB_POINTER); break;
    case 26: V(0,(AstNode * *)) = ag_rp_26(PCB_POINTER, V(2,(AstNode * *))); break;
    case 27: V(0,(AstNode * *)) = ag_rp_27(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 28: VNO AG_WRAP_6(ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 29: VRO(AG_WRAP_6 *, ag_rp_29(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 30: VNO AG_WRAP_6(ag_rp_30(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 31: VRO(AG_WRAP_6 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(5, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(5, AG_WRAP_7 *); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); break;
    case 34: V(0,(AstNode * *)) = ag_rp_34(PCB_POINTER, VW(2, AG_WRAP_7 *), VW(4, AG_WRAP_7 *)); 
            VWD(2, AG_WRAP_7 *); VWD(4, AG_WRAP_7 *); break;
    case 35: V(0,(AstNode * *)) = ag_rp_35(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_7 *); break;
    case 36: V(0,(AstNode * *)) = ag_rp_36(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(6, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(6, AG_WRAP_4 *); break;
    case 37: VNO AG_WRAP_4(ag_rp_37(PCB_POINTER)); break;
    case 38: VRO(AG_WRAP_4 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 39: VRO(AG_WRAP_7 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 40: V(0,(AstNode * *)) = ag_rp_40(PCB_POINTER); break;
    case 41: V(0,(AstNode * *)) = ag_rp_41(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 42: V(0,(AstNode * *)) = ag_rp_42(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 43: V(0,(AstNode * *)) = ag_rp_43(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *)), V(4,(AstNode * *))); break;
    case 44: V(0,(AstNode * *)) = ag_rp_44(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 45: V(0,(AstNode * *)) = ag_rp_45(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 46: V(0,(AstNode * *)) = ag_rp_46(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 47: V(0,(AstNode * *)) = ag_rp_47(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 48: V(0,(AstNode * *)) = ag_rp_48(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 49: V(0,(AstNode * *)) = ag_rp_49(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 50: V(0,(AstNode * *)) = ag_rp_50(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 51: V(0,(AstNode * *)) = ag_rp_51(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 52: V(0,(AstNode * *)) = ag_rp_52(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 53: V(0,(AstNode * *)) = ag_rp_53(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 54: V(0,(AstNode * *)) = ag_rp_54(PCB_POINTER, V(1,(AstNode * *))); break;
    case 55: V(0,(AstNode * *)) = ag_rp_55(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 56: V(0,(AstNode * *)) = ag_rp_56(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 57: V(0,(AstNode * *)) = ag_rp_57(PCB_POINTER, V(1,(AstNode * *))); break;
    case 58: V(0,(AstNode * *)) = ag_rp_58(PCB_POINTER, V(0,(AstNode * *))); break;
    case 59: V(0,(AstNode * *)) = ag_rp_59(PCB_POINTER, V(1,(AstNode * *))); break;
    case 60: V(0,(AstNode * *)) = ag_rp_60(PCB_POINTER, V(1,(AstNode * *))); break;
    case 61: V(0,(AstNode * *)) = ag_rp_61(PCB_POINTER, V(0,(AstNode * *))); break;
    case 62: V(0,(AstNode * *)) = ag_rp_62(PCB_POINTER, V(0,(AstNode * *))); break;
    case 63: V(0,(AstNode * *)) = ag_rp_63(PCB_POINTER, V(3,(AstNode * *))); break;
    case 64: V(0,(AstNode * *)) = ag_rp_64(PCB_POINTER, V(3,(AstNode * *))); break;
    case 65: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 66: V(0,(AstNode * *)) = ag_rp_66(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 67: V(0,(AstNode * *)) = ag_rp_67(PCB_POINTER, V(0,(AstNode * *)), VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 68: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_68(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 69: VNO AG_WRAP_6(ag_rp_69(PCB_POINTER)); break;
    case 70: VNO AG_WRAP_6(ag_rp_70(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 71: VRO(AG_WRAP_6 *, ag_rp_71(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 72: V(0,(AstNode * *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 73: V(0,(AstNode * *)) = ag_rp_73(PCB_POINTER, V(0,(double *))); break;
    case 74: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_74(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 75: V(0,(AstNode * *)) = ag_rp_75(PCB_POINTER, V(0,(int *))); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: V(0,(Opcode *)) = ag_rp_78(PCB_POINTER); break;
    case 79: V(0,(Opcode *)) = ag_rp_79(PCB_POINTER); break;
    case 80: V(0,(Opcode *)) = ag_rp_80(PCB_POINTER); break;
    case 81: V(0,(Opcode *)) = ag_rp_81(PCB_POINTER); break;
    case 82: V(0,(Opcode *)) = ag_rp_82(PCB_POINTER); break;
    case 83: V(0,(Opcode *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(Opcode *)) = ag_rp_84(PCB_POINTER); break;
    case 85: V(0,(Opcode *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(Opcode *)) = ag_rp_86(PCB_POINTER); break;
    case 87: V(0,(Opcode *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(Opcode *)) = ag_rp_88(PCB_POINTER); break;
    case 89: V(0,(Opcode *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(Opcode *)) = ag_rp_90(PCB_POINTER); break;
    case 91: V(0,(Opcode *)) = ag_rp_91(PCB_POINTER); break;
    case 92: V(0,(Opcode *)) = ag_rp_92(PCB_POINTER); break;
    case 93: V(0,(Opcode *)) = ag_rp_93(PCB_POINTER); break;
    case 94: V(0,(Opcode *)) = ag_rp_94(PCB_POINTER); break;
    case 95: V(0,(Opcode *)) = ag_rp_95(PCB_POINTER); break;
    case 96: V(0,(Opcode *)) = ag_rp_96(PCB_POINTER); break;
    case 97: V(0,(Opcode *)) = ag_rp_97(PCB_POINTER); break;
    case 98: V(0,(Opcode *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(Opcode *)) = ag_rp_99(PCB_POINTER); break;
    case 100: V(0,(Opcode *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(Opcode *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(Opcode *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(Opcode *)) = ag_rp_103(PCB_POINTER); break;
    case 104: VNO AG_WRAP_4(ag_rp_104(PCB_POINTER, V(0,(int *)))); break;
    case 105: VRO(AG_WRAP_4 *, ag_rp_105(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 106: V(0,(double *)) = ag_rp_106(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 107: V(0,(double *)) = ag_rp_107(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 108: V(0,(double *)) = ag_rp_108(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 109: V(0,(double *)) = ag_rp_109(PCB_POINTER, V(0,(double *))); break;
    case 110: V(0,(double *)) = ag_rp_110(PCB_POINTER, V(1,(double *))); break;
    case 111: V(0,(double *)) = ag_rp_111(PCB_POINTER, V(0,(long *))); break;
    case 112: V(0,(double *)) = ag_rp_112(PCB_POINTER, V(0,(long *))); break;
    case 113: V(0,(double *)) = ag_rp_113(PCB_POINTER, V(0,(long *))); break;
    case 114: V(0,(long *)) = ag_rp_114(PCB_POINTER, V(1,(long *))); break;
    case 115: V(0,(long *)) = ag_rp_115(PCB_POINTER, V(1,(long *))); break;
    case 116: V(0,(double *)) = ag_rp_116(PCB_POINTER, V(0,(int *))); break;
    case 117: V(0,(double *)) = ag_rp_117(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 118: V(0,(long *)) = ag_rp_118(PCB_POINTER, V(0,(int *))); break;
    case 119: V(0,(long *)) = ag_rp_119(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 120: V(0,(long *)) = ag_rp_120(PCB_POINTER, V(0,(int *))); break;
    case 121: V(0,(long *)) = ag_rp_121(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 122: V(0,(long *)) = ag_rp_122(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 123: V(0,(long *)) = ag_rp_123(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 124: V(0,(long *)) = ag_rp_124(PCB_POINTER); break;
    case 125: V(0,(long *)) = ag_rp_125(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 126: V(0,(long *)) = ag_rp_126(PCB_POINTER); break;
    case 127: V(0,(long *)) = ag_rp_127(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 128: V(0,(int *)) = ag_rp_128(PCB_POINTER, V(0,(int *))); break;
    case 129: V(0,(int *)) = ag_rp_129(PCB_POINTER, V(0,(int *))); break;
    case 130: VRO(AG_WRAP_5 *, ag_rp_130(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 131: VNO AG_WRAP_5(ag_rp_131(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 132: VNO AG_WRAP_4(ag_rp_132(PCB_POINTER)); break;
    case 133: VRO(AG_WRAP_4 *, ag_rp_133(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 134: V(0,(int *)) = ag_rp_134(PCB_POINTER); break;
    case 135: V(0,(int *)) = ag_rp_135(PCB_POINTER); break;
    case 136: V(0,(int *)) = ag_rp_136(PCB_POINTER); break;
    case 137: V(0,(int *)) = ag_rp_137(PCB_POINTER); break;
    case 138: V(0,(int *)) = ag_rp_138(PCB_POINTER); break;
    case 139: V(0,(int *)) = ag_rp_139(PCB_POINTER); break;
    case 140: V(0,(int *)) = ag_rp_140(PCB_POINTER); break;
    case 141: V(0,(int *)) = ag_rp_141(PCB_POINTER); break;
    case 142: V(0,(int *)) = ag_rp_142(PCB_POINTER); break;
    case 143: V(0,(int *)) = ag_rp_143(PCB_POINTER); break;
    case 144: V(0,(int *)) = ag_rp_144(PCB_POINTER); break;
    case 145: V(0,(int *)) = ag_rp_145(PCB_POINTER, V(1,(int *))); break;
    case 146: V(0,(int *)) = ag_rp_146(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 147: V(0,(int *)) = ag_rp_147(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 148: V(0,(int *)) = ag_rp_148(PCB_POINTER, V(1,(int *))); break;
    case 149: V(0,(int *)) = ag_rp_149(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 150: V(0,(int *)) = ag_rp_150(PCB_POINTER, V(0,(int *))); break;
    case 151: V(0,(int *)) = ag_rp_151(PCB_POINTER, V(0,(int *))); break;
    case 152: V(0,(int *)) = ag_rp_152(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[255] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "local statement",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"else\"",
  "WHILE",
  "'('",
  "expression",
  "')'",
  "FOR",
  "optional expression",
  "';'",
  "simple statement",
  "DO",
  "\"while\"",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"array\"",
  "'['",
  "']'",
  "structure declaration",
  "function definition",
  "\"local\"",
  "name list",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "\"struct\"",
  "\"function\"",
  "function body",
  "conditional expression",
  "lvalue",
  "assignment op",
  "logical or expression",
  "'\\?'",
  "':'",
  "logical and expression",
  "\"||\"",
  "inclusive or expression",
  "\"&&\"",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "equality op",
  "shift expression",
  "relational op",
  "additive expression",
  "shift op",
  "multiplicative expression",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"else\"",
  "')'",
  "'('",
  "';'",
  "\"while\"",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "']'",
  "'['",
  "name",
  "\"array\"",
  "\"local\"",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'}'",
  "'{'",
  "\"dump\"",
  "','",
  "\"print\"",
  "\"struct\"",
  "\"function\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "'+'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


