/*
 dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.
 Modified to support structures and instances.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. Reduction procedures have been added to enable it to
 compile bytecode as it parses. Changes in the syntax have been minimal.

 dci.syn is compiled with the AnaGram parser generator, which creates dci.h
 and dci.cpp.

 To build a demonstration program for this parser, compile and link as a
 single project:
   dci.cpp                 // This parser, compiles bytecode
   demo.cpp                // Demo program
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348263
#include "dci.h"
#endif

#ifndef DCI_H_1027348263
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 558, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::checkLoop() {
  if (loopDepth) return;
  PCONTEXT(*this) = FileLocation(pointer, line, column);
  reportError("No loop active");
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d),
    loopDepth(0)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::codeMember(const AgString &name) {
  return CodeFragment().append(MEMBER,fieldNames.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  argCode.append(CALL, idFunction(name, argCount));
  return argCode;
}

CodeFragment dci_pcb_struct::codeStructDecl(
    const AgString &name,
    const AgStack<AgString> &fields,
    const AgStack<AgString> &instances)
{
  int nid = dictionary.intern(name);
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);

  CodeFragment structDef;
  structDef.append(LOCATE, nid)
           .append(PUSHC, sid)
           .append(SAP);
  for (int i = 0; i < instances.size(); i++) {
    structDef.append(LOCATE, dictionary.intern(instances[i]))
             .append(PUSHC, sid)
             .append(INSTANCE)
             .append(SAP);
  }
  return structDef;
}

CodeFragment dci_pcb_struct::codeStructDecl(
    const AgString &name,
    const AgStack<AgString> &fields)
{
  int nid = dictionary.intern(name);
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);

  CodeFragment structDef;
  structDef.append(LOCATE, nid)
           .append(PUSHC, sid)
           .append(SAP);
  return structDef;
}

CodeFragment dci_pcb_struct::codeInstanceDecl(
    const AgStack<AgString> &fields,
    const AgStack<AgString> &instances)
{
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);
  CodeFragment instanceDef;
  for (int i = 0; i < instances.size(); i++) {
    instanceDef.append(LOCATE, dictionary.intern(instances[i]))
               .append(PUSHC, sid)
               .append(INSTANCE)
               .append(SAP);
  }
  return instanceDef;
}

CodeFragment dci_pcb_struct::codeInstanceDecl(
    const AgString &name,
    const AgStack<AgString> &instances)
{
  CodeFragment instanceDef;
  int n = dictionary.intern(name);
  for (int i = 0; i < instances.size(); i++) {
    instanceDef.append(LOCATE, dictionary.intern(instances[i]))
               .append(LOCATE, n)
               .append(FETCH)
               .append(INSTANCE)
               .append(SAP);
  }
  return instanceDef;
}

ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList(),
  fieldNames()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
  fieldNames = pcb.fieldNames.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, CodeFragment &x) {
/* Line 138, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 142, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 143, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 161, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 163, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 165, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 168, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.loopDepth--,CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 172, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 174, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 177, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.loopDepth--, CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 178, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 191, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.loopDepth--, CodeFragment::doLoop(s,x);
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(POP);
}

static inline CodeFragment ag_rp_14(PCB_DECL) {
/* Line 193, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_15(PCB_DECL) {
/* Line 195, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendBreak(BR);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendContinue(BR);
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 197, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 198, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(RETURN);
}

static inline void ag_rp_19(PCB_DECL) {
/* Line 204, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_20(PCB_DECL) {
/* Line 207, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 210, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  PCB.loopDepth++;
}

static inline CodeFragment ag_rp_22(PCB_DECL, CodeFragment &x) {
/* Line 213, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_23(PCB_DECL, CodeFragment &s) {
/* Line 216, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_24(PCB_DECL, AgString &n) {
/* Line 220, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_25(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 221, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_26(PCB_DECL, CodeFragment &code) {
/* Line 225, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 227, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_28(PCB_DECL, AgString &n, AgStack<AgString> &fl, AgStack<AgString> &il) {
/* Line 231, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.codeStructDecl(n, fl, il);
}

static inline CodeFragment ag_rp_29(PCB_DECL, AgString &n, AgStack<AgString> &fl) {
/* Line 232, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.codeStructDecl(n, fl);
}

static inline CodeFragment ag_rp_30(PCB_DECL, AgStack<AgString> &fl, AgStack<AgString> &nl) {
/* Line 234, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.codeInstanceDecl(fl, nl);
}

static inline CodeFragment ag_rp_31(PCB_DECL, AgString &n, AgStack<AgString> &nl) {
/* Line 235, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.codeInstanceDecl(n, nl);
}

static inline AgStack<AgString> ag_rp_32(PCB_DECL, AgString &n) {
/* Line 238, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_33(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 239, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return stack.push(n);
}

static inline CodeFragment ag_rp_34(PCB_DECL) {
/* Line 243, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_35(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 250, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(POP).concat(y);
}

static inline CodeFragment ag_rp_36(PCB_DECL, CodeFragment &lv, Opcode op, CodeFragment &x) {
/* Line 256, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return lv.concat(x).append(op);
}

static inline CodeFragment ag_rp_37(PCB_DECL, CodeFragment &c, CodeFragment &x, CodeFragment &y) {
/* Line 262, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return CodeFragment::ifElse(c, x, y);
}

static inline CodeFragment ag_rp_38(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 272, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(LOR, y.size()).concat(y);
}

static inline CodeFragment ag_rp_39(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 277, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(LAND, y.size()).concat(y);
}

static inline CodeFragment ag_rp_40(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 283, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(IOR);
}

static inline CodeFragment ag_rp_41(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 288, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(XOR);
}

static inline CodeFragment ag_rp_42(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 293, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(AND);
}

static inline CodeFragment ag_rp_43(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 298, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_44(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 303, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_45(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 308, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_46(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 313, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_47(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 318, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_48(PCB_DECL, CodeFragment &x) {
/* Line 322, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_49(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 323, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_50(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 331, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_51(PCB_DECL, CodeFragment &x) {
/* Line 339, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_52(PCB_DECL, Constant &x) {
/* Line 340, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_53(PCB_DECL, CodeFragment &x) {
/* Line 341, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_54(PCB_DECL, CodeFragment &x) {
/* Line 342, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(I_FETCH);
}

static inline CodeFragment ag_rp_55(PCB_DECL, CodeFragment &x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(D_FETCH);
}

static inline CodeFragment ag_rp_56(PCB_DECL, CodeFragment &x) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(FETCH_I);
}

static inline CodeFragment ag_rp_57(PCB_DECL, CodeFragment &x) {
/* Line 345, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(FETCH_D);
}

static inline CodeFragment ag_rp_58(PCB_DECL, CodeFragment &x) {
/* Line 347, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_59(PCB_DECL, CodeFragment &x) {
/* Line 348, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_60(PCB_DECL, AgString &n) {
/* Line 351, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_61(PCB_DECL, CodeFragment &lv, AgString &n) {
/* Line 352, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return lv.concat(PCB.codeMember(n));
}

static inline CodeFragment ag_rp_62(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 355, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_63(PCB_DECL) {
/* Line 358, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_64(PCB_DECL, CodeFragment &code) {
/* Line 362, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_65(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 363, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_66(PCB_DECL, long x) {
/* Line 366, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_67(PCB_DECL, double x) {
/* Line 367, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_68(PCB_DECL, int x) {
/* Line 369, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 373, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return STORE;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return ADDM;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return SUBM;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 376, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return MULM;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 377, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return DIVM;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return MODM;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return IORM;
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 380, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return ANDM;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 381, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return XORM;
}

static inline Opcode ag_rp_78(PCB_DECL) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return LSM;
}

static inline Opcode ag_rp_79(PCB_DECL) {
/* Line 383, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return RSM;
}

static inline Opcode ag_rp_80(PCB_DECL) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return POWM;
}

static inline Opcode ag_rp_81(PCB_DECL) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_82(PCB_DECL) {
/* Line 388, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return NE;
}

static inline Opcode ag_rp_83(PCB_DECL) {
/* Line 391, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return LT;
}

static inline Opcode ag_rp_84(PCB_DECL) {
/* Line 392, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return LE;
}

static inline Opcode ag_rp_85(PCB_DECL) {
/* Line 393, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return GT;
}

static inline Opcode ag_rp_86(PCB_DECL) {
/* Line 394, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return GE;
}

static inline Opcode ag_rp_87(PCB_DECL) {
/* Line 397, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return LS;
}

static inline Opcode ag_rp_88(PCB_DECL) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return RS;
}

static inline Opcode ag_rp_89(PCB_DECL) {
/* Line 401, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_90(PCB_DECL) {
/* Line 402, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_91(PCB_DECL) {
/* Line 405, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_92(PCB_DECL) {
/* Line 406, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return DIV;
}

static inline Opcode ag_rp_93(PCB_DECL) {
/* Line 407, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_94(PCB_DECL) {
/* Line 410, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_95(PCB_DECL) {
/* Line 411, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return NOT;
}

static inline Opcode ag_rp_96(PCB_DECL) {
/* Line 412, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return COM;
}

static inline AgString ag_rp_97(PCB_DECL, int c) {
/* Line 430, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_98(PCB_DECL, AgString &ns, int c) {
/* Line 431, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return ns.append(c);
}

static inline double ag_rp_99(PCB_DECL, double x, long e) {
/* Line 437, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_100(PCB_DECL, double x, long e) {
/* Line 438, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_101(PCB_DECL, double i, double f) {
/* Line 441, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return i+f;
}

static inline double ag_rp_102(PCB_DECL, double i) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return i;
}

static inline double ag_rp_103(PCB_DECL, double f) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return f;
}

static inline double ag_rp_104(PCB_DECL, long x) {
/* Line 446, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline double ag_rp_105(PCB_DECL, long x) {
/* Line 447, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x;
}

static inline double ag_rp_106(PCB_DECL, long x) {
/* Line 448, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_107(PCB_DECL, long x) {
/* Line 451, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return (long)x;
}

static inline long ag_rp_108(PCB_DECL, long x) {
/* Line 452, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return -(long)x;
}

static inline double ag_rp_109(PCB_DECL, int d) {
/* Line 455, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_110(PCB_DECL, int d, double f) {
/* Line 456, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_111(PCB_DECL, int d) {
/* Line 464, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return d-'0';
}

static inline long ag_rp_112(PCB_DECL, long x, int d) {
/* Line 465, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_113(PCB_DECL, int d) {
/* Line 468, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return d-'0';
}

static inline long ag_rp_114(PCB_DECL, long x, int d) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_115(PCB_DECL, long x, int d) {
/* Line 472, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_116(PCB_DECL, long x, int d) {
/* Line 473, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_117(PCB_DECL) {
/* Line 476, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 0;
}

static inline long ag_rp_118(PCB_DECL, long n, int d) {
/* Line 477, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_119(PCB_DECL) {
/* Line 480, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 0;
}

static inline long ag_rp_120(PCB_DECL, long n, int d) {
/* Line 481, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_121(PCB_DECL, int x) {
/* Line 484, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return x - '0';
}

static inline int ag_rp_122(PCB_DECL, int d) {
/* Line 485, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_123(PCB_DECL, Value &s) {
/* Line 490, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_124(PCB_DECL, Constant &s, Value &e) {
/* Line 491, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_125(PCB_DECL, AgString &b) {
/* Line 494, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_126(PCB_DECL) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return AgString();
}

static inline AgString ag_rp_127(PCB_DECL, AgString &s, int c) {
/* Line 499, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return s.append(c);
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 511, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\'';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 512, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '"';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 513, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\?';
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 514, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\\';
}

static inline int ag_rp_132(PCB_DECL) {
/* Line 515, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\a';
}

static inline int ag_rp_133(PCB_DECL) {
/* Line 516, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\b';
}

static inline int ag_rp_134(PCB_DECL) {
/* Line 517, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\f';
}

static inline int ag_rp_135(PCB_DECL) {
/* Line 518, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\n';
}

static inline int ag_rp_136(PCB_DECL) {
/* Line 519, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\r';
}

static inline int ag_rp_137(PCB_DECL) {
/* Line 520, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\t';
}

static inline int ag_rp_138(PCB_DECL) {
/* Line 521, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return '\v';
}

static inline int ag_rp_139(PCB_DECL, int d) {
/* Line 527, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return d-'0';
}

static inline int ag_rp_140(PCB_DECL, int n, int d) {
/* Line 530, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_141(PCB_DECL, int n, int d) {
/* Line 533, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_142(PCB_DECL, int d) {
/* Line 536, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return d;
}

static inline int ag_rp_143(PCB_DECL, int n, int d) {
/* Line 537, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_144(PCB_DECL, int d) {
/* Line 540, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return d-'0';
}

static inline int ag_rp_145(PCB_DECL, int d) {
/* Line 541, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_146(PCB_DECL, int c) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/struct/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  8, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 4,
  6, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 6, 0, 0,
  0, 6, 0, 0, 0, 0, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0,
  0, 6, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 4, 6, 0, 0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 0, 0, 0, 0, 6, 6,
  6, 6, 6, 6, 6, 7, 7, 6, 9, 9, 0, 0, 0, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 9, 0, 0, 0, 9, 0, 6, 6, 9, 0, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >
#undef AG_WRAP_9
#define AG_WRAP_9 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      case 9: ag_delete_object((AG_WRAP_9 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,  0,
   15, 16, 17, 18,  0,  0,  0, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
   30, 31, 32, 33, 34,  0,  0, 35,  0, 36,  0, 37,  0, 38,  0, 39,  0, 40,
    0, 41,  0, 42,  0, 43,  0, 44,  0, 45,  0, 46,  0, 47,  0, 48, 49,  0,
   50, 51, 52, 53, 54, 55, 56, 57,  0, 58, 59, 60, 61, 62, 63,  0, 64, 65,
   66, 67,  0, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82,
   83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0, 97, 98,  0, 99,100,101,102,103,104,105,106,
    0,  0,107,108,109,110,  0,  0,  0,111,112,113,114,115,116,117,118,  0,
    0,119,120,121,122,123,124,125,126,127,  0,  0,  0,  0,  0,128,129,130,
  131,132,133,134,135,136,137,138,  0,  0,  0,139,140,141,142,143,144,145,
  146
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,180,  0,184,  0,185,  0,186,  0,187,  0,188,  0,189,  0,190,
  0,194,  0,196,  0,197,  0,209,  0,210,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 88,120,255,111,117,255, 43, 45, 47, 48, 98, 99,100,102,
  105,112,114,115,119,255, 42,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,
  255, 45, 61,255, 42, 47, 61,255, 88,120,255, 61,255, 60, 61,255, 61,255,
   61, 62,255,117,255,111,117,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47,
   48, 60, 61, 62, 94, 98, 99,100,101,102,105,108,112,114,115,119,124,255,
   88,120,255,111,117,255, 43, 45, 48, 98, 99,100,102,105,112,114,115,119,
  255, 34, 39, 63, 92, 97, 98,102,110,114,116,118,120,255, 92,255, 42, 47,
  255, 60, 61,255, 61, 62,255, 33, 38, 42, 47, 60, 61, 62,124,255, 60, 61,
  255, 61, 62,255, 33, 38, 42, 60, 61, 62,124,255, 42, 47,255, 88,120,255,
   43, 45, 47, 48,100,108,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,255,
   88,120,255, 43, 45, 48,100,108,255, 88,120,255, 43, 45, 48,255, 60, 61,
  255, 61, 62,255, 33, 38, 60, 61, 62,124,255, 33, 38, 60, 61, 62,124,255,
   33, 38, 61,124,255, 38,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,
  255, 45, 61,255, 42, 47, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255,
   61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255,124,255,
   38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255, 61,255, 60, 61,
  255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61,
   62, 94,124,255, 42, 47,255, 47,255, 42, 47,255, 88,120,255,111,117,255,
   43, 45, 47, 48, 98, 99,100,101,102,105,112,114,115,119,255, 60, 61,255,
   61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,124,255,119,255, 88,120,255,
  111,117,255, 43, 45, 48, 98, 99,100,101,102,105,112,114,115,119,255, 42,
   47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,255, 33, 38, 42, 43,
   45, 47, 48, 60, 61, 62, 98, 99,100,102,105,112,114,115,119,124,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,7,4,3,4,0,0,4,0,4,1,0,4,0,
  0,4,0,0,4,0,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,7,4,6,7,4,0,0,4,3,3,2,2,2,
  2,2,2,2,3,2,3,7,7,2,7,7,7,7,7,7,7,7,2,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,
  7,7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,
  3,4,0,0,4,0,0,4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,0,0,4,
  3,3,2,2,4,0,0,4,3,3,2,7,7,4,0,0,4,3,3,2,4,0,0,4,0,0,4,3,3,2,3,2,3,4,3,
  3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,4,1,
  0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,3,4,0,0,4,0,4,1,0,4,0,0,
  4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,4,0,0,4,2,4,
  0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,7,7,4,0,0,4,0,0,4,3,3,3,3,3,
  2,3,2,3,4,7,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,7,7,4,0,0,4,0,0,4,0,0,
  4,0,0,4,5,7,4,3,3,3,3,3,2,2,2,3,2,7,7,2,7,7,7,7,7,7,3,4
};

static const unsigned char ag_key_parm[] = {
    0,111,116,  0,141,140,  0, 10, 16,  0,207,208,  0,  0,  4,  6,  0, 12,
   14, 18,  8, 20,  2,  0,115,  0,201,222,  0,226,  0,206,218,  0,207,216,
    0,208,217,  0,111,116,219,  0,141,140,  0,224,  0,233,230,  0,225,  0,
  232,234,  0, 24,  0, 10, 16,  0,221,200,  0,228,220,  0,  0,  0,  0,  0,
    0,  0,227,  0,223,  4,  6,  0,  0, 12, 14, 22, 18,  8, 20,  2,  0,  0,
  141,140,  0, 10, 16,  0,207,208,  0,  4,  6,  0, 12, 14, 18,  8, 20,  2,
    0,154,153,155,156,157,158,159,160,161,162,163,168,  0,  0,  0,111,116,
    0,233,230,  0,232,234,  0,228,201,206,  0,  0,227,  0,200,  0,233,230,
    0,232,234,  0,228,201,206,  0,227,  0,200,  0,111,116,  0,141,140,  0,
  207,208,  0,  0, 24, 22,  0,111,116,  0,141,140,  0,207,208,  0,  0,  0,
  141,140,  0,207,208,  0, 24, 22,  0,141,140,  0,207,208,  0,  0,233,230,
    0,232,234,  0,228,201,  0,227,  0,200,  0,228,201,230,227,232,200,  0,
  228,201,227,200,  0,201,200,  0,201,222,  0,226,  0,206,218,  0,207,216,
    0,208,217,  0,111,116,219,  0,224,  0,233,230,  0,225,  0,232,234,  0,
  221,200,  0,228,220,  0,  0,  0,  0,  0,  0,227,  0,223,  0,  0,200,  0,
  201,222,  0,226,  0,206,218,  0,207,216,  0,208,217,  0,224,  0,233,230,
    0,225,  0,232,234,  0,221,200,  0,228,220,  0,  0,  0,  0,219,  0,227,
    0,223,  0,  0,111,116,  0,  0,  0,111,116,  0,141,140,  0, 10, 16,  0,
  207,208,  0,  0,  4,  6,  0,  0, 12, 14, 18,  8, 20,  2,  0,233,230,  0,
  232,234,  0,228,201,206,207,208,  0,227,  0,200,  0,  2,  0,141,140,  0,
   10, 16,  0,207,208,  0,  4,  6,  0,  0, 12, 14, 18,  8, 20,  2,  0,111,
  116,  0,141,140,  0,233,230,  0,232,234,  0, 10, 16,  0,228,201,206,207,
  208,  0,  0,  0,227,  0,  4,  6,  0, 12, 14, 18,  8, 20,  2,200,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  0,  0,  0,  0, 17,  0,  0,  2,  1,  4,  4,  9,  7, 20,
   23, 25, 30, 36, 42,  0, 47,  0,  0,  0,  0,  0,  0, 29,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 47,  0,  0,  0,  0,
    0, 52,  0, 70,  0, 57, 74,  0,  0,  0,  0, 49, 51, 26, 31, 34, 37, 40,
   44, 49, 53, 54, 55, 57, 62, 59, 77, 81, 84, 86, 90, 95,101,107, 62,  0,
    0,  0,  0,  0,129,  0,112,114, 90,116,121, 93,132,135,137,142,148,154,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,109,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,159,161,163,124,127,165,130,167,  0,  0,  0,
    0,  0,  0,  0,169,171,173,142,175,145,177,  0,  0,  0,  0,  0,  0,  0,
  179,181,156,159,183,189,  0,  0,  0,  0,  0,  0,  0,193,195,169,172,  0,
    0,  0,  0,197,199,180,201,207,  0,  0,  0,  0,211,213,189,  0,  0,  0,
    0,  0,  0,  0,215,217,196,219,199,221,  0,223,225,227,229,231,233,  0,
  235,237,239,241,  0,243,245,  0,  0,  0,  0,  0,  0,227,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,242,  0,  0,  0,  0,  0,247,  0,
    0,  0,  0,247,249,224,229,232,235,238,244,251,249,253,252,  0,255,  0,
    0,  0,  0,  0,  0,273,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,284,  0,
    0,  0,  0,  0,289,  0,  0,  0,  0,257,259,270,275,278,281,261,286,263,
  291,265,294,  0,  0,  0,  0,310,  0,  0,  0,  0,  0,  0,  0,  0,284,  0,
  267,269,315,318,271,276,321,287,291,294,296,301,307,313,  0,  0,  0,  0,
    0,  0,  0,318,320,322,324,326,339,328,342,330,  0,332,  0,  0,  0,  0,
    0,354,  0,337,339,357,341,346,360,357,361,364,366,371,377,383,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,413,  0,388,390,392,394,
  396,377,380,383,398,386,400,405,389,416,419,421,426,432,438,443,  0
};

static const unsigned short ag_key_index[] = {
   10,  0, 24, 65, 96,  0,  0, 24, 24, 96,122,  0,  0,122,133,  0,133,133,
  133,133,133,133,133,133,133,148,162,175,175,175,  0,  0,183,175,148,192,
  192,202,202,202,209,216,221,221,221,255,268,297,  0,313,175,313, 10,297,
   10,  0,192,  0,175,313,313, 96,324,313,313,313,  0,  0,  0,192,  0,  0,
    0, 96,  0,  0,  0, 96,122,133,  0,  0,122,122,122,  0,133,  0,  0,148,
  148,313,  0,313,  0,  0,175,192,345,175,175,175,192,192,175,175,192,175,
  175,175,175,192,175,175,192,175,192,175,192,175,192,175,192,175,192,192,
  175,192,313,  0,175,175,175,175,175,175,175,175,175,175,175,175,192,  0,
    0, 96,175,192,  0,  0,192,355,192,192,192,363,  0,  0,392,192,192,202,
  202,202,209,216,221,221,  0,  0,  0,  0,  0,  0,324,  0,  0,  0,  0,  0,
   10, 96,133,133,192,175,192,  0,  0,  0,192,192, 96,  0,  0,  0,  0,  0,
    0,192,  0, 96
};

static const unsigned char ag_key_ends[] = {
43,0, 45,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
116,114,117,99,116,0, 104,105,108,101,0, 47,0, 61,0, 61,0, 61,0, 
61,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 98,108,101,0, 
109,112,0, 108,115,101,0, 111,114,0, 102,0, 111,110,103,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 61,0, 38,0, 61,0, 124,0, 61,0, 
38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 
38,0, 124,0, 61,0, 61,0, 61,0, 61,0, 124,0, 61,0, 61,0, 
61,0, 61,0, 61,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 
104,105,108,101,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
116,114,117,99,116,0, 104,105,108,101,0, 124,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,174,174,174,174,174,174,174,174,173,120,173,173,173,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,173,239,145,174,
  174,238,204,170,182,181,236,205,195,235,211,237,138,175,175,175,175,175,
  175,175,137,137,198,183,229,215,231,199,174,176,176,176,176,124,176,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,174,167,174,203,177,174,176,176,176,176,124,176,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,192,202,191,
  240,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,173,170,145,141,140,138,137,124,120,116,111,9,0,1,178,
  179,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,167,145,138,137,124,120,0,118,119,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,167,145,138,137,124,120,115,0,113,
  114,
173,120,116,111,0,1,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,9,0,7,8,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,167,145,138,137,124,0,
120,0,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,167,145,138,137,124,120,0,
115,0,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,9,0,2,3,4,5,6,10,11,12,13,
  15,16,17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,
  44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,
  109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,168,167,163,162,161,160,159,158,157,
  156,155,154,153,145,138,137,124,0,146,
175,138,137,0,133,
175,138,137,0,127,133,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,168,167,163,162,161,160,159,158,157,156,
  155,154,153,145,138,137,124,0,149,150,151,152,164,165,166,171,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,173,145,120,116,111,0,1,178,179,
211,124,0,
124,0,
176,175,138,137,124,0,143,
238,237,236,235,234,233,232,231,230,229,228,227,211,206,205,204,203,202,201,
  200,199,198,195,183,181,175,173,138,137,124,120,116,111,0,
238,237,236,235,234,233,232,231,230,229,228,227,211,206,205,204,203,202,201,
  200,199,198,195,183,181,175,173,138,137,124,120,116,111,0,133,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,173,120,116,111,0,1,178,179,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,173,120,116,111,0,1,178,179,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,173,120,116,111,0,1,178,179,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,177,176,173,124,120,116,111,0,1,178,179,
238,237,236,235,234,233,232,231,230,229,228,227,206,205,204,203,202,201,200,
  199,198,195,183,181,177,176,173,124,120,116,111,0,1,178,179,
145,0,5,241,
240,239,235,211,210,209,208,207,205,183,182,181,177,176,175,173,170,145,141,
  140,138,137,124,120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
177,176,124,0,4,44,193,
177,176,124,0,4,44,193,
240,239,235,211,210,209,208,207,205,182,177,176,175,170,145,141,140,138,137,
  124,0,2,3,4,5,6,16,17,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,
  71,72,74,75,76,77,78,79,83,104,108,109,123,126,128,129,130,134,142,193,
  212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
206,0,73,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,128,129,
  130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,128,129,
  130,134,142,193,212,213,214,241,
238,237,236,0,68,105,106,107,
235,205,0,66,70,104,
234,233,0,64,102,103,
232,231,230,229,0,62,98,99,100,101,
228,227,0,60,96,97,
204,0,58,
203,0,56,
202,201,0,52,54,
238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,220,
  219,218,217,216,215,211,208,207,206,205,204,203,202,201,200,199,198,195,
  192,191,183,182,181,177,176,175,173,138,137,124,120,116,111,0,1,178,179,
200,0,50,
182,0,16,
199,0,47,
192,177,176,173,124,120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
177,176,173,124,120,116,111,0,1,178,179,
240,239,235,211,208,207,205,197,196,194,192,191,190,189,188,187,186,185,184,
  183,182,177,176,175,173,170,145,141,140,138,137,124,120,116,111,0,1,178,
  179,
226,225,224,223,222,221,220,219,218,217,216,215,211,208,207,0,45,75,76,80,
  84,85,86,87,88,89,90,91,92,93,94,95,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,173,170,145,141,140,138,137,124,120,116,111,0,1,178,179,
192,177,176,124,0,4,35,193,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
177,176,124,0,4,193,
240,239,235,211,208,207,205,183,182,177,176,175,173,170,145,141,140,138,137,
  124,120,116,111,0,1,178,179,
183,173,120,116,111,0,1,178,179,
183,173,120,116,111,0,1,178,179,
240,239,235,211,208,207,205,197,196,194,192,191,190,189,188,187,186,185,184,
  183,182,177,176,175,170,145,141,140,138,137,124,0,8,
240,239,235,211,208,207,205,197,196,194,192,191,190,189,188,187,186,185,184,
  183,182,181,180,177,176,175,173,170,145,141,140,138,137,124,120,116,111,
  9,0,1,178,179,
182,173,120,116,111,0,1,178,179,
182,173,120,116,111,0,1,178,179,
182,173,120,116,111,0,1,178,179,
183,0,21,
195,183,0,21,38,
195,183,0,21,38,
240,239,235,211,208,207,205,183,182,177,176,175,170,145,141,140,138,137,124,
  0,2,3,4,5,6,16,17,21,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,
  214,241,
183,0,21,
183,0,21,
195,183,0,21,38,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,10,11,12,13,15,
  16,17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,44,
  46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,
182,0,16,
182,0,16,
182,0,16,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,10,11,12,13,15,
  16,17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,44,
  46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,
240,239,238,237,236,235,231,229,215,211,205,204,203,202,199,198,195,192,191,
  183,182,181,177,176,175,174,173,170,168,167,163,162,161,160,159,158,157,
  156,155,154,153,145,138,137,124,0,147,149,150,151,152,164,165,166,
175,138,137,0,127,133,
175,138,0,
176,175,138,137,124,0,169,
175,138,0,
175,138,0,
176,175,138,137,124,0,169,
170,0,
175,138,137,0,127,133,
235,205,175,138,137,0,125,131,
235,205,175,138,137,0,125,131,
211,0,80,
211,0,80,
181,173,120,116,111,0,1,178,179,
181,0,18,
181,173,120,116,111,0,1,178,179,
181,0,18,
195,181,0,18,38,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,128,129,
  130,134,142,193,212,213,214,241,
211,208,207,0,75,76,80,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,128,129,
  130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,65,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,128,
  129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,123,126,
  128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,123,
  126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,123,
  126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,
  109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,
  108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,
  104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,
  83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,181,177,176,175,170,145,141,140,138,137,124,
  0,2,3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,81,82,83,104,108,109,123,126,128,129,130,134,142,193,212,213,
  214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,17,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
177,176,173,124,120,116,111,0,1,178,179,
177,176,124,0,4,193,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
177,176,124,0,4,42,193,
192,177,176,124,0,4,35,42,193,
240,239,235,211,208,207,205,197,196,194,192,191,190,189,188,187,186,185,184,
  183,182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,10,11,12,13,
  15,16,17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,39,40,41,
  43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,
  108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
177,176,124,0,4,193,
195,183,0,21,38,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
184,0,24,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,17,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,183,182,177,176,175,170,145,141,140,138,137,124,
  0,2,3,4,5,6,16,17,20,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,
  214,241,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,17,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
180,0,14,
175,138,137,0,132,
175,138,137,0,132,
240,239,238,237,236,235,234,233,232,231,230,229,228,227,211,208,207,206,205,
  204,203,202,201,200,199,198,197,196,195,194,192,190,189,188,187,186,185,
  184,183,182,181,177,176,175,173,170,145,141,140,138,137,124,120,116,111,
  0,1,178,179,
211,208,207,182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,16,44,
  72,74,75,76,77,83,123,126,128,129,130,134,142,193,212,213,214,241,
211,208,207,182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,16,44,
  72,74,75,76,77,83,123,126,128,129,130,134,142,193,212,213,214,241,
238,237,236,0,68,105,106,107,
235,205,0,66,70,104,
234,233,0,64,102,103,
232,231,230,229,0,62,98,99,100,101,
228,227,0,60,96,97,
204,0,58,
203,0,56,
195,0,38,
181,0,18,
198,195,0,38,48,
195,191,0,36,38,
195,0,38,
177,176,124,0,4,42,193,
240,239,235,211,208,207,205,197,196,194,192,191,190,189,188,187,186,185,184,
  183,182,180,177,176,175,173,170,145,141,140,138,137,124,120,116,111,9,0,
  1,178,179,
182,0,16,
195,181,0,18,38,
195,0,38,
183,0,21,
195,181,0,18,38,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,173,170,145,141,140,138,137,124,120,116,111,0,1,178,179,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,44,46,
  49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,
175,138,137,0,133,
175,138,137,0,133,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,182,177,176,175,173,170,145,141,140,138,137,124,
  120,116,111,0,1,178,179,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,
  77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
177,176,124,0,4,193,
177,176,124,0,4,42,193,
195,191,0,36,38,
240,239,235,211,208,207,205,182,177,176,175,170,145,141,140,138,137,124,0,2,
  3,4,5,6,16,17,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,214,241,
240,239,235,211,208,207,205,183,182,177,176,175,170,145,141,140,138,137,124,
  0,2,3,4,5,6,16,17,20,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,
  214,241,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,44,46,
  49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,
195,0,38,
177,176,124,0,4,42,193,
195,181,0,18,38,
183,0,21,
195,0,38,
183,0,21,
240,239,235,211,208,207,205,182,181,177,176,175,170,145,141,140,138,137,124,
  0,2,3,4,5,6,16,17,20,40,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,83,104,108,109,123,126,128,129,130,134,142,193,212,213,
  214,241,
181,0,18,
240,239,235,211,208,207,205,197,196,194,192,190,189,188,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,124,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,37,39,40,41,43,44,46,
  49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,83,104,108,109,
  123,126,128,129,130,134,142,193,212,213,214,241,

};


static unsigned const char ag_astt[5079] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,1,1,1,8,7,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,
  7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,7,0,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,5,3,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,2,7,
  2,2,1,2,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,
  2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,5,5,
  5,7,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,5,5,5,7,
  2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,5,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,2,2,2,7,2,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,2,2,2,1,1,5,1,2,2,1,1,5,1,2,2,1,1,1,1,5,1,
  2,2,2,2,1,1,5,1,2,2,1,5,1,1,5,1,1,1,5,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,
  1,1,7,1,1,3,1,5,1,1,4,1,1,5,1,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,4,1,2,2,1,2,2,2,2,2,2,2,2,2,2,2,2,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,7,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,
  5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,1,7,3,1,1,7,3,1,1,1,
  7,3,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,1,7,
  2,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,
  2,2,1,2,2,1,1,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,7,
  1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,
  1,2,2,2,2,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,
  2,2,2,2,2,2,2,2,2,2,2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,
  2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,
  7,2,1,1,4,1,1,4,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,1,7,2,
  1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,
  1,1,1,1,1,1,1,4,2,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,
  1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,2,
  2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,1,5,1,1,1,
  7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,2,2,2,1,1,1,
  1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,
  2,2,2,7,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,
  1,7,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  4,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,
  1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,2,2,7,1,2,2,2,7,1,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,2,1,1,2,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,2,2,
  1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,4,1,2,2,1,1,4,1,2,2,1,
  1,1,1,4,1,2,2,2,2,1,1,4,1,2,2,1,4,1,1,4,1,1,5,1,1,7,2,1,1,7,1,1,1,1,7,1,1,
  1,4,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,1,1,7,2,1,1,5,1,1,7,1,1,1,7,1,1,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,2,2,1,1,
  1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,2,10,10,10,4,2,1,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,
  1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,2,2,2,7,2,1,2,2,2,7,2,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,
  7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,
  1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,2,
  2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1,2,2,2,4,2,1,1,1,
  1,7,1,1,1,7,1,1,4,1,1,7,2,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,
  1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  2,7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,4,3,1,2,4,0,3,3,
  4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
352,352,1,2,354,352,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
  129,129,129,129,129,129,129,129,129,129,129,129,129,129,131,
132,6,
124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,
  124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,126,
127,8,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,1,9,90,91,47,167,93,3,3,3,77,76,32,72,75,16,13,
  73,26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,42,
  41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
  170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
  170,170,170,170,170,170,170,170,10,78,
158,158,158,142,158,
79,79,79,12,140,79,
85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,
  85,85,81,80,187,186,185,184,183,182,181,180,179,178,177,85,85,85,85,13,
  85,85,85,84,83,82,85,85,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,3,353,3,1,2,14,3,3,416,
86,87,15,
88,135,
166,165,165,165,166,152,164,
151,151,151,151,151,151,151,151,151,151,151,151,143,151,151,151,151,151,151,
  151,151,151,151,151,151,160,151,160,157,143,151,151,151,18,
150,150,150,150,150,150,150,150,150,150,150,150,141,150,150,150,150,150,150,
  150,150,150,150,150,150,154,150,154,154,141,150,150,150,19,154,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,3,3,1,2,20,3,3,389,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,3,3,1,2,21,3,3,388,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,3,3,1,2,22,3,3,387,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,3,353,3,1,2,23,3,3,383,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,3,353,3,1,2,24,3,3,382,
10,92,168,14,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,
  353,353,353,353,3,1,2,26,3,3,357,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,27,3,3,415,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,28,3,3,414,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,29,3,3,410,
133,133,133,30,83,89,45,
133,133,133,31,83,90,45,
27,28,29,12,91,93,23,24,33,26,133,133,153,13,10,163,163,159,153,133,32,90,
  91,47,167,93,32,95,95,95,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,
  34,74,31,30,34,94,92,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,
  14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,33,3,3,380,
96,71,97,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,35,90,91,47,
  167,93,32,98,70,70,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,36,90,91,47,
  167,93,32,98,69,69,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
99,100,101,64,102,116,117,118,
29,33,62,103,114,115,
104,105,60,106,112,113,
107,108,109,110,58,111,108,109,110,111,
112,113,56,114,106,107,
115,54,116,
117,52,118,
119,121,50,122,120,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,134,134,134,3,134,134,134,3,1,2,45,3,3,368,
123,48,124,
26,83,125,
126,46,127,
353,353,353,3,353,3,1,2,49,3,3,372,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,50,3,3,371,
353,353,3,353,3,1,2,51,3,3,369,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,52,3,3,367,
130,131,132,133,134,135,136,137,138,139,140,141,128,23,24,75,142,78,79,129,
  94,95,96,97,98,99,100,101,102,103,104,105,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,54,3,3,363,
52,133,133,133,55,144,143,45,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,56,90,91,47,
  167,93,32,32,32,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,
  30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
133,133,133,57,30,45,
353,353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,
  353,3,1,2,58,3,3,362,
353,3,3,1,2,59,3,3,361,
353,3,3,1,2,60,3,3,360,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,61,145,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,353,62,
  3,3,358,
353,3,3,1,2,63,3,3,364,
353,3,3,1,2,64,3,3,359,
353,3,3,1,2,65,3,3,365,
62,66,24,
146,62,67,23,147,
146,62,68,22,148,
27,28,29,12,23,24,33,62,26,133,133,153,13,10,163,163,159,153,133,69,90,91,
  47,167,93,32,149,20,149,149,53,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
62,70,19,
62,71,18,
146,62,72,15,150,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,73,90,91,47,167,93,151,151,151,77,76,32,72,75,16,
  13,73,26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,
  42,41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,
  18,17,163,45,22,21,20,14,
26,74,152,
26,75,153,
26,76,154,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,77,90,91,47,167,93,6,6,155,77,76,32,72,75,16,13,
  73,26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,42,
  41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,171,
  171,171,171,171,171,171,171,171,171,81,80,187,186,185,184,183,182,181,
  180,179,178,177,169,171,171,171,78,171,171,171,171,84,83,82,171,
79,79,79,148,149,79,
191,191,80,
197,196,196,196,197,81,194,
193,193,189,
192,192,188,
197,196,196,196,197,176,195,
198,85,
79,79,79,139,138,79,
156,157,157,157,157,87,137,157,
156,157,157,157,157,88,136,157,
128,77,129,
128,76,129,
353,3,3,1,2,91,3,3,385,
158,92,159,
353,3,3,1,2,93,3,3,384,
158,94,160,
146,158,95,73,150,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,96,3,3,381,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,97,90,91,47,
  167,93,32,98,72,72,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
128,23,24,75,78,79,129,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,99,3,3,413,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,100,3,3,412,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,101,3,3,411,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,102,90,91,47,
  167,93,32,98,67,67,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,
  17,163,45,22,21,20,14,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,103,90,91,47,
  167,93,32,98,161,161,161,36,35,34,74,31,30,34,25,119,120,121,16,15,19,
  11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,104,3,3,409,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,105,3,3,408,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,106,90,91,47,
  167,93,32,98,162,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,
  11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,107,3,3,407,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,108,3,3,406,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,109,3,3,405,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,110,3,3,404,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,111,90,91,47,
  167,93,32,98,163,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,
  19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,112,3,3,403,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,113,3,3,402,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,114,90,91,47,
  167,93,32,98,164,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,
  15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,115,3,3,379,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,116,90,91,47,
  167,93,32,98,165,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,
  16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,117,3,3,378,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,118,90,91,47,
  167,93,32,98,166,41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,
  121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,119,3,3,377,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,120,90,91,47,
  167,93,32,98,167,42,41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,
  120,121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,121,3,3,376,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,122,90,91,47,
  167,93,32,98,51,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,
  119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,123,3,3,375,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,124,90,91,47,
  167,93,32,98,49,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,30,34,
  25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
27,28,29,12,23,24,33,26,86,133,133,153,13,10,163,163,159,153,133,125,90,91,
  47,167,93,32,88,88,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,
  31,30,34,169,168,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,126,3,3,374,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,127,90,91,47,
  167,93,32,170,170,170,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,3,353,3,1,2,128,3,3,386,
133,133,133,129,84,45,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,130,3,3,401,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,131,3,3,400,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,132,3,3,399,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,133,3,3,398,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,134,3,3,397,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,135,3,3,396,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,136,3,3,395,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,137,3,3,394,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,138,3,3,393,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,139,3,3,392,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,140,3,3,391,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,141,3,3,390,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,142,90,91,47,
  167,93,32,45,45,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,
  30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
133,133,133,143,38,171,45,
52,133,133,133,144,38,173,172,45,
27,28,29,12,23,24,33,49,50,51,52,174,65,63,54,58,59,60,64,62,26,133,133,153,
  13,10,163,163,159,153,133,145,90,91,47,167,93,3,3,3,77,76,32,72,75,16,
  13,73,26,13,71,70,69,68,67,66,25,27,74,61,29,57,56,72,55,72,53,48,46,44,
  43,42,41,40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,
  11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,146,3,3,370,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,147,90,91,47,
  167,93,32,33,33,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,
  30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
133,133,133,148,31,45,
146,62,149,21,150,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,150,90,91,47,
  167,93,32,43,43,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,
  30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
64,151,175,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,152,90,91,47,
  167,93,32,176,176,176,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
27,28,29,12,23,24,33,40,26,133,133,153,13,10,163,163,159,153,133,153,90,91,
  47,167,93,32,177,178,177,177,53,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,154,90,91,47,
  167,93,32,179,179,179,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
180,5,181,
155,155,155,156,182,
155,155,155,157,183,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,158,3,3,
  356,
12,23,24,26,133,133,153,13,10,163,163,159,153,133,159,90,91,47,167,93,32,98,
  82,74,31,30,82,25,16,15,19,11,18,17,163,45,22,21,20,14,
12,23,24,26,133,133,153,13,10,163,163,159,153,133,160,90,91,47,167,93,32,98,
  81,74,31,30,81,25,16,15,19,11,18,17,163,45,22,21,20,14,
99,100,101,65,102,116,117,118,
29,33,63,103,114,115,
104,105,61,106,112,113,
107,108,109,110,59,111,108,109,110,111,
112,113,57,114,106,107,
115,55,116,
117,53,118,
146,87,184,
158,169,85,
185,146,170,150,186,
146,174,171,188,187,
146,37,187,
133,133,133,173,38,189,45,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,353,174,3,3,
  366,
26,175,190,
146,158,176,28,150,
146,41,150,
62,178,191,
146,158,179,192,150,
353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,353,
  353,353,353,353,3,353,353,353,353,353,353,353,3,1,2,180,3,3,355,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,181,90,91,47,167,93,7,10,77,76,32,72,75,16,13,73,
  26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,42,41,
  40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,
  163,45,22,21,20,14,
156,156,156,147,156,
156,156,156,146,156,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,184,90,91,47,
  167,93,32,89,89,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,
  30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
353,353,353,353,353,353,353,353,353,353,353,3,353,353,353,353,353,353,353,3,
  1,2,185,3,3,373,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,186,90,91,47,
  167,93,32,47,98,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,74,31,30,
  34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
133,133,133,187,39,45,
133,133,133,188,38,193,45,
146,174,189,194,187,
27,28,29,12,23,24,33,26,133,133,153,13,10,163,163,159,153,133,190,90,91,47,
  167,93,32,195,195,195,53,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
27,28,29,12,23,24,33,40,26,133,133,153,13,10,163,163,159,153,133,191,90,91,
  47,167,93,32,177,196,177,177,53,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,192,90,91,47,167,93,8,11,77,76,32,72,75,16,13,73,
  26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,42,41,
  40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,
  163,45,22,21,20,14,
146,36,187,
133,133,133,35,38,197,45,
146,158,195,198,150,
62,196,199,
146,34,187,
62,198,14,
27,28,29,12,23,24,33,26,40,133,133,153,13,10,163,163,159,153,133,199,90,91,
  47,167,93,32,177,200,177,177,53,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,163,45,22,21,20,14,
158,200,201,
27,28,29,12,23,24,33,49,50,51,52,65,63,54,58,59,60,64,62,26,133,133,153,13,
  10,163,163,159,153,133,201,90,91,47,167,93,9,12,77,76,32,72,75,16,13,73,
  26,13,71,70,69,68,67,66,25,27,74,61,57,56,72,55,72,53,48,46,44,43,42,41,
  40,39,38,37,37,37,36,35,34,74,31,30,34,25,119,120,121,16,15,19,11,18,17,
  163,45,22,21,20,14,

};


static const unsigned short ag_sbt[] = {
     0,  39,  76, 114, 120, 154, 188, 190, 225, 227, 328, 375, 380, 386,
   439, 472, 475, 477, 484, 518, 553, 585, 617, 649, 684, 719, 723, 753,
   779, 805, 831, 838, 845, 913, 939, 942, 993,1044,1052,1058,1064,1074,
  1080,1083,1086,1091,1147,1150,1153,1156,1168,1194,1205,1244,1276,1314,
  1322,1385,1391,1418,1427,1436,1469,1511,1520,1529,1538,1541,1546,1551,
  1617,1620,1623,1628,1728,1731,1734,1737,1837,1891,1897,1900,1907,1910,
  1913,1920,1922,1928,1936,1944,1947,1950,1959,1962,1971,1974,1979,2005,
  2056,2063,2089,2115,2141,2192,2244,2270,2296,2349,2375,2401,2427,2453,
  2507,2533,2559,2614,2640,2696,2722,2779,2805,2863,2889,2949,2975,3036,
  3102,3128,3192,3203,3209,3235,3261,3287,3313,3339,3365,3391,3417,3443,
  3469,3495,3521,3584,3591,3600,3702,3728,3791,3797,3802,3865,3868,3932,
  3998,4062,4065,4070,4075,4134,4174,4214,4222,4228,4234,4244,4250,4253,
  4256,4259,4262,4267,4272,4275,4282,4323,4326,4331,4334,4337,4342,4380,
  4479,4484,4489,4552,4578,4640,4646,4653,4658,4722,4788,4887,4890,4897,
  4902,4905,4908,4911,4977,4980,5079
};


static const unsigned short ag_sbe[] = {
    35,  73, 111, 118, 151, 187, 189, 224, 226, 258, 373, 378, 383, 430,
   468, 474, 476, 482, 517, 551, 581, 613, 645, 680, 715, 720, 749, 775,
   801, 827, 834, 841, 865, 935, 940, 960,1011,1047,1054,1060,1068,1076,
  1081,1084,1088,1143,1148,1151,1154,1164,1190,1201,1240,1259,1310,1318,
  1340,1388,1414,1423,1432,1467,1507,1516,1525,1534,1539,1543,1548,1570,
  1618,1621,1625,1658,1729,1732,1735,1767,1882,1894,1899,1905,1909,1912,
  1918,1921,1925,1933,1941,1945,1948,1955,1960,1967,1972,1976,2001,2023,
  2059,2085,2111,2137,2159,2210,2266,2292,2314,2371,2397,2423,2449,2471,
  2529,2555,2577,2636,2658,2718,2740,2801,2823,2885,2907,2971,2993,3055,
  3124,3146,3199,3206,3231,3257,3283,3309,3335,3361,3387,3413,3439,3465,
  3491,3517,3539,3587,3595,3631,3724,3746,3794,3799,3820,3866,3886,3951,
  4016,4063,4068,4073,4130,4148,4188,4217,4224,4230,4238,4246,4251,4254,
  4257,4260,4264,4269,4273,4278,4319,4324,4328,4332,4335,4339,4376,4410,
  4482,4487,4507,4574,4596,4643,4649,4655,4676,4741,4818,4888,4893,4899,
  4903,4906,4909,4930,4978,5010,5079
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,5,9,4,5,9,1,7,2,1,1,2,2,2,3,2,2,2,1,1,1,4,3,2,3,2,3,6,
  5,5,3,1,3,0,1,1,3,1,3,1,5,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,2,
  2,1,3,3,1,1,2,2,2,2,1,4,4,1,3,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,
  2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 22, 22, 22, 22,
   22, 22, 22, 22, 22, 22, 22, 23, 15, 19, 13, 25, 29, 29, 30, 30, 31, 31,
   31, 31, 42, 42, 20, 20, 17, 17, 40, 40, 43, 43, 46, 46, 49, 49, 51, 51,
   53, 53, 55, 55, 57, 57, 59, 59, 61, 61, 63, 63, 65, 65, 67, 67, 67, 69,
   69, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 44, 44, 77, 81, 81, 82, 82,
   74, 74, 74, 74, 45, 45, 45, 45, 45, 45, 45, 45, 45, 45, 45, 45, 60, 60,
   62, 62, 62, 62, 64, 64, 66, 66, 68, 68, 68, 71, 71, 71,  1,113,113,114,
  114,  1,118,118,119,119,  1,193,193,213,213,213,123,123,123,126,126,126,
  131,131,125,125,127,127,212,212,212,128,128,132,132,129,129,130,130,142,
  142,134,134,143,143, 83, 83,241,146,146,147,147,149,149,149,150,150,150,
  150,150,150,150,150,150,150,150,151,151,151,164,165,166,152,152,169,169,
  214,171,171,133,110,110,112,112,112,112,112,112,112,112,112,112,112,112,
  112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,
  112,112,112,112,117,117,117,117,117,117,117,117,117,117,117,117,117,117,
  117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,
  117,121,121,121,122,122,122,122,122,122,135,135,136,136,136,139,139,144,
  144,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
  148,148,148,148,148,148,148,148,148,148,148,148,148,148,172,172,172,172,
  172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
  172,172,172,172,172,172,172,172,172,178,178,179,179, 14, 18, 16, 21, 24,
   26, 27, 28, 32, 33, 34, 36, 35,  4, 37, 38, 39, 41, 48, 47, 50, 52, 54,
   56, 58, 70, 73, 75, 76, 78, 79, 80,  2,  3,  6, 84, 85, 86, 87, 88, 89,
   90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,104,105,106,107,
  108,109,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_6 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 13: VRO(AG_WRAP_6 *, ag_rp_13(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER)); break;
    case 15: VNO AG_WRAP_6(ag_rp_15(PCB_POINTER)); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: ag_rp_19(PCB_POINTER); break;
    case 20: ag_rp_20(PCB_POINTER); break;
    case 21: ag_rp_21(PCB_POINTER); break;
    case 22: VNO AG_WRAP_6(ag_rp_22(PCB_POINTER, VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 23: VNO AG_WRAP_6(ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: VNO AG_WRAP_6(ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 28: VNO AG_WRAP_6(ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *), VW(5, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); VWD(5, AG_WRAP_9 *); break;
    case 29: VNO AG_WRAP_6(ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); break;
    case 30: VNO AG_WRAP_6(ag_rp_30(PCB_POINTER, VW(2, AG_WRAP_9 *), VW(4, AG_WRAP_9 *))); 
            VWD(2, AG_WRAP_9 *); VWD(4, AG_WRAP_9 *); break;
    case 31: VNO AG_WRAP_6(ag_rp_31(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_9 *); break;
    case 32: VRO(AG_WRAP_4 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 33: VRO(AG_WRAP_9 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_9 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 34: VNO AG_WRAP_6(ag_rp_34(PCB_POINTER)); break;
    case 35: VRO(AG_WRAP_6 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 36: VRO(AG_WRAP_6 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 37: VRO(AG_WRAP_6 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 38: VRO(AG_WRAP_6 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VRO(AG_WRAP_6 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 40: VRO(AG_WRAP_6 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 42: VRO(AG_WRAP_6 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 43: VRO(AG_WRAP_6 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 44: VRO(AG_WRAP_6 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 45: VRO(AG_WRAP_6 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 46: VRO(AG_WRAP_6 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 47: VRO(AG_WRAP_6 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 48: VNO AG_WRAP_6(ag_rp_48(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 49: VNO AG_WRAP_6(ag_rp_49(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 50: VRO(AG_WRAP_6 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 51: VNO AG_WRAP_6(ag_rp_51(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 52: VRO(AG_WRAP_8 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 53: VRO(AG_WRAP_6 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 54: VNO AG_WRAP_6(ag_rp_54(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 55: VNO AG_WRAP_6(ag_rp_55(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 56: VRO(AG_WRAP_6 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 57: VRO(AG_WRAP_6 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 58: VNO AG_WRAP_6(ag_rp_58(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 59: VNO AG_WRAP_6(ag_rp_59(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 60: VRO(AG_WRAP_4 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 61: VRO(AG_WRAP_6 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 62: VRO(AG_WRAP_4 *, ag_rp_62(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 63: VNO AG_WRAP_7(ag_rp_63(PCB_POINTER)); break;
    case 64: VRO(AG_WRAP_6 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 65: VRO(AG_WRAP_7 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 66: VNO AG_WRAP_8(ag_rp_66(PCB_POINTER, V(0,(long *)))); break;
    case 67: VNO AG_WRAP_8(ag_rp_67(PCB_POINTER, V(0,(double *)))); break;
    case 68: VNO AG_WRAP_8(ag_rp_68(PCB_POINTER, V(0,(int *)))); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: V(0,(Opcode *)) = ag_rp_78(PCB_POINTER); break;
    case 79: V(0,(Opcode *)) = ag_rp_79(PCB_POINTER); break;
    case 80: V(0,(Opcode *)) = ag_rp_80(PCB_POINTER); break;
    case 81: V(0,(Opcode *)) = ag_rp_81(PCB_POINTER); break;
    case 82: V(0,(Opcode *)) = ag_rp_82(PCB_POINTER); break;
    case 83: V(0,(Opcode *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(Opcode *)) = ag_rp_84(PCB_POINTER); break;
    case 85: V(0,(Opcode *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(Opcode *)) = ag_rp_86(PCB_POINTER); break;
    case 87: V(0,(Opcode *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(Opcode *)) = ag_rp_88(PCB_POINTER); break;
    case 89: V(0,(Opcode *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(Opcode *)) = ag_rp_90(PCB_POINTER); break;
    case 91: V(0,(Opcode *)) = ag_rp_91(PCB_POINTER); break;
    case 92: V(0,(Opcode *)) = ag_rp_92(PCB_POINTER); break;
    case 93: V(0,(Opcode *)) = ag_rp_93(PCB_POINTER); break;
    case 94: V(0,(Opcode *)) = ag_rp_94(PCB_POINTER); break;
    case 95: V(0,(Opcode *)) = ag_rp_95(PCB_POINTER); break;
    case 96: V(0,(Opcode *)) = ag_rp_96(PCB_POINTER); break;
    case 97: VNO AG_WRAP_4(ag_rp_97(PCB_POINTER, V(0,(int *)))); break;
    case 98: VRO(AG_WRAP_4 *, ag_rp_98(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 99: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 100: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 101: V(0,(double *)) = ag_rp_101(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 102: V(0,(double *)) = ag_rp_102(PCB_POINTER, V(0,(double *))); break;
    case 103: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(1,(double *))); break;
    case 104: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(long *))); break;
    case 105: V(0,(double *)) = ag_rp_105(PCB_POINTER, V(0,(long *))); break;
    case 106: V(0,(double *)) = ag_rp_106(PCB_POINTER, V(0,(long *))); break;
    case 107: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(1,(long *))); break;
    case 108: V(0,(long *)) = ag_rp_108(PCB_POINTER, V(1,(long *))); break;
    case 109: V(0,(double *)) = ag_rp_109(PCB_POINTER, V(0,(int *))); break;
    case 110: V(0,(double *)) = ag_rp_110(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 111: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(int *))); break;
    case 112: V(0,(long *)) = ag_rp_112(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 113: V(0,(long *)) = ag_rp_113(PCB_POINTER, V(0,(int *))); break;
    case 114: V(0,(long *)) = ag_rp_114(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 115: V(0,(long *)) = ag_rp_115(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 116: V(0,(long *)) = ag_rp_116(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 117: V(0,(long *)) = ag_rp_117(PCB_POINTER); break;
    case 118: V(0,(long *)) = ag_rp_118(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 119: V(0,(long *)) = ag_rp_119(PCB_POINTER); break;
    case 120: V(0,(long *)) = ag_rp_120(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 121: V(0,(int *)) = ag_rp_121(PCB_POINTER, V(0,(int *))); break;
    case 122: V(0,(int *)) = ag_rp_122(PCB_POINTER, V(0,(int *))); break;
    case 123: VRO(AG_WRAP_5 *, ag_rp_123(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 124: VRO(AG_WRAP_8 *, ag_rp_124(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 125: VNO AG_WRAP_5(ag_rp_125(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 126: VNO AG_WRAP_4(ag_rp_126(PCB_POINTER)); break;
    case 127: VRO(AG_WRAP_4 *, ag_rp_127(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 128: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_132(PCB_POINTER); break;
    case 133: V(0,(int *)) = ag_rp_133(PCB_POINTER); break;
    case 134: V(0,(int *)) = ag_rp_134(PCB_POINTER); break;
    case 135: V(0,(int *)) = ag_rp_135(PCB_POINTER); break;
    case 136: V(0,(int *)) = ag_rp_136(PCB_POINTER); break;
    case 137: V(0,(int *)) = ag_rp_137(PCB_POINTER); break;
    case 138: V(0,(int *)) = ag_rp_138(PCB_POINTER); break;
    case 139: V(0,(int *)) = ag_rp_139(PCB_POINTER, V(1,(int *))); break;
    case 140: V(0,(int *)) = ag_rp_140(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 141: V(0,(int *)) = ag_rp_141(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 142: V(0,(int *)) = ag_rp_142(PCB_POINTER, V(1,(int *))); break;
    case 143: V(0,(int *)) = ag_rp_143(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 144: V(0,(int *)) = ag_rp_144(PCB_POINTER, V(0,(int *))); break;
    case 145: V(0,(int *)) = ag_rp_145(PCB_POINTER, V(0,(int *))); break;
    case 146: V(0,(int *)) = ag_rp_146(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[242] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"else\"",
  "WHILE",
  "'('",
  "expression",
  "')'",
  "FOR",
  "optional expression",
  "';'",
  "simple statement",
  "DO",
  "\"while\"",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "structure declaration",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "\"struct\"",
  "name list",
  "conditional expression",
  "lvalue",
  "assignment op",
  "logical or expression",
  "'\\?'",
  "':'",
  "logical and expression",
  "\"||\"",
  "inclusive or expression",
  "\"&&\"",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "equality op",
  "shift expression",
  "relational op",
  "additive expression",
  "shift op",
  "multiplicative expression",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"else\"",
  "')'",
  "'('",
  "';'",
  "\"while\"",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'}'",
  "'{'",
  "name",
  "\"dump\"",
  "','",
  "\"print\"",
  "\"struct\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "'+'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


