/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.
 Modified to support structures and instances.

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In cll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146229
#include "dxi.h"
#endif

#ifndef DXI_H_1031146229
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 613, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    loopActive(0),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

void dxi_pcb_struct::forLoop
    (const FileLocation &condLoc, const FileLocation &incLoc)
{
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.loopActive++;          // Mark the parser as being in an active loop
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    if (loopPCB.mode == executeMode) {
      loopPCB.parseSyntax("forField", condLoc);    // Evaluate condition field
      Value result = dxi_value(&loopPCB);
      if (result.isDefined() && result.isFalse()) loopPCB.mode = breakMode;
    }
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // Mode could have been set to continueMode
    loopPCB.parseSyntax("forField", incLoc);
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

void dxi_pcb_struct::interruptLoop(Mode desiredMode) {
  if (mode != executeMode) return;
  if (loopActive == 0) reportError("No loop active");
  mode = desiredMode;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

Value dxi_pcb_struct::powm(Value &lvalue, const Value &exp) const {
  if (mode != executeMode) return Value();
  Value base = lvalue;
  lvalue = pow(base.deref(), exp);
  return lvalue.deref();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  return pcb.returnValue;
}

// declare structure
void dxi_pcb_struct::defineStructure (
    const AgString &structureName,
    const AgStack<AgString> &fieldNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  Value structure(fieldDictionary);
  dataset.value(structureName) = structure;
}

// define instances of named structure
void dxi_pcb_struct::defineInstances (
    const AgString &structureName,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  const AgDictionary<AgString> &fieldDictionary = dataset.value(structureName).getDictionary();
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}

// define instances of an anonymous structure
void dxi_pcb_struct::defineInstances (
    const AgStack<AgString> &fieldNames,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return Value();
}

static inline Value ag_rp_5(PCB_DECL, Value &x) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x;
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 200, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 203, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 206, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 209, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 212, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 235, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 237, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 241, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_14(PCB_DECL) {
/* Line 255, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.simpleLoop("doLoop");
}

static inline void ag_rp_15(PCB_DECL) {
/* Line 256, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_16(PCB_DECL, FileLocation c, FileLocation i) {
/* Line 257, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.forLoop(c, i);
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 261, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.interruptLoop(breakMode);
}

static inline void ag_rp_18(PCB_DECL) {
/* Line 262, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.interruptLoop(continueMode);
}

static inline void ag_rp_19(PCB_DECL, Value &x) {
/* Line 263, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_20(PCB_DECL) {
/* Line 278, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static void ag_rp_21(PCB_DECL, Value &x) {
/* Line 283, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static inline void ag_rp_22(PCB_DECL) {
/* Line 286, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.mode = scanMode;
}

static inline FileLocation ag_rp_23(PCB_DECL) {
/* Line 289, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return CONTEXT;
}

static void ag_rp_24(PCB_DECL, Value &x) {
/* Line 294, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_25(PCB_DECL) {
/* Line 298, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_26(PCB_DECL, AgString &n) {
/* Line 305, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_27(PCB_DECL, AgString &n) {
/* Line 306, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_28(PCB_DECL, Value &x) {
/* Line 310, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_29(PCB_DECL, Value &x) {
/* Line 311, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.print(x);
}

static void ag_rp_30(PCB_DECL, AgString &name, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 315, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
      PCB.defineStructure(name, fields);
      PCB.defineInstances(name, instances);
    
}

static inline void ag_rp_31(PCB_DECL, AgString &name, AgStack<AgString> &fields) {
/* Line 319, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.defineStructure(name, fields);
}

static inline void ag_rp_32(PCB_DECL, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 321, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.defineInstances(fields, instances);
}

static inline void ag_rp_33(PCB_DECL, AgString &name, AgStack<AgString> &instances) {
/* Line 323, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  PCB.defineInstances(name, instances);
}

static inline AgStack<AgString> ag_rp_34(PCB_DECL, AgString &n) {
/* Line 326, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_35(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 327, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return stack.push(n);
}

static inline Value ag_rp_36(PCB_DECL) {
/* Line 330, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return Value();
}

static inline Value ag_rp_37(PCB_DECL, Value &x) {
/* Line 336, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x;
}

static inline Value ag_rp_38(PCB_DECL, Value &lv, Value &x) {
/* Line 340, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv = x);
}

static inline Value ag_rp_39(PCB_DECL, Value &lv, Value &x) {
/* Line 341, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv += x);
}

static inline Value ag_rp_40(PCB_DECL, Value &lv, Value &x) {
/* Line 342, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv -= x);
}

static inline Value ag_rp_41(PCB_DECL, Value &lv, Value &x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv *= x);
}

static inline Value ag_rp_42(PCB_DECL, Value &lv, Value &x) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv /= x);
}

static inline Value ag_rp_43(PCB_DECL, Value &lv, Value &x) {
/* Line 345, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv %= x);
}

static inline Value ag_rp_44(PCB_DECL, Value &lv, Value &x) {
/* Line 346, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv |= x);
}

static inline Value ag_rp_45(PCB_DECL, Value &lv, Value &x) {
/* Line 347, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv ^= x);
}

static inline Value ag_rp_46(PCB_DECL, Value &lv, Value &x) {
/* Line 348, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv &= x);
}

static inline Value ag_rp_47(PCB_DECL, Value &lv, Value &x) {
/* Line 349, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv <<= x);
}

static inline Value ag_rp_48(PCB_DECL, Value &lv, Value &x) {
/* Line 350, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv >>= x);
}

static inline Value ag_rp_49(PCB_DECL, Value &lv, Value &x) {
/* Line 351, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return PCB.powm(lv,x);
}

static void ag_rp_50(PCB_DECL, Value &c) {
/* Line 357, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (PCB.mode == executeMode && c.isFalse())  PCB.mode = elseMode;
}

static void ag_rp_51(PCB_DECL, Value &c, Value &x) {
/* Line 359, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (!PCB.changeMode(elseMode, executeMode))
         PCB.changeMode(executeMode, blockMode);
}

static inline Value ag_rp_52(PCB_DECL, Value &c, Value &x, Value &y) {
/* Line 362, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return PCB.mode = CONTEXT.mode, VALUE(c.isTrue() ? x : y);
}

static void ag_rp_53(PCB_DECL, Value &x) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (PCB.mode == executeMode && x.isTrue()) PCB.mode = blockMode;
}

static inline Value ag_rp_54(PCB_DECL, Value &x, Value &y) {
/* Line 369, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? x : y);
}

static void ag_rp_55(PCB_DECL, Value &x) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = blockMode;
}

static inline Value ag_rp_56(PCB_DECL, Value &x, Value &y) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return PCB.restoreMode(), VALUE(x.isTrue() ? y : x);
}

static inline Value ag_rp_57(PCB_DECL, Value &x, Value &y) {
/* Line 380, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x |= y);
}

static inline Value ag_rp_58(PCB_DECL, Value &x, Value &y) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x ^= y);
}

static inline Value ag_rp_59(PCB_DECL, Value &x, Value &y) {
/* Line 388, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x &= y);
}

static inline Value ag_rp_60(PCB_DECL, Value &x, Value &y) {
/* Line 392, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_61(PCB_DECL, Value &x, Value &y) {
/* Line 393, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_62(PCB_DECL, Value &x, Value &y) {
/* Line 397, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_63(PCB_DECL, Value &x, Value &y) {
/* Line 398, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_64(PCB_DECL, Value &x, Value &y) {
/* Line 399, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_65(PCB_DECL, Value &x, Value &y) {
/* Line 400, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_66(PCB_DECL, Value &x, Value &y) {
/* Line 404, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x <<= y);
}

static inline Value ag_rp_67(PCB_DECL, Value &x, Value &y) {
/* Line 405, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x >>= y);
}

static inline Value ag_rp_68(PCB_DECL, Value &x, Value &y) {
/* Line 409, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x += y);
}

static inline Value ag_rp_69(PCB_DECL, Value &x, Value &y) {
/* Line 410, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x -= y);
}

static inline Value ag_rp_70(PCB_DECL, Value &x, Value &y) {
/* Line 414, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x *= y);
}

static inline Value ag_rp_71(PCB_DECL, Value &x, Value &y) {
/* Line 415, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x /= y);
}

static inline Value ag_rp_72(PCB_DECL, Value &x, Value &y) {
/* Line 416, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x %= y);
}

static inline Value ag_rp_73(PCB_DECL, Value &x) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_74(PCB_DECL, Value &x) {
/* Line 421, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_75(PCB_DECL, Value &x) {
/* Line 422, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_76(PCB_DECL, Value &x) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(~x);
}

static inline Value ag_rp_77(PCB_DECL, Value &x, Value &y) {
/* Line 431, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_78(PCB_DECL, Value &x) {
/* Line 439, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x;
}

static inline Value ag_rp_79(PCB_DECL, Value &lv) {
/* Line 441, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_80(PCB_DECL, Value &lv) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(++lv);
}

static inline Value ag_rp_81(PCB_DECL, Value &lv) {
/* Line 443, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(--lv);
}

static inline Value ag_rp_82(PCB_DECL, Value &lv) {
/* Line 444, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv++);
}

static inline Value ag_rp_83(PCB_DECL, Value &lv) {
/* Line 445, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(lv--);
}

static inline Value ag_rp_84(PCB_DECL, Value &x) {
/* Line 447, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_85(PCB_DECL, Value &x) {
/* Line 448, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_86(PCB_DECL, AgString &n) {
/* Line 451, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_87(PCB_DECL, Value &lv, AgString &n) {
/* Line 452, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(&lv.getMember(n));
}

static inline Value ag_rp_88(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 455, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(callFunction(n, args));
}

static inline AgStack<Value> ag_rp_89(PCB_DECL) {
/* Line 458, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_90(PCB_DECL, Value &x) {
/* Line 462, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_91(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 463, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_92(PCB_DECL, long x) {
/* Line 466, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_93(PCB_DECL, double x) {
/* Line 467, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_94(PCB_DECL, int x) {
/* Line 469, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_95(PCB_DECL, int c) {
/* Line 487, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_96(PCB_DECL, AgString &ns, int c) {
/* Line 488, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return ns.append(c);
}

static inline double ag_rp_97(PCB_DECL, double x, long e) {
/* Line 493, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_98(PCB_DECL, double x, long e) {
/* Line 494, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_99(PCB_DECL, double i, double f) {
/* Line 497, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return i+f;
}

static inline double ag_rp_100(PCB_DECL, double i) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return i;
}

static inline double ag_rp_101(PCB_DECL, double f) {
/* Line 499, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return f;
}

static inline double ag_rp_102(PCB_DECL, long x) {
/* Line 502, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x;
}

static inline double ag_rp_103(PCB_DECL, long x) {
/* Line 503, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x;
}

static inline double ag_rp_104(PCB_DECL, long x) {
/* Line 504, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_105(PCB_DECL, long x) {
/* Line 507, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return (long)x;
}

static inline long ag_rp_106(PCB_DECL, long x) {
/* Line 508, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_107(PCB_DECL, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_108(PCB_DECL, int d, double f) {
/* Line 512, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_109(PCB_DECL, int d) {
/* Line 520, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return d-'0';
}

static inline long ag_rp_110(PCB_DECL, long x, int d) {
/* Line 521, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_111(PCB_DECL, int d) {
/* Line 524, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return d-'0';
}

static inline long ag_rp_112(PCB_DECL, long x, int d) {
/* Line 525, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_113(PCB_DECL, long x, int d) {
/* Line 528, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_114(PCB_DECL, long x, int d) {
/* Line 529, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_115(PCB_DECL) {
/* Line 532, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 0;
}

static inline long ag_rp_116(PCB_DECL, long n, int d) {
/* Line 533, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_117(PCB_DECL) {
/* Line 536, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 0;
}

static inline long ag_rp_118(PCB_DECL, long n, int d) {
/* Line 537, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_119(PCB_DECL, int x) {
/* Line 540, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return x - '0';
}

static inline int ag_rp_120(PCB_DECL, int d) {
/* Line 541, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_121(PCB_DECL, Value &s, Value &e) {
/* Line 547, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_122(PCB_DECL, AgString &b) {
/* Line 550, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_123(PCB_DECL) {
/* Line 553, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_124(PCB_DECL, AgString &s, int c) {
/* Line 554, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_125(PCB_DECL) {
/* Line 566, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\'';
}

static inline int ag_rp_126(PCB_DECL) {
/* Line 567, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '"';
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 568, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\?';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 569, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\\';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 570, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\a';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 571, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\b';
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 572, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\f';
}

static inline int ag_rp_132(PCB_DECL) {
/* Line 573, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\n';
}

static inline int ag_rp_133(PCB_DECL) {
/* Line 574, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\r';
}

static inline int ag_rp_134(PCB_DECL) {
/* Line 575, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\t';
}

static inline int ag_rp_135(PCB_DECL) {
/* Line 576, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return '\v';
}

static inline int ag_rp_136(PCB_DECL, int d) {
/* Line 582, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return d-'0';
}

static inline int ag_rp_137(PCB_DECL, int n, int d) {
/* Line 585, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_138(PCB_DECL, int n, int d) {
/* Line 588, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_139(PCB_DECL, int d) {
/* Line 591, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return d;
}

static inline int ag_rp_140(PCB_DECL, int n, int d) {
/* Line 592, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_141(PCB_DECL, int d) {
/* Line 595, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return d-'0';
}

static inline int ag_rp_142(PCB_DECL, int d) {
/* Line 596, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_143(PCB_DECL, int c) {
/* Line 605, W:/agdev/agex/packages/xidek/cll/struct/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0,
  5, 5, 5, 5, 5, 5, 5, 5, 4, 5, 4, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0,
  4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 5, 0, 0, 5, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0,
  0, 5, 5, 0, 0, 0, 0, 5, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
  5, 5, 0, 6, 6, 0, 0, 7, 7, 0, 0, 5, 0, 5, 0, 0, 0, 0, 5, 0, 5, 5, 0,
  0, 7, 0, 0, 0, 7, 0, 0, 7
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  6,  7,  0,  0,  0,  0,  8,  9, 10,
    0, 11, 12, 13, 14,  0,  0, 15, 16, 17,  0,  0,  0,  0,  0,  0, 18, 19,
   20,  0, 14, 21, 22, 23,  0, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34,
    0,  0, 35,  0, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,  0, 48,
   49, 50,  0, 51, 52,  0, 53, 54,  0, 55,  0, 56,  0, 57,  0, 58, 59,  0,
   60, 61, 62, 63,  0, 64, 65,  0, 66, 67,  0, 68, 69, 70,  0, 71, 72, 73,
   74,  0, 75, 76,  0, 77, 78, 79, 80, 81,  0, 82, 83, 84, 85, 86, 87,  0,
   88, 89, 90, 91,  0, 92,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 93,
   94,  0, 95, 96, 97, 98, 99,100,101,102,  0,  0,103,104,105,106,  0,  0,
    0,107,108,109,110,111,112,113,114,  0,  0,115,116,117,118,  0,119,120,
  121,122,  0,  0,  0,  0,  0,123,124,125,126,127,128,129,130,131,132,133,
    0,  0,  0,134,135,136,137,138,139,140,141
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0, 20,  0, 21,  0, 22,  0, 23,  0, 24,  0,201,  0,202,  0,203,
  0,204,  0,205,  0,206,  0,208,  0,209,  0,213,  0,215,  0,216,
  0,254,  0,255,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 99,116,255, 47,100,102,115,119,255, 42,255, 38, 61,255,
   61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47, 61,255, 88,120,255,
   61,255, 60, 61,255, 61,255, 61, 62,255, 76,117,255,111,117,255, 70,255,
  114,255,111,255, 97,114,255, 99,116,255, 76,255,101,255,108,255,105,255,
  104,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 48, 60, 61, 62, 94, 98,
   99,100,101,102,105,108,112,114,115,119,124,255, 99,116,255,100,102,115,
  119,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,255, 42, 47,255, 88,120,
  255,111,117,255, 43, 45, 47, 48, 98, 99,100,102,105,112,114,115,119,255,
   42, 47,255, 47,255, 88,120,255, 43, 45, 48,255, 88,120,255,111,117,255,
   43, 45, 48, 98, 99,100,102,105,112,114,115,119,255, 34, 39, 63, 92, 97,
   98,102,110,114,116,118,120,255, 92,255, 42, 47,255, 60, 61,255, 61, 62,
  255, 33, 38, 42, 47, 60, 61, 62,124,255, 60, 61,255, 61, 62,255, 33, 38,
   42, 60, 61, 62,124,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,100,108,
  255, 88,120,255, 43, 45, 48,100,108,255, 60, 61,255, 61, 62,255, 33, 38,
   60, 61, 62,124,255, 33, 38, 60, 61, 62,124,255, 33, 38, 61,124,255, 38,
  124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255, 42, 47,
   61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38,
   42, 43, 45, 47, 60, 61, 62, 94,124,255, 38, 61,255, 61,255, 42, 61,255,
   43, 61,255, 45, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255, 61,124,
  255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255,124,255, 42, 47,
  255, 88,120,255,111,117,255, 43, 45, 47, 48, 98, 99,100,101,102,105,112,
  114,115,119,255,119,255, 60, 61,255, 61, 62,255, 33, 38, 42, 43, 45, 60,
   61, 62,124,255, 42, 47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,
  255, 33, 38, 42, 43, 45, 47, 48, 60, 61, 62, 98, 99,100,101,102,105,112,
  114,115,119,124,255, 88,120,255,111,117,255, 43, 45, 48, 98, 99,100,101,
  102,105,112,114,115,119,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,7,2,7,4,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,0,
  4,0,4,1,0,4,0,4,0,1,4,7,7,4,6,7,4,7,4,6,4,2,4,7,7,4,7,2,4,7,4,6,4,2,4,
  2,4,2,4,0,0,4,3,3,2,2,2,2,2,2,2,3,2,3,7,7,2,7,2,7,7,7,7,2,2,2,4,7,7,4,
  7,7,2,7,4,0,0,4,0,0,4,3,3,2,2,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,
  7,7,7,4,0,0,4,2,4,0,0,4,3,3,2,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,7,4,
  0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,3,4,0,0,
  4,0,0,4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,3,3,2,7,7,4,0,
  0,4,0,0,4,3,3,2,3,2,3,4,3,3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,
  0,0,4,0,0,4,0,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,
  4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,
  2,3,2,3,2,4,3,4,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,2,7,7,7,7,7,7,7,4,7,4,0,
  0,4,0,0,4,3,3,3,3,3,2,3,2,3,4,0,0,4,0,0,4,0,0,4,0,0,4,5,7,4,3,3,3,3,3,
  2,2,2,3,2,7,7,2,7,7,7,7,7,7,7,3,4,0,0,4,5,7,4,3,3,2,7,7,2,7,7,7,7,7,7,
  7,4
};

static const unsigned short ag_key_parm[] = {
    0,125,130,  0,  0,  6,  0,  0,  4,  8,  0,  2,  0,129,  0,232,225,  0,
  228,  0,251,220,  0,252,218,  0,253,219,  0,125,130,221,  0,155,154,  0,
  226,  0,242,239,  0,227,  0,241,243,  0,  4, 34,  0, 16, 26,  0,  8,  0,
   20,  0,  0,  0,  6, 30,  0,  0,  0,  0,  2,  0, 18,  0,  0,  0,  0,  0,
    0,  0,223,231,  0,237,222,  0,  0,  0,  0,  0,  0,  0,236,  0,224, 10,
   12,  0, 24,  0, 22, 32, 28, 14,  0,  0,  0,  0,  0,  6,  0,  4,  8,  0,
    2,  0,125,130,  0,155,154,  0,252,253,  0,  0,  0,125,130,  0,155,154,
    0, 16, 26,  0,252,253,  0,  0, 10, 12,  0, 20, 22, 28, 14, 30, 18,  0,
  125,130,  0,  0,  0,155,154,  0,252,253,  0,  0,155,154,  0, 16, 26,  0,
  252,253,  0, 10, 12,  0, 20, 22, 28, 14, 30, 18,  0,168,167,169,170,171,
  172,173,174,175,176,177,182,  0,  0,  0,125,130,  0,242,239,  0,241,243,
    0,237,232,251,  0,  0,236,  0,231,  0,242,239,  0,241,243,  0,237,232,
  251,  0,236,  0,231,  0,125,130,  0,155,154,  0,252,253,  0,  0, 34, 32,
    0,155,154,  0,252,253,  0, 34, 32,  0,242,239,  0,241,243,  0,237,232,
    0,236,  0,231,  0,237,232,239,236,241,231,  0,237,232,236,231,  0,232,
  231,  0,232,225,  0,228,  0,251,220,  0,252,218,  0,253,219,  0,125,130,
  221,  0,226,  0,242,239,  0,227,  0,241,243,  0,223,231,  0,237,222,  0,
    0,  0,  0,  0,  0,236,  0,224,  0,  0,232,225,  0,228,  0,251,220,  0,
  252,218,  0,253,219,  0,226,  0,242,239,  0,227,  0,241,243,  0,223,231,
    0,237,222,  0,  0,  0,  0,221,  0,236,  0,224,  0,  0,231,  0,125,130,
    0,155,154,  0, 16, 26,  0,252,253,  0,  0, 10, 12,  0, 24, 20, 22, 28,
   14, 30, 18,  0, 18,  0,242,239,  0,241,243,  0,237,232,251,252,253,  0,
  236,  0,231,  0,125,130,  0,155,154,  0,242,239,  0,241,243,  0, 16, 26,
    0,237,232,251,252,253,  0,  0,  0,236,  0, 10, 12,  0, 24, 20, 22, 28,
   14, 30, 18,231,  0,155,154,  0, 16, 26,  0,252,253,  0, 10, 12,  0, 24,
   20, 22, 28, 14, 30, 18,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 19,  0,  1,  0,  6,  4, 27,  0, 36,  0,  0,  0,  0,
    0,  0, 18,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 36,  0,  0,  0,  0,  0, 41,  0, 59, 63,  0, 46, 67,  0, 74,  0,
   52,  0, 54,  0,101,108,  0, 96, 58,  0,112,  0, 64,  0, 66,  0, 68,  0,
   70,  0,  0,  0,  0, 38, 40, 15, 20, 23, 26, 29, 33, 38, 42, 43, 44, 46,
   51, 49, 70, 56, 79, 81, 85, 90, 61, 72, 74,  0,130,135,  0,116,122,102,
  143,  0,  0,  0,  0,  0,  0,  0,152,154,110,113,  0,  0,  0,  0,  0,  0,
    0,  0,173,  0,156,158,121,124,160,165,127,176,179,181,186,192,198,  0,
    0,  0,  0,144,  0,  0,  0,  0,203,205,149,  0,  0,  0,  0,  0,224,  0,
  207,209,156,211,216,159,227,230,232,237,243,249,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,175,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,254,256,258,190,193,260,196,262,  0,  0,  0,  0,  0,  0,  0,264,266,
  268,208,270,211,272,  0,  0,  0,  0,  0,  0,  0,274,276,222,225,278,284,
    0,  0,  0,  0,288,290,235,292,298,  0,  0,  0,  0,  0,  0,  0,302,304,
  244,306,247,308,  0,310,312,314,316,318,320,  0,322,324,326,328,  0,330,
  332,  0,  0,  0,  0,  0,  0,275,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,290,  0,  0,  0,  0,  0,295,  0,  0,  0,  0,334,336,272,
  277,280,283,286,292,338,297,340,300,  0,  0,  0,  0,  0,  0,319,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,330,  0,  0,  0,  0,  0,335,  0,  0,  0,
    0,342,344,316,321,324,327,346,332,348,337,350,340,  0,352,  0,  0,  0,
    0,  0,  0,  0,  0,371,  0,354,356,358,361,358,363,364,374,378,381,383,
  388,394,400,  0,405,  0,  0,  0,  0,  0,  0,  0,410,412,414,416,418,384,
  420,387,422,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,449,
    0,424,426,428,430,432,400,403,406,434,409,436,441,412,452,456,459,461,
  466,472,478,483,  0,  0,  0,  0,  0,502,  0,485,487,437,489,494,440,505,
  509,512,514,519,525,531,  0
};

static const unsigned short ag_key_index[] = {
    7,  0, 13, 77,105,  0,  0, 13, 13,116,130,130,147,130,152,162,162,  0,
  162,188,  0,  0,188,199,  0,199,199,199,199,199,199,199,199,199,214,228,
    0,  0,238,116,116,116,116,214,152,152,152,152,250,250,250,257,264,269,
  269,269,303,269,343,356,343,  0,  0,147,147,116,147,130,367,367,  0,152,
    0,116,147,147,162,367,  0,147,  0,  0,  0,152,  0,  0,  0,152,  0,162,
  382,152,162,162,188,199,  0,  0,188,188,188,  0,199,  0,  0,214,214,147,
    0,147,  0,  0,116,152,390,116,152,116,152,116,152,152,152,116,152,116,
  152,116,152,116,152,116,152,116,152,116,152,116,152,116,152,116,152,116,
  152,116,152,152,116,152,116,152,147,  0,116,152,116,152,116,152,116,152,
  116,152,116,152,116,152,116,152,116,152,116,152,116,152,116,152,116,152,
  415,  0,  0,162,152,152,  0,  0,  0,  0,152,443,  0,  0,  0,  0,152,152,
  250,250,250,250,250,250,250,250,257,257,264,269,269,152,  0,  0,152,152,
    0,  0,  0,367,  0,152,  0,130,162,199,199,269,152,269,  0,  0,  0,  0,
    0,116,152,  0,  0,152,  0
};

static const unsigned char ag_key_ends[] = {
111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
47,0, 61,0, 61,0, 61,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 111,111,112,0, 98,108,101,0, 
109,112,0, 108,115,101,0, 105,101,108,100,0, 102,0, 111,110,103,0, 
114,105,110,116,0, 101,116,117,114,110,0, 114,105,112,116,0, 
116,101,109,101,110,116,0, 117,99,116,0, 111,111,112,0, 
111,76,111,111,112,0, 111,114,70,105,101,108,100,0, 114,105,112,116,0, 
97,116,101,109,101,110,116,0, 104,105,108,101,76,111,111,112,0, 
43,0, 45,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 43,0, 45,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 111,117,98,108,101,0, 111,110,103,0, 
61,0, 38,0, 61,0, 124,0, 61,0, 38,0, 61,0, 61,0, 61,0, 124,0, 
61,0, 38,0, 61,0, 124,0, 38,0, 124,0, 61,0, 61,0, 61,0, 
61,0, 61,0, 61,0, 61,0, 61,0, 61,0, 124,0, 43,0, 45,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 114,105,110,116,0, 
101,116,117,114,110,0, 116,114,117,99,116,0, 104,105,108,101,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 124,0, 43,0, 45,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 116,114,117,99,116,0, 
104,105,108,101,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned short ag_tcv[] = {
   14,188,188,188,188,188,188,188,188,187,134,187,187,187,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,187,249,159,188,
  188,248,235,184,207,199,246,244,214,245,256,247,152,189,189,189,189,189,
  189,189,151,151,229,200,238,217,240,230,188,190,190,190,190,138,190,191,
  191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,
  191,188,181,188,234,191,188,190,190,190,190,138,190,191,191,191,191,191,
  191,191,191,191,191,191,191,191,191,191,191,191,191,191,191,211,233,210,
  250,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,
  188,188,188,188
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned short ag_tstt[] = {
187,134,130,125,24,23,22,21,20,0,1,192,193,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,181,159,152,151,138,134,0,132,133,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,181,159,152,151,138,134,129,0,127,
  128,
187,134,130,125,0,1,
24,23,22,21,20,0,7,8,9,10,11,12,194,195,196,197,198,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,181,159,152,151,138,0,
134,0,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,181,159,152,151,138,134,0,
129,0,
256,253,252,250,249,245,244,207,200,199,191,190,189,187,184,159,155,154,152,
  151,138,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,0,1,192,193,
207,187,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,14,0,1,192,
  193,
256,253,252,250,249,245,244,207,200,199,191,190,189,184,159,155,154,152,151,
  138,0,2,3,4,5,6,18,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,
  104,105,109,110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,
  212,257,258,259,260,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,60,
  61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,60,
  61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
207,0,15,48,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,14,0,13,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,182,181,177,176,175,174,173,172,171,
  170,169,168,167,159,152,151,138,0,160,
189,152,151,0,147,
189,152,151,0,141,147,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,182,181,177,176,175,174,173,172,171,170,
  169,168,167,159,152,151,138,0,163,164,165,166,178,179,180,185,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,187,159,134,130,125,0,1,192,193,
256,138,0,
138,0,
190,189,152,151,138,0,157,
256,251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,
  231,230,229,214,200,199,189,187,152,151,138,134,130,125,0,
256,251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,
  231,230,229,214,200,199,189,187,152,151,138,134,130,125,0,147,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,187,134,130,125,0,1,192,193,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,187,134,130,125,0,1,192,193,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,187,134,130,125,0,1,192,193,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,191,190,187,138,134,130,125,0,1,192,193,
251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,233,232,231,
  230,229,214,200,199,191,190,187,138,134,130,125,0,1,192,193,
159,0,5,260,
256,255,254,253,252,250,249,245,244,207,200,199,191,190,189,187,184,159,155,
  154,152,151,138,134,130,125,0,1,192,193,
191,190,138,0,4,61,212,
191,190,138,0,4,61,212,
256,255,254,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,
  138,0,2,3,4,5,6,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,118,119,123,137,140,142,143,144,148,
  156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
251,0,113,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
248,247,246,0,106,107,108,
245,244,0,103,104,
243,242,0,100,101,
241,240,239,238,0,95,96,97,98,
237,236,0,92,93,
235,0,90,
234,0,88,
233,0,86,
256,253,252,251,248,247,246,245,244,243,242,241,240,239,238,237,236,235,234,
  233,232,231,230,229,228,227,226,225,224,223,222,221,220,219,218,217,214,
  211,210,207,200,199,191,190,189,187,152,151,138,134,130,125,0,1,192,193,
232,0,83,
207,0,48,
231,230,0,75,80,
256,253,252,228,227,226,225,224,223,222,221,220,219,218,217,0,62,63,64,65,
  66,67,68,69,70,71,72,73,115,116,120,
214,0,55,
200,199,0,34,35,
207,187,134,130,125,0,1,192,193,
211,191,190,187,138,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
191,190,187,138,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,210,208,207,206,205,204,203,202,
  201,200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,0,1,192,
  193,
256,253,252,250,249,245,244,216,215,213,211,210,209,208,207,206,205,204,203,
  202,201,200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,14,
  0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,210,209,208,207,206,205,204,203,
  202,201,200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,14,
  0,1,192,193,
211,191,190,138,0,4,52,212,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
191,190,138,0,4,212,
256,253,252,250,249,245,244,207,200,191,190,189,187,184,159,155,154,152,151,
  138,134,130,125,0,1,192,193,
200,187,134,130,125,0,1,192,193,
200,187,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,210,208,207,206,205,204,203,202,
  201,200,191,190,189,184,159,155,154,152,151,138,0,13,
256,253,252,250,249,245,244,216,215,213,211,210,209,208,207,206,205,204,203,
  202,201,200,199,191,190,189,187,184,159,155,154,152,151,138,134,130,125,
  14,0,1,192,193,
207,0,48,
207,187,134,130,125,0,1,192,193,
200,0,34,
214,200,0,34,55,
214,200,0,34,55,
256,253,252,250,249,245,244,207,200,191,190,189,184,159,155,154,152,151,138,
  0,2,3,4,5,6,18,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,
  257,258,259,260,
200,0,34,
200,0,34,
214,200,0,34,55,
256,253,252,250,249,245,244,207,200,191,190,189,184,159,155,154,152,151,138,
  0,2,3,4,5,6,33,36,48,49,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,
  257,258,259,260,
207,0,48,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,60,
  61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
205,0,17,45,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,
  259,260,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,16,25,26,27,29,
  30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,60,
  61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,14,0,2,3,4,5,6,16,25,26,27,
  29,30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,
  60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,
  115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,250,249,248,247,246,245,244,240,238,235,234,233,230,229,217,214,211,210,
  207,200,199,191,190,189,188,187,184,182,181,177,176,175,174,173,172,171,
  170,169,168,167,159,152,151,138,0,161,163,164,165,166,178,179,180,
189,152,151,0,141,147,
189,152,0,
190,189,152,151,138,0,183,
189,152,0,
189,152,0,
190,189,152,151,138,0,183,
184,0,
189,152,151,0,141,147,
245,244,189,152,151,0,139,145,
245,244,189,152,151,0,139,145,
256,0,120,
256,0,120,
199,187,134,130,125,0,1,192,193,
199,0,35,
199,187,134,130,125,0,1,192,193,
199,0,35,
214,199,0,35,55,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,0,115,116,120,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,103,104,105,109,110,111,112,114,115,116,117,123,137,140,
  142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,102,103,104,105,109,110,111,112,114,115,116,117,123,137,
  140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,102,103,104,105,109,110,111,112,114,115,116,117,123,137,
  140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,94,99,102,103,104,105,109,110,111,112,114,115,116,117,123,
  137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,91,94,99,102,103,104,105,109,110,111,112,114,115,116,117,
  123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,91,94,99,102,103,104,105,109,110,111,112,114,115,116,117,
  123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,
  116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,
  84,
256,253,252,250,249,245,244,207,199,191,190,189,184,159,155,154,152,151,138,
  0,2,3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,121,122,123,137,140,142,143,144,148,156,212,
  257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,
  81,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,
  76,
191,190,187,138,134,130,125,0,1,192,193,
191,190,138,0,4,212,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
256,253,252,251,250,249,248,247,246,245,244,243,242,241,240,239,238,237,236,
  235,234,233,232,231,230,229,216,215,214,213,211,210,209,208,207,206,205,
  204,203,202,201,200,199,191,190,189,187,184,159,155,154,152,151,138,134,
  130,125,14,0,1,192,193,
191,190,138,0,4,59,212,
211,191,190,138,0,4,52,59,212,
256,253,252,250,249,245,244,216,215,213,211,210,208,207,206,205,204,203,202,
  201,200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,16,25,26,27,
  29,30,31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,53,54,56,57,
  58,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,
  115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,200,191,190,189,184,159,155,154,152,151,138,
  0,2,3,4,5,6,18,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,
  257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
191,190,138,0,4,212,
200,0,34,
214,0,55,
200,0,34,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,
  259,260,
209,0,28,51,
207,0,15,48,
214,199,0,35,55,
189,152,151,0,146,
189,152,151,0,146,
256,253,252,207,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,48,61,
  112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,207,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,48,61,
  112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
248,247,246,0,106,107,108,
248,247,246,0,106,107,108,
245,244,0,103,104,
245,244,0,103,104,
243,242,0,100,101,
243,242,0,100,101,
243,242,0,100,101,
243,242,0,100,101,
241,240,239,238,0,95,96,97,98,
241,240,239,238,0,95,96,97,98,
237,236,0,92,93,
235,0,90,
234,0,88,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,
  115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
214,0,55,
199,0,35,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,61,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,
  114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,36,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,
  110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,
  259,260,
214,210,0,53,55,
214,0,55,
191,190,138,0,4,59,212,
256,253,252,250,249,245,244,216,215,213,211,210,209,208,207,206,205,204,203,
  202,201,200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,14,
  0,1,192,193,
200,0,34,
256,253,252,250,249,245,244,207,199,191,190,189,184,159,155,154,152,151,138,
  0,2,3,4,5,6,33,36,48,49,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,
  105,109,110,111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,
  257,258,259,260,
214,199,0,35,55,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,187,184,159,155,154,152,151,138,134,130,125,0,1,192,193,
256,253,252,250,249,245,244,216,215,213,211,208,207,206,205,204,203,202,201,
  200,191,190,189,184,159,155,154,152,151,138,0,2,3,4,5,6,25,26,27,29,30,
  31,32,34,36,37,38,39,40,41,42,43,44,45,46,47,48,50,52,54,56,57,58,60,61,
  74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,112,114,115,116,
  117,123,137,140,142,143,144,148,156,212,257,258,259,260,
189,152,151,0,147,
189,152,151,0,147,
233,0,86,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,57,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,
  111,112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,
  260,
232,0,83,
229,214,0,55,77,
191,190,138,0,4,212,
191,190,138,0,4,59,212,
214,210,0,53,55,
199,0,35,
256,253,252,250,249,245,244,207,191,190,189,187,184,159,155,154,152,151,138,
  134,130,125,0,1,192,193,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,
  78,
214,0,55,
191,190,138,0,4,59,212,
256,253,252,250,249,245,244,207,191,190,189,184,159,155,154,152,151,138,0,2,
  3,4,5,6,48,60,61,74,79,82,85,87,89,91,94,99,102,103,104,105,109,110,111,
  112,114,115,116,117,123,137,140,142,143,144,148,156,212,257,258,259,260,
214,0,55,

};


static unsigned const char ag_astt[6546] = {
  1,1,1,1,8,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,2,2,2,2,2,7,1,1,1,1,1,0,1,1,1,1,1,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,
  1,1,3,1,1,1,1,1,1,1,1,4,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,2,1,2,2,
  2,1,2,1,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,1,7,1,1,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,
  1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,
  1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,10,5,10,2,4,5,5,5,7,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,10,5,10,10,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,5,2,
  1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,
  1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,1,1,1,1,5,1,1,1,1,5,1,1,1,1,1,1,5,1,1,1,
  1,1,1,5,1,1,1,5,1,1,5,1,1,5,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,
  1,5,1,1,4,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,1,1,5,1,1,1,7,2,2,5,1,1,1,1,7,1,1,3,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,
  1,1,5,7,1,1,3,1,2,2,2,7,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,
  1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,3,1,1,7,3,
  1,1,1,7,3,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,2,
  1,7,2,1,1,7,3,1,1,1,1,1,1,1,1,1,5,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,2,2,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,2,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,2,1,2,2,2,1,2,1,2,1,1,1,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,2,7,2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,
  2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,
  8,7,2,1,1,1,8,8,8,7,2,1,1,4,1,1,4,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,
  1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,2,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,
  1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,
  2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,
  1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,
  1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,
  2,7,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,
  2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,2,2,2,7,2,1,1,1,2,2,2,
  7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,
  2,2,1,1,2,3,3,3,1,3,2,2,1,3,1,3,1,1,1,1,1,1,2,2,1,1,1,1,1,3,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,4,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,1,7,2,1,5,1,1,7,1,1,1,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,1,7,2,1,1,1,7,2,1,2,2,2,7,1,2,2,2,
  7,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,1,1,2,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,2,1,1,1,1,1,
  1,1,2,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,
  1,1,4,1,1,1,1,4,1,1,1,1,4,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,4,1,
  1,1,4,1,1,4,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,5,1,1,7,2,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,
  7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,1,1,1,1,1,1,1,7,1,1,1,4,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,5,7,1,1,3,1,7,2,1,1,1,1,1,1,1,
  1,5,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,2,2,1,2,2,2,1,2,1,2,
  1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,10,10,10,4,2,10,10,10,4,2,1,4,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1,1,1,7,1,1,2,2,2,7,2,1,2,2,2,
  7,2,1,1,1,1,7,1,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,4,1,2,2,2,4,2,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,4,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
364,364,1,2,366,364,
10,9,8,7,6,4,18,17,16,15,14,0,13,12,11,10,9,
139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,139,
  139,139,139,139,139,139,139,139,139,139,139,139,139,139,141,
142,6,
134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
  134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,136,
137,8,
365,365,365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,
  365,365,3,1,2,9,3,3,371,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,10,3,3,370,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,11,3,3,369,
365,3,3,1,2,12,3,3,368,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,365,13,3,3,367,
21,32,33,39,40,41,42,35,53,53,143,143,163,22,19,173,173,169,163,143,14,128,
  129,58,34,131,62,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,15,128,129,58,34,131,4,4,4,89,4,19,20,87,4,86,4,
  85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,57,55,54,53,
  52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,16,128,129,58,34,131,90,90,90,89,90,19,20,87,90,
  86,90,85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,57,55,
  54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,173,56,31,30,29,23,
35,17,92,91,
93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,93,
  93,93,93,93,93,93,18,93,
180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
  180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,180,
  180,180,180,180,180,180,180,180,19,94,
168,168,168,152,168,
95,95,95,21,150,95,
101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,
  101,101,101,101,101,101,101,101,97,96,197,196,195,194,193,192,191,190,
  189,188,187,101,101,101,101,22,101,101,101,100,99,98,101,101,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,3,365,3,1,2,23,3,3,433,
102,103,24,
104,145,
176,175,175,175,176,162,174,
153,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,161,
  161,161,161,161,161,161,170,161,170,167,153,161,161,161,27,
151,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,
  160,160,160,160,160,160,164,160,164,164,151,160,160,160,28,164,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,3,3,1,2,29,3,3,432,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,3,3,1,2,30,3,3,431,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,3,3,1,2,31,3,3,430,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,3,365,3,1,2,32,3,3,426,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,3,365,3,1,2,33,3,3,425,
19,130,178,23,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,
  365,365,365,365,3,1,2,35,3,3,380,
143,143,143,36,121,105,56,
143,143,143,37,121,106,56,
21,107,109,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,38,
  128,129,58,34,131,111,38,111,111,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,110,108,34,25,24,28,20,27,26,173,56,31,30,29,
  23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,39,3,3,423,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,40,3,3,422,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,41,3,3,418,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,42,3,3,417,
112,109,113,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,44,128,129,58,
  34,131,38,114,47,46,108,108,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,45,128,129,58,
  34,131,38,114,47,46,107,107,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,46,128,129,58,
  34,131,38,114,47,46,106,106,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,47,128,129,58,
  34,131,38,114,47,46,105,105,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
115,117,119,97,120,118,116,
41,42,94,122,121,
123,125,89,126,124,
127,129,131,133,86,134,132,130,128,
135,137,84,138,136,
139,82,140,
141,80,142,
143,77,144,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,144,144,144,3,144,144,144,3,1,2,56,3,3,385,
145,74,146,
35,121,147,
148,150,70,151,149,
152,32,33,154,156,158,160,162,164,166,168,170,172,174,176,113,177,175,173,
  171,169,167,165,163,161,159,157,155,116,117,153,
178,54,179,
77,180,62,5,5,
365,3,3,1,2,63,3,3,379,
365,365,365,3,365,3,1,2,64,3,3,389,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,65,3,3,388,
365,365,3,365,3,1,2,66,3,3,386,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,67,3,3,384,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,365,68,3,3,
  378,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,365,69,3,3,
  377,
67,143,143,143,70,182,181,56,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,71,128,129,58,
  34,131,38,45,45,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
143,143,143,72,43,56,
365,365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,
  365,3,1,2,73,3,3,376,
365,3,3,1,2,74,3,3,375,
365,3,3,1,2,75,3,3,374,
183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,183,
  183,183,183,183,183,183,183,183,183,183,183,183,76,183,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,365,77,
  3,3,373,
35,78,184,
365,3,3,1,2,79,3,3,381,
77,80,30,
178,77,81,29,185,
178,77,82,28,186,
21,32,33,39,40,41,42,35,53,143,143,163,22,19,173,173,169,163,143,83,128,129,
  58,34,131,187,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
77,84,26,
77,85,25,
178,77,86,22,179,
21,32,33,39,40,41,42,35,37,143,143,163,22,19,173,173,169,163,143,87,128,129,
  58,34,131,189,188,38,39,188,188,60,59,57,55,54,53,52,51,50,49,48,47,46,
  48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
35,88,190,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,89,128,129,58,34,131,15,15,191,89,191,19,20,87,
  191,86,191,85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,
  57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
68,90,192,34,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,91,128,129,58,
  34,131,193,38,193,193,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,
  44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,92,128,129,58,34,131,2,2,2,89,2,19,20,87,2,86,2,
  85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,57,55,54,53,
  52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,1,93,128,129,58,34,131,12,12,12,89,12,19,20,87,
  12,86,12,85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,57,
  55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,173,56,31,30,29,23,
181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,181,
  181,181,181,181,181,181,181,181,181,97,96,197,196,195,194,193,192,191,
  190,189,188,187,179,181,181,181,94,181,181,181,181,100,99,98,181,
95,95,95,158,159,95,
201,201,96,
207,206,206,206,207,97,204,
203,203,199,
202,202,198,
207,206,206,206,207,186,205,
208,101,
95,95,95,149,148,95,
194,195,195,195,195,103,147,195,
194,195,195,195,195,104,146,195,
152,115,153,
152,114,153,
365,3,3,1,2,107,3,3,428,
180,108,196,
365,3,3,1,2,109,3,3,427,
180,110,197,
178,180,111,111,179,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,112,3,3,424,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,113,128,129,
  58,34,131,38,114,47,46,110,110,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,173,56,31,30,29,23,
152,32,33,113,116,117,153,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,115,3,3,421,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,116,128,129,
  58,34,131,38,114,47,46,103,103,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,117,3,3,420,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,118,128,129,
  58,34,131,38,114,47,46,102,102,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,119,3,3,419,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,120,128,129,
  58,34,131,38,114,47,46,101,101,45,44,43,43,37,36,43,34,25,24,28,20,27,
  26,173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,121,128,129,
  58,34,131,38,114,198,47,46,198,198,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,122,128,129,
  58,34,131,38,114,199,47,46,199,199,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,123,3,3,416,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,124,128,129,
  58,34,131,38,114,200,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,125,3,3,415,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,126,128,129,
  58,34,131,38,114,201,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,
  27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,127,3,3,414,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,128,128,129,
  58,34,131,38,114,202,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,129,3,3,413,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,130,128,129,
  58,34,131,38,114,203,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,131,3,3,412,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,132,128,129,
  58,34,131,38,114,204,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,133,3,3,411,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,134,128,129,
  58,34,131,38,114,205,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,135,3,3,410,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,136,128,129,
  58,34,131,38,114,206,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,137,3,3,409,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,138,128,129,
  58,34,131,38,114,207,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,
  28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,139,3,3,408,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,140,128,129,
  58,34,131,38,114,208,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,
  24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,141,3,3,407,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,142,128,129,
  58,34,131,38,114,209,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,
  25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,143,3,3,406,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,144,128,129,
  58,34,131,38,114,210,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,
  34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,145,3,3,405,
78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,78,146,211,
21,32,33,39,40,41,42,35,124,143,143,163,22,19,173,173,169,163,143,147,128,
  129,58,34,131,38,126,126,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,213,212,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,148,3,3,404,
75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,75,149,214,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,150,3,3,403,
71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,71,151,215,
365,365,3,365,3,1,2,152,3,3,429,
143,143,143,153,122,56,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,154,3,3,401,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,155,128,129,
  58,34,131,38,69,69,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,156,3,3,400,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,157,128,129,
  58,34,131,38,68,68,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,158,3,3,399,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,159,128,129,
  58,34,131,38,67,67,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,160,3,3,398,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,161,128,129,
  58,34,131,38,66,66,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,162,3,3,397,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,163,128,129,
  58,34,131,38,65,65,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,164,3,3,396,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,165,128,129,
  58,34,131,38,64,64,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,166,3,3,395,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,167,128,129,
  58,34,131,38,63,63,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,168,3,3,394,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,169,128,129,
  58,34,131,38,62,62,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,170,3,3,393,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,171,128,129,
  58,34,131,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,172,3,3,392,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,173,128,129,
  58,34,131,38,60,60,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,174,3,3,391,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,175,128,129,
  58,34,131,38,59,59,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,176,3,3,390,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,177,128,129,
  58,34,131,38,58,58,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,178,3,3,387,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,179,128,129,
  58,34,131,38,56,56,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,
  365,180,3,3,372,
143,143,143,181,51,216,56,
67,143,143,143,182,51,218,217,56,
21,32,33,39,40,41,42,64,65,66,67,219,79,35,63,68,69,73,74,75,77,143,143,163,
  22,19,173,173,169,163,143,183,128,129,58,34,131,12,12,12,89,12,19,20,87,
  12,86,12,85,84,83,82,81,80,19,20,78,78,38,88,76,42,72,71,86,70,86,60,59,
  57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,
  20,27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,35,53,143,143,163,22,19,173,173,169,163,143,184,128,
  129,58,34,131,220,61,38,61,61,60,59,57,55,54,53,52,51,50,49,48,47,46,48,
  48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,185,128,129,
  58,34,131,38,46,46,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
143,143,143,186,44,56,
77,187,27,
178,38,179,
77,189,221,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,190,128,129,
  58,34,131,222,38,222,222,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
223,14,224,41,
35,192,3,91,
178,180,193,35,179,
165,165,165,194,225,
165,165,165,195,226,
21,32,33,35,143,143,163,22,19,173,173,169,163,143,196,128,129,58,34,131,38,
  114,120,120,37,36,120,34,25,24,28,20,27,26,173,56,31,30,29,23,
21,32,33,35,143,143,163,22,19,173,173,169,163,143,197,128,129,58,34,131,38,
  114,119,119,37,36,119,34,25,24,28,20,27,26,173,56,31,30,29,23,
115,117,119,99,120,118,116,
115,117,119,98,120,118,116,
41,42,96,122,121,
41,42,95,122,121,
123,125,93,126,124,
123,125,92,126,124,
123,125,91,126,124,
123,125,90,126,124,
127,129,131,133,88,134,132,130,128,
127,129,131,133,87,134,132,130,128,
135,137,85,138,136,
139,83,140,
141,81,142,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,211,128,129,
  58,34,131,38,114,227,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,
  43,34,25,24,28,20,27,26,173,56,31,30,29,23,
178,125,228,
180,213,123,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,214,128,129,
  58,34,131,38,114,229,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,
  36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,215,128,129,
  58,34,131,230,38,230,230,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,
  45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
178,219,216,232,231,
178,50,231,
143,143,143,218,51,233,56,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,365,219,3,3,
  383,
77,220,36,
21,32,33,39,40,41,42,35,37,143,143,163,22,19,173,173,169,163,143,221,128,
  129,58,34,131,234,188,38,39,188,188,60,59,57,55,54,53,52,51,50,49,48,47,
  46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
178,180,222,40,179,
365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,365,
  365,365,365,365,3,365,365,365,365,365,365,365,3,1,2,223,3,3,382,
21,32,33,39,40,41,42,64,65,66,67,79,35,63,68,69,73,74,75,77,143,143,163,22,
  19,173,173,169,163,143,224,128,129,58,34,131,16,17,89,17,19,20,87,17,86,
  17,85,84,83,82,81,80,19,20,78,78,38,88,76,72,71,86,70,86,60,59,57,55,54,
  53,52,51,50,49,48,47,46,48,48,45,44,43,43,37,36,43,34,25,24,28,20,27,26,
  173,56,31,30,29,23,
166,166,166,157,166,
166,166,166,156,166,
143,79,144,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,228,128,129,
  58,34,131,38,127,127,60,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,
  43,43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
145,76,146,
235,178,230,179,236,
143,143,143,231,52,56,
143,143,143,232,51,237,56,
178,219,233,238,231,
180,234,21,
365,365,365,365,365,365,365,365,365,365,365,3,365,365,365,365,365,365,365,3,
  1,2,235,3,3,402,
72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,72,236,239,
178,49,231,
143,143,143,48,51,240,56,
21,32,33,39,40,41,42,35,143,143,163,22,19,173,173,169,163,143,239,128,129,
  58,34,131,38,73,114,59,57,55,54,53,52,51,50,49,48,47,46,48,48,45,44,43,
  43,37,36,43,34,25,24,28,20,27,26,173,56,31,30,29,23,
178,47,231,

};


static const unsigned short ag_sbt[] = {
     0,  13,  50,  88,  94, 111, 145, 147, 182, 184, 212, 250, 288, 297,
   336, 402, 502, 602, 606, 639, 686, 691, 697, 750, 783, 786, 788, 795,
   829, 864, 896, 928, 960, 995,1030,1034,1064,1071,1078,1145,1171,1197,
  1223,1249,1252,1302,1352,1402,1452,1459,1464,1469,1478,1483,1486,1489,
  1492,1548,1551,1554,1559,1590,1593,1598,1607,1619,1645,1656,1695,1736,
  1777,1785,1847,1853,1880,1889,1898,1931,1973,1976,1985,1988,1993,1998,
  2063,2066,2069,2074,2140,2143,2243,2247,2310,2410,2511,2565,2571,2574,
  2581,2584,2587,2594,2596,2602,2610,2618,2621,2624,2633,2636,2645,2648,
  2653,2679,2729,2736,2762,2812,2838,2888,2914,2964,3015,3066,3092,3144,
  3170,3222,3248,3301,3327,3380,3406,3459,3485,3538,3564,3618,3644,3698,
  3724,3779,3805,3861,3887,3944,3970,3990,4055,4081,4101,4127,4147,4158,
  4164,4190,4252,4278,4340,4366,4428,4454,4516,4542,4604,4630,4692,4718,
  4780,4806,4868,4894,4956,4982,5044,5070,5132,5158,5220,5246,5308,5370,
  5377,5386,5488,5553,5615,5621,5624,5627,5630,5693,5697,5701,5706,5711,
  5716,5756,5796,5803,5810,5815,5820,5825,5830,5835,5840,5849,5858,5863,
  5866,5869,5927,5930,5933,5992,6055,6060,6063,6070,6111,6114,6180,6185,
  6223,6322,6327,6332,6335,6397,6400,6405,6411,6418,6423,6426,6452,6472,
  6475,6482,6543,6546
};


static const unsigned short ag_sbe[] = {
     9,  47,  85,  92,  99, 144, 146, 181, 183, 208, 246, 284, 293, 332,
   356, 432, 532, 603, 637, 684, 689, 694, 741, 779, 785, 787, 793, 828,
   862, 892, 924, 956, 991,1026,1031,1060,1067,1074,1098,1167,1193,1219,
  1245,1250,1270,1320,1370,1420,1455,1461,1466,1473,1480,1484,1487,1490,
  1544,1549,1552,1556,1574,1591,1595,1603,1615,1641,1652,1691,1732,1773,
  1781,1803,1850,1876,1885,1894,1929,1969,1974,1981,1986,1990,1995,2017,
  2064,2067,2071,2093,2141,2173,2244,2265,2340,2441,2556,2568,2573,2579,
  2583,2586,2592,2595,2599,2607,2615,2619,2622,2629,2634,2641,2646,2650,
  2675,2697,2732,2758,2780,2834,2856,2910,2932,2982,3033,3088,3110,3166,
  3188,3244,3266,3323,3345,3402,3424,3481,3503,3560,3582,3640,3662,3720,
  3742,3801,3823,3883,3905,3966,3988,4009,4077,4099,4123,4145,4154,4161,
  4186,4208,4274,4296,4362,4384,4450,4472,4538,4560,4626,4648,4714,4736,
  4802,4824,4890,4912,4978,5000,5066,5088,5154,5176,5242,5264,5366,5373,
  5381,5417,5507,5571,5618,5622,5625,5628,5648,5694,5698,5703,5709,5714,
  5730,5770,5799,5806,5812,5817,5822,5827,5832,5837,5844,5853,5860,5864,
  5867,5887,5928,5931,5951,6010,6057,6061,6066,6107,6112,6133,6182,6219,
  6253,6325,6330,6333,6353,6398,6402,6408,6414,6420,6424,6448,6470,6473,
  6478,6500,6544,6546
};


static const unsigned char ag_fl[] = {
  2,3,3,4,2,3,1,1,1,1,1,0,2,1,1,2,4,4,1,1,1,5,2,1,1,2,2,3,2,2,2,1,1,1,1,
  3,4,0,1,1,4,1,3,2,3,2,3,6,5,5,3,1,3,0,1,1,3,1,3,3,3,3,3,3,3,3,3,3,3,3,
  1,0,0,7,1,0,4,1,0,4,1,3,1,3,1,3,1,3,3,1,3,3,3,3,1,3,3,1,3,3,1,3,3,3,1,
  2,2,2,2,1,3,3,1,1,2,2,2,2,1,4,4,1,3,4,0,1,1,3,1,1,1,1,1,1,2,0,1,3,1,2,
  0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,
  1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned short ag_ptt[] = {
    0, 12, 12, 12, 12, 12,194,195,196,197,198, 13, 13, 16, 16, 25, 25, 26,
   26, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 31, 46, 17, 15,
   32, 49, 49, 33, 27, 28, 37, 41, 41, 42, 42, 43, 43, 43, 43, 59, 59, 18,
   18, 36, 36, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 60, 76,
   78, 60, 74, 81, 74, 79, 84, 79, 82, 82, 85, 85, 87, 87, 89, 89, 89, 91,
   91, 91, 91, 91, 94, 94, 94, 99, 99, 99,102,102,102,102,105,105,105,105,
  105,109,109,112,112,112,112,112,112,112,112,112,112, 61, 61,117,121,121,
  122,122,114,114,114,114,  1,127,127,128,128,  1,132,132,133,133,  1,212,
  212,258,258,258,137,137,137,140,140,140,145,145,139,139,141,141,257,257,
  257,142,142,146,146,143,143,144,144,156,156,148,148,157,157,123,123,260,
  160,160,161,161,163,163,163,164,164,164,164,164,164,164,164,164,164,164,
  165,165,165,178,179,180,166,166,183,183,259,185,185,147, 19, 19,124,124,
  126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,
  126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,131,131,
  131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
  131,131,131,131,131,131,131,131,131,131,131,131,131,135,135,135,136,136,
  136,136,136,136,149,149,150,150,150,153,153,158,158,162,162,162,162,162,
  162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,162,
  162,162,162,162,162,162,162,162,186,186,186,186,186,186,186,186,186,186,
  186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,186,
  186,186,186,192,192,193,193,  7,  8,  9, 10, 11, 35, 34, 38, 39, 40, 44,
   45, 47, 48, 50, 51, 53, 52,  4, 54, 55, 56, 58, 62, 63, 64, 65, 66, 67,
   68, 69, 70, 71, 72, 73, 77, 75, 80, 83, 86, 88, 90, 92, 93, 95, 96, 97,
   98,100,101,103,104,106,107,108,110,111,113,115,116,118,119,120,  2,  3,
    6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: VNO AG_WRAP_5(ag_rp_5(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER); break;
    case 12: ag_rp_15(PCB_POINTER); break;
    case 13: ag_rp_16(PCB_POINTER, V(1,(FileLocation *)), V(3,(FileLocation *))); break;
    case 14: VWD(0,AG_WRAP_5 *); break;
    case 15: ag_rp_17(PCB_POINTER); break;
    case 16: ag_rp_18(PCB_POINTER); break;
    case 17: ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 18: ag_rp_20(PCB_POINTER); break;
    case 19: ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 20: ag_rp_22(PCB_POINTER); break;
    case 21: V(0,(FileLocation *)) = ag_rp_23(PCB_POINTER); break;
    case 22: ag_rp_24(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 23: ag_rp_25(PCB_POINTER); break;
    case 24: ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: ag_rp_27(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 27: ag_rp_29(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 28: ag_rp_30(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(5, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(5, AG_WRAP_7 *); break;
    case 29: ag_rp_31(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); break;
    case 30: ag_rp_32(PCB_POINTER, VW(2, AG_WRAP_7 *), VW(4, AG_WRAP_7 *)); 
            VWD(2, AG_WRAP_7 *); VWD(4, AG_WRAP_7 *); break;
    case 31: ag_rp_33(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_7 *); break;
    case 32: VRO(AG_WRAP_4 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 33: VRO(AG_WRAP_7 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 34: VNO AG_WRAP_5(ag_rp_36(PCB_POINTER)); break;
    case 35: VRO(AG_WRAP_5 *, ag_rp_37(PCB_POINTER, VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 36: VRO(AG_WRAP_5 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 37: VRO(AG_WRAP_5 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: VRO(AG_WRAP_5 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 44: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 45: VRO(AG_WRAP_5 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 46: VRO(AG_WRAP_5 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 47: VRO(AG_WRAP_5 *, ag_rp_49(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 48: ag_rp_50(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 49: ag_rp_51(PCB_POINTER, VW(-5, AG_WRAP_5 *), VW(-2, AG_WRAP_5 *)); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 51: ag_rp_53(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 52: VRO(AG_WRAP_5 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 53: ag_rp_55(PCB_POINTER, VW(-2, AG_WRAP_5 *)); break;
    case 54: VRO(AG_WRAP_5 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 55: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 56: VRO(AG_WRAP_5 *, ag_rp_58(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 57: VRO(AG_WRAP_5 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 58: VRO(AG_WRAP_5 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 59: VRO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 60: VRO(AG_WRAP_5 *, ag_rp_62(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 61: VRO(AG_WRAP_5 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 62: VRO(AG_WRAP_5 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 63: VRO(AG_WRAP_5 *, ag_rp_65(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 64: VRO(AG_WRAP_5 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 65: VRO(AG_WRAP_5 *, ag_rp_67(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 66: VRO(AG_WRAP_5 *, ag_rp_68(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 67: VRO(AG_WRAP_5 *, ag_rp_69(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 68: VRO(AG_WRAP_5 *, ag_rp_70(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 69: VRO(AG_WRAP_5 *, ag_rp_71(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 70: VRO(AG_WRAP_5 *, ag_rp_72(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 71: VNO AG_WRAP_5(ag_rp_73(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 72: VNO AG_WRAP_5(ag_rp_74(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 73: VNO AG_WRAP_5(ag_rp_75(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 74: VNO AG_WRAP_5(ag_rp_76(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 75: VRO(AG_WRAP_5 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 76: VNO AG_WRAP_5(ag_rp_78(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 77: VRO(AG_WRAP_5 *, ag_rp_79(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 78: VNO AG_WRAP_5(ag_rp_80(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 79: VNO AG_WRAP_5(ag_rp_81(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 80: VRO(AG_WRAP_5 *, ag_rp_82(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 81: VRO(AG_WRAP_5 *, ag_rp_83(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 82: VNO AG_WRAP_5(ag_rp_84(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 83: VNO AG_WRAP_5(ag_rp_85(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 84: VRO(AG_WRAP_4 *, ag_rp_86(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 85: VRO(AG_WRAP_5 *, ag_rp_87(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 86: VRO(AG_WRAP_4 *, ag_rp_88(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 87: VNO AG_WRAP_6(ag_rp_89(PCB_POINTER)); break;
    case 88: VRO(AG_WRAP_5 *, ag_rp_90(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 89: VRO(AG_WRAP_6 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 90: VNO AG_WRAP_5(ag_rp_92(PCB_POINTER, V(0,(long *)))); break;
    case 91: VNO AG_WRAP_5(ag_rp_93(PCB_POINTER, V(0,(double *)))); break;
    case 92: VNO AG_WRAP_5(ag_rp_94(PCB_POINTER, V(0,(int *)))); break;
    case 93: VNO AG_WRAP_4(ag_rp_95(PCB_POINTER, V(0,(int *)))); break;
    case 94: VRO(AG_WRAP_4 *, ag_rp_96(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 95: V(0,(double *)) = ag_rp_97(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 96: V(0,(double *)) = ag_rp_98(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 97: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 98: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(double *))); break;
    case 99: V(0,(double *)) = ag_rp_101(PCB_POINTER, V(1,(double *))); break;
    case 100: V(0,(double *)) = ag_rp_102(PCB_POINTER, V(0,(long *))); break;
    case 101: V(0,(double *)) = ag_rp_103(PCB_POINTER, V(0,(long *))); break;
    case 102: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(long *))); break;
    case 103: V(0,(long *)) = ag_rp_105(PCB_POINTER, V(1,(long *))); break;
    case 104: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(1,(long *))); break;
    case 105: V(0,(double *)) = ag_rp_107(PCB_POINTER, V(0,(int *))); break;
    case 106: V(0,(double *)) = ag_rp_108(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 107: V(0,(long *)) = ag_rp_109(PCB_POINTER, V(0,(int *))); break;
    case 108: V(0,(long *)) = ag_rp_110(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 109: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(int *))); break;
    case 110: V(0,(long *)) = ag_rp_112(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 111: V(0,(long *)) = ag_rp_113(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 112: V(0,(long *)) = ag_rp_114(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 113: V(0,(long *)) = ag_rp_115(PCB_POINTER); break;
    case 114: V(0,(long *)) = ag_rp_116(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 115: V(0,(long *)) = ag_rp_117(PCB_POINTER); break;
    case 116: V(0,(long *)) = ag_rp_118(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 117: V(0,(int *)) = ag_rp_119(PCB_POINTER, V(0,(int *))); break;
    case 118: V(0,(int *)) = ag_rp_120(PCB_POINTER, V(0,(int *))); break;
    case 119: VRO(AG_WRAP_5 *, ag_rp_121(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 120: VNO AG_WRAP_5(ag_rp_122(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 121: VNO AG_WRAP_4(ag_rp_123(PCB_POINTER)); break;
    case 122: VRO(AG_WRAP_4 *, ag_rp_124(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 123: V(0,(int *)) = ag_rp_125(PCB_POINTER); break;
    case 124: V(0,(int *)) = ag_rp_126(PCB_POINTER); break;
    case 125: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 126: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 127: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 128: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_132(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_133(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_134(PCB_POINTER); break;
    case 133: V(0,(int *)) = ag_rp_135(PCB_POINTER); break;
    case 134: V(0,(int *)) = ag_rp_136(PCB_POINTER, V(1,(int *))); break;
    case 135: V(0,(int *)) = ag_rp_137(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 136: V(0,(int *)) = ag_rp_138(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 137: V(0,(int *)) = ag_rp_139(PCB_POINTER, V(1,(int *))); break;
    case 138: V(0,(int *)) = ag_rp_140(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 139: V(0,(int *)) = ag_rp_141(PCB_POINTER, V(0,(int *))); break;
    case 140: V(0,(int *)) = ag_rp_142(PCB_POINTER, V(0,(int *))); break;
    case 141: V(0,(int *)) = ag_rp_143(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[261] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "syntax selection",
  "statement list",
  "eof",
  "loop condition",
  "statement",
  "DO WHILE",
  "optional expression",
  "",
  "\"script\"",
  "\"whileLoop\"",
  "\"doLoop\"",
  "\"statement\"",
  "\"forField\"",
  "open statement",
  "closed statement",
  "if condition",
  "else",
  "simple statement",
  "DO",
  "WHILE",
  "for initialization",
  "for field",
  "';'",
  "')'",
  "expression",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "structure declaration",
  "\"do\"",
  "\"while\"",
  "FOR",
  "\"for\"",
  "'('",
  "",
  "\"if\"",
  "\"else\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "\"struct\"",
  "name list",
  "conditional expression",
  "lvalue",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "logical or expression",
  "'\\?'",
  "",
  "':'",
  "",
  "logical and expression",
  "\"||\"",
  "",
  "inclusive or expression",
  "\"&&\"",
  "",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "\"==\"",
  "\"!=\"",
  "shift expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "additive expression",
  "\"<<\"",
  "\">>\"",
  "multiplicative expression",
  "'+'",
  "'-'",
  "unary expression",
  "'*'",
  "'/'",
  "'%'",
  "factor",
  "'!'",
  "'~'",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "WHILE LOOP",
  "DO LOOP",
  "STATEMENT",
  "FOR FIELD",
  "')'",
  "';'",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "\"do\"",
  "\"while\"",
  "\"for\"",
  "'('",
  "\"if\"",
  "\"else\"",
  "'}'",
  "'{'",
  "name",
  "\"dump\"",
  "','",
  "\"print\"",
  "\"struct\"",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"^=\"",
  "\"&=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'+'",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned short)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned short)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned short)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


