/*
 dxi.syn -- Direct Execution Script Interpreter
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays, structs and functions.

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dxi.h and dxi.cpp are created from
 dxi.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   dxi.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>                                     // to declare tolower()
#include <iostream.h>                     // Used by dump and print statements

/*
 It is sometimes necessary to skip over code without executing it. The Mode
 enumeration specifies the various execution modes.
*/
enum Mode {
  executeMode,                                                    // executing
  continueMode,         // continue statement encountered, skip to end of loop
  breakMode,               // break statement encountered, skip to end of loop
  elseMode,                                      // Skipping to else statement
  blockMode,                                       // skipping a block of code
  scanMode                       // scanning code to determine loop parameters
};

/*
 The parser uses "pointer input". The ParseContext struct is used to store
 the value of the pointer and the execution mode at critical locations in the
 script, such as at the head of a loop, at the exit point of a loop, or at
 the increment and condition fields of a for loop. Since ParseContext
 inherits from FileLocation, it also stores line and column number.

 In the configuration section of the parser, the "context type" is specified
 as ParseContext. The GET_CONTEXT macro constructs a ParseContext object to
 store on the context stack.

 dxi_pcb_struct is the "parser control block". The ParseContext constructor
 retrieves the necessary information from the pcb.
*/
struct dxi_pcb_struct;          // Because of forward reference in constructor
struct ParseContext : public FileLocation {
  Mode mode;
  ParseContext() : FileLocation(), mode(executeMode) {}
  ParseContext(const char *key)
    : FileLocation((const unsigned char *) key), mode(executeMode) {}
  ParseContext(dxi_pcb_struct &);
};

// struct to describe a for loop
struct ForControl {
  Value lvalue;
  long begin;
  long increment;
  long end;
  ForControl() : lvalue(), begin(0), increment(1), end(0) {}
  ForControl(const Value &lv,
      const Value &b, const Value &i,
      const Value &e)
  : lvalue(lv),
    begin(b.getLong()),
    increment(i.getLong()),
    end(e.getLong())
  {
    // Nothing to do
  }
};

/*
 The VALUE macro controls the execution of expressions in accordance with the
 current execution mode.
*/
#define VALUE(x) PCB.mode == executeMode ? Value(x) : Value()

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 28 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DXI_H_1031146476
#include "dxi.h"
#endif

#ifndef DXI_H_1031146476
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dxi_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

Value dxi_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 599, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = PCB.location()

ParseContext::ParseContext(dxi_pcb_struct &pcb)
  : FileLocation(pcb.pointer, pcb.line, pcb.column),
    mode(pcb.mode)
{ /* empty */ }

// constructor for parser control block
dxi_pcb_struct::dxi_pcb_struct(Dataset &d)
  : dataset(d),
    mode(executeMode),
    returnValue()
{ /* empty */ }

void dxi_pcb_struct::parseSyntax
    (const char *selector, const FileLocation &s) {
  pointer = (const unsigned char *) selector;
  startLocation = s;
  try {
    dxi(this);
  }
  catch (ErrorMessage e) {
    reportError(e.message());
  }
}

void dxi_pcb_type::setStart() {
  setLocation(startLocation);
}

void dxi_pcb_struct::simpleLoop(const char *selector) {
  ParseContext loopLocation(*this);
  dxi_pcb_struct loopPCB(dataset);
  loopPCB.mode = mode == executeMode ? executeMode : breakMode;    // Set mode
  for (;;) {
    loopPCB.parseSyntax(selector, loopLocation);
    setLocation(ParseContext(loopPCB));          //Transfer location to parent
    if (loopPCB.mode == breakMode) return;
    loopPCB.mode = executeMode;    // mode could have been set to continueMode
  }
}

ForControl dxi_pcb_struct::forControl(const Value &lv, const Value &b, const Value &i, const Value &e) {
  if (mode == executeMode) return ForControl(lv, b, i, e);
  return ForControl();
}

void dxi_pcb_struct::forLoop(ForControl &fc) {
  restoreMode();       // mode was set to scanMode to pick up loop header info
  ParseContext statementLocation(*this);         // Capture statement location
  dxi_pcb_struct loopPCB(dataset);
  if (fc.increment == 0) reportError("Loop increment is zero");
  // Calculate number of times through the loop
  int count = (fc.end + fc.increment - fc.begin)/fc.increment;
  int index = fc.begin;
  // Set mode for parsing loop
  loopPCB.mode = (mode == executeMode && count > 0) ? executeMode : breakMode;
  if (loopPCB.mode == executeMode) fc.lvalue = index;
  while (count--) {
    loopPCB.parseSyntax("statement", statementLocation);
    setLocation(ParseContext(loopPCB));       // Set location in parent parser
    index += fc.increment;
    if (loopPCB.mode == executeMode) fc.lvalue = index;
    if (loopPCB.mode == breakMode) return;
  }
}

ParseContext dxi_pcb_struct::location() {
  return ParseContext(*this);
}

// Set parse location in source text
void dxi_pcb_struct::setLocation(const FileLocation &l) {
	pointer = l.pointer;
	line = l.line;
	column = l.column;
}

// Set return value
void dxi_pcb_struct::setReturnValue(const Value &v) {
  if (mode != executeMode) return;   // exit setReturnValue
  returnValue = v;
  exit_flag = AG_SUCCESS_CODE;  // Exit parser
}

// if mode == currentMode change mode to desiredMode and return true.
// Otherwise, leave mode unchanged and return false.
int dxi_pcb_struct::changeMode(Mode currentMode, Mode desiredMode) {
  if (mode != currentMode) return 0;
  mode = desiredMode;
  return 1;
}

// if mode == currentMode, restore the mode in effect at the beginning of the
// rule. Otherwise, leave the mode unchanged.
void dxi_pcb_struct::restoreMode(Mode currentMode) {
  if (mode == currentMode) mode = PCONTEXT(*this).mode;
}

// Unconditionally restore the mode in effect at the beginning of the rule.
void dxi_pcb_struct::restoreMode() {
  mode = PCONTEXT(*this).mode;
}

// dump variable name and value
void dxi_pcb_struct::dump(const AgString &n) const {
  if (mode != executeMode) return;
  cout << (const char *) n
       << " = "
       << (const char *) dataset.value(n).asLiteral()
       << "\n";
}

// print value
void dxi_pcb_struct::print(const Value &value) const {
  if (mode != executeMode) return;
  cout << (const char *) value.asString();
}

// Record syntax error
void dxi_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dxi_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  ParseContext &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

Value interpret(const char *text, Dataset &d) {
  dxi_pcb_struct pcb(d);
  pcb.parseSyntax("script", FileLocation((const unsigned char *)text));
  int i;
  for (i = pcb.localList.size(); i--;) {
    d.value(pcb.localList[i]) = pcb.saveStack.pop();
  }
  return pcb.returnValue;
}

void dxi_pcb_struct::createArray(const AgString &n, const Value &x) {
  if (mode != executeMode) return;   // exit createArray
  dataset.value(n) = Value(x, 0);
}

// declare structure
void dxi_pcb_struct::defineStructure (
    const AgString &structureName,
    const AgStack<AgString> &fieldNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  Value structure(fieldDictionary);
  dataset.value(structureName) = structure;
}

// define instances of named structure
void dxi_pcb_struct::defineInstances (
    const AgString &structureName,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  const AgDictionary<AgString> &fieldDictionary = dataset.value(structureName).getDictionary();
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}

// define instances of an anonymous structure
void dxi_pcb_struct::defineInstances (
    const AgStack<AgString> &fieldNames,
    const AgStack<AgString> &instanceNames)
{
  if (mode != executeMode) return;
  AgDictionary<AgString> fieldDictionary(fieldNames.contents());
  for (int i = 0; i < instanceNames.size(); i++) {
    Instance instance(fieldDictionary);
    dataset.value(instanceNames[i]) = Value(instance);
  }
}

Value dxi_pcb_struct::callFunction(
  const AgString &name,
  AgStack<Value> &args)
{
  if (mode != executeMode) return Value();
  int nArgs = args.size();
  // Is there a variable with this name?
  int index = dataset.dictionary.lookup(name);
  if (index >= 0 && dataset[index].isFunction()) {
    LocalFunction &f = dataset[index].getFunction();
    if (nArgs == f.parameterList.size()) {
      int i;
      dxi_pcb_struct callPCB(dataset);
      // Save existing values for parameter variables
      for (i = 0; i < nArgs; i++) {
        Value &value = dataset.value(f.parameterList[i]);
        saveStack.push(value);
        value = args[i];                    // set variable to parameter value
      }

      const char *text = (const char *) f.text;
      callPCB.parseSyntax("script", FileLocation((const unsigned char *) text));

      // Restore local variables
      for (i = callPCB.localList.size(); i--;) {
        dataset.value(callPCB.localList[i]) = callPCB.saveStack.pop();
      }

      // Restore original values for parameter variables
      for (i = nArgs; i--;) {
        dataset.value(f.parameterList[i]) = saveStack.pop();
      }
      return callPCB.returnValue;
    }
  }
  // Not a locally defined function, try a library function
  return ::callFunction(idFunction(name, args.size()), args);
}


void dxi_pcb_struct::defineFunction(
    const AgString &n,
    const AgStack<AgString> &nameList,
    const AgString &text)
{
  mode = PCONTEXT(*this).mode;
  if (mode != executeMode) return;
  LocalFunction f(nameList, text);
  dataset.value(n) = Value(f);
}

void dxi_pcb_struct::localVariables(const AgStack<AgString> &list) {
  if (mode != executeMode) return;
  localList = list;
  int i;
  for (i = 0; i < list.size(); i++) {
    saveStack.push(dataset.value(list[i]));
  }
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
Value dxi_value(PCB_DECL) {
  Value returnValue;
  returnValue = (*(AgObjectWrapper< Value > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline Value ag_rp_1(PCB_DECL) {
/* Line 221, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return Value();
}

static inline void ag_rp_5(PCB_DECL) {
/* Line 228, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_6(PCB_DECL) {
/* Line 231, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_7(PCB_DECL) {
/* Line 234, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_8(PCB_DECL) {
/* Line 237, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.setStart();
}

static inline void ag_rp_9(PCB_DECL) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.restoreMode(elseMode);
}

static inline void ag_rp_10(PCB_DECL) {
/* Line 262, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_11(PCB_DECL) {
/* Line 266, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.restoreMode(blockMode);
}

static inline void ag_rp_12(PCB_DECL) {
/* Line 280, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.simpleLoop("repeatLoop");
}

static inline void ag_rp_13(PCB_DECL) {
/* Line 281, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.simpleLoop("whileLoop");
}

static inline void ag_rp_14(PCB_DECL, ForControl &fc) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.forLoop(fc);
}

static inline void ag_rp_15(PCB_DECL, Value &x) {
/* Line 286, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.setReturnValue(x);
}

static inline void ag_rp_16(PCB_DECL, AgString &n, Value &x) {
/* Line 289, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.createArray(n, x);
}

static inline void ag_rp_17(PCB_DECL) {
/* Line 297, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.changeMode(continueMode, executeMode);
}

static inline void ag_rp_18(PCB_DECL, AgStack<AgString> &list) {
/* Line 307, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.localVariables(list);
}

static void ag_rp_19(PCB_DECL, Value &x) {
/* Line 312, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
if (PCB.mode != executeMode || x.isFalse()) PCB.mode = breakMode;
}

static void ag_rp_20(PCB_DECL, Value &x) {
/* Line 316, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
if (PCB.mode != executeMode || x.isTrue()) PCB.mode = breakMode;
}

static inline ForControl ag_rp_21(PCB_DECL, Value &lv, Value &b, Value &i, Value &e) {
/* Line 320, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return PCB.forControl(lv, b, i, e);
}

static inline Value ag_rp_22(PCB_DECL) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(1);
}

static inline Value ag_rp_23(PCB_DECL, Value &x) {
/* Line 324, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x;
}

static void ag_rp_24(PCB_DECL, Value &x) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
if (PCB.mode == executeMode && x.isFalse()) PCB.mode = elseMode;
}

static inline void ag_rp_25(PCB_DECL) {
/* Line 333, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.changeMode(executeMode, blockMode),PCB.changeMode(elseMode, executeMode);
}

static inline void ag_rp_26(PCB_DECL, Value &lv, Value &x) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  VALUE(lv = x);
}

static inline void ag_rp_27(PCB_DECL, AgString &n) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_28(PCB_DECL, AgString &n) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.dump(n);
}

static inline void ag_rp_29(PCB_DECL, Value &x) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.print(x);
}

static inline void ag_rp_30(PCB_DECL, Value &x) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.print(x);
}

static void ag_rp_31(PCB_DECL, AgString &name, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
      PCB.defineStructure(name, fields);
      PCB.defineInstances(name, instances);
    
}

static inline void ag_rp_32(PCB_DECL, AgString &name, AgStack<AgString> &fields) {
/* Line 357, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.defineStructure(name, fields);
}

static inline void ag_rp_33(PCB_DECL, AgStack<AgString> &fields, AgStack<AgString> &instances) {
/* Line 359, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.defineInstances(fields, instances);
}

static inline void ag_rp_34(PCB_DECL, AgString &name, AgStack<AgString> &instances) {
/* Line 361, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.defineInstances(name, instances);
}

static inline void ag_rp_35(PCB_DECL, AgString &n, AgStack<AgString> &list) {
/* Line 364, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.mode = scanMode;
}

static inline void ag_rp_36(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 365, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  PCB.defineFunction(n, list, text);
}

static AgString ag_rp_37(PCB_DECL) {
/* Line 368, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_38(PCB_DECL, AgString &n) {
/* Line 373, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_39(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 374, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return stack.push(n);
}

static inline Value ag_rp_40(PCB_DECL) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return Value();
}

static inline Value ag_rp_41(PCB_DECL, Value &x, Value &y) {
/* Line 383, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x < y);
}

static inline Value ag_rp_42(PCB_DECL, Value &x, Value &y) {
/* Line 384, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x <= y);
}

static inline Value ag_rp_43(PCB_DECL, Value &x, Value &y) {
/* Line 385, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x > y);
}

static inline Value ag_rp_44(PCB_DECL, Value &x, Value &y) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x >= y);
}

static inline Value ag_rp_45(PCB_DECL, Value &x, Value &y) {
/* Line 387, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x == y);
}

static inline Value ag_rp_46(PCB_DECL, Value &x, Value &y) {
/* Line 388, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x != y);
}

static inline Value ag_rp_47(PCB_DECL, Value &x, Value &y) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x + y);
}

static inline Value ag_rp_48(PCB_DECL, Value &x, Value &y) {
/* Line 393, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x - y);
}

static inline Value ag_rp_49(PCB_DECL, Value &x, Value &y) {
/* Line 394, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x | y);
}

static inline Value ag_rp_50(PCB_DECL, Value &x, Value &y) {
/* Line 395, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x ^ y);
}

static inline Value ag_rp_51(PCB_DECL, Value &x, Value &y) {
/* Line 399, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x * y);
}

static inline Value ag_rp_52(PCB_DECL, Value &x, Value &y) {
/* Line 400, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x.rdiv(y));
}

static inline Value ag_rp_53(PCB_DECL, Value &x, Value &y) {
/* Line 401, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x.idiv(y));
}

static inline Value ag_rp_54(PCB_DECL, Value &x, Value &y) {
/* Line 402, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x % y);
}

static inline Value ag_rp_55(PCB_DECL, Value &x, Value &y) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x & y);
}

static inline Value ag_rp_56(PCB_DECL, Value &x, Value &y) {
/* Line 404, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x << y);
}

static inline Value ag_rp_57(PCB_DECL, Value &x, Value &y) {
/* Line 405, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x >> y);
}

static inline Value ag_rp_58(PCB_DECL, Value &x) {
/* Line 409, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_59(PCB_DECL, Value &x) {
/* Line 410, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(-x);
}

static inline Value ag_rp_60(PCB_DECL, Value &x) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(!x);
}

static inline Value ag_rp_61(PCB_DECL, Value &x, Value &y) {
/* Line 419, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(pow(x, y));
}

static inline Value ag_rp_62(PCB_DECL, Value &x) {
/* Line 427, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x;
}

static inline Value ag_rp_63(PCB_DECL, Value &lv) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(lv.deref());
}

static inline Value ag_rp_64(PCB_DECL, Value &x) {
/* Line 431, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x.makeInteger());
}

static inline Value ag_rp_65(PCB_DECL, Value &x) {
/* Line 432, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x.makeReal());
}

static inline Value ag_rp_66(PCB_DECL, AgString &n) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(&PCB.dataset.value(n));
}

static inline Value ag_rp_67(PCB_DECL, Value &lv, Value &x) {
/* Line 436, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(lv[x]);
}

static inline Value ag_rp_68(PCB_DECL, Value &lv, AgString &n) {
/* Line 437, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(&lv.getMember(n));
}

static inline Value ag_rp_69(PCB_DECL, AgString &n, AgStack<Value> &args) {
/* Line 440, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return PCB.callFunction(n, args);
}

static inline AgStack<Value> ag_rp_70(PCB_DECL) {
/* Line 443, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return AgStack<Value>();
}

static inline AgStack<Value> ag_rp_71(PCB_DECL, Value &x) {
/* Line 447, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return AgStack<Value>().push(x);
}

static inline AgStack<Value> ag_rp_72(PCB_DECL, AgStack<Value> &stack, Value &x) {
/* Line 448, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return stack.push(x);
}

static inline Value ag_rp_73(PCB_DECL, long x) {
/* Line 451, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_74(PCB_DECL, double x) {
/* Line 452, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x);
}

static inline Value ag_rp_75(PCB_DECL, int x) {
/* Line 454, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(x);
}

static inline AgString ag_rp_76(PCB_DECL, int c) {
/* Line 473, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_77(PCB_DECL, AgString &ns, int c) {
/* Line 474, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_78(PCB_DECL, double x, long e) {
/* Line 479, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_79(PCB_DECL, double x, long e) {
/* Line 480, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x*pow(10,e);
}

static inline double ag_rp_80(PCB_DECL, double i, double f) {
/* Line 483, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return i+f;
}

static inline double ag_rp_81(PCB_DECL, double i) {
/* Line 484, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return i;
}

static inline double ag_rp_82(PCB_DECL, double f) {
/* Line 485, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return f;
}

static inline double ag_rp_83(PCB_DECL, long x) {
/* Line 488, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x;
}

static inline double ag_rp_84(PCB_DECL, long x) {
/* Line 489, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x;
}

static inline double ag_rp_85(PCB_DECL, long x) {
/* Line 490, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return makeDecimal(x);
}

static inline long ag_rp_86(PCB_DECL, long x) {
/* Line 493, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return (long)x;
}

static inline long ag_rp_87(PCB_DECL, long x) {
/* Line 494, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return -(long)x;
}

static inline double ag_rp_88(PCB_DECL, int d) {
/* Line 497, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_89(PCB_DECL, int d, double f) {
/* Line 498, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_90(PCB_DECL, int d) {
/* Line 506, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return d-'0';
}

static inline long ag_rp_91(PCB_DECL, long x, int d) {
/* Line 507, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_92(PCB_DECL, int d) {
/* Line 510, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return d-'0';
}

static inline long ag_rp_93(PCB_DECL, long x, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_94(PCB_DECL, long x, int d) {
/* Line 514, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_95(PCB_DECL, long x, int d) {
/* Line 515, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_96(PCB_DECL) {
/* Line 518, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 0;
}

static inline long ag_rp_97(PCB_DECL, long n, int d) {
/* Line 519, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_98(PCB_DECL) {
/* Line 522, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 0;
}

static inline long ag_rp_99(PCB_DECL, long n, int d) {
/* Line 523, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_100(PCB_DECL, int x) {
/* Line 526, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return x - '0';
}

static inline int ag_rp_101(PCB_DECL, int d) {
/* Line 527, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_102(PCB_DECL, Value &s, Value &e) {
/* Line 533, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(s+=e);
}

static inline Value ag_rp_103(PCB_DECL, AgString &b) {
/* Line 536, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return VALUE(b);
}

static inline AgString ag_rp_104(PCB_DECL) {
/* Line 539, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return AgString();
}

static inline AgString ag_rp_105(PCB_DECL, AgString &s, int c) {
/* Line 540, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return s.append(c);
}

static inline int ag_rp_106(PCB_DECL) {
/* Line 552, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\'';
}

static inline int ag_rp_107(PCB_DECL) {
/* Line 553, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '"';
}

static inline int ag_rp_108(PCB_DECL) {
/* Line 554, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\?';
}

static inline int ag_rp_109(PCB_DECL) {
/* Line 555, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\\';
}

static inline int ag_rp_110(PCB_DECL) {
/* Line 556, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\a';
}

static inline int ag_rp_111(PCB_DECL) {
/* Line 557, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\b';
}

static inline int ag_rp_112(PCB_DECL) {
/* Line 558, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\f';
}

static inline int ag_rp_113(PCB_DECL) {
/* Line 559, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\n';
}

static inline int ag_rp_114(PCB_DECL) {
/* Line 560, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\r';
}

static inline int ag_rp_115(PCB_DECL) {
/* Line 561, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\t';
}

static inline int ag_rp_116(PCB_DECL) {
/* Line 562, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return '\v';
}

static inline int ag_rp_117(PCB_DECL, int d) {
/* Line 568, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return d-'0';
}

static inline int ag_rp_118(PCB_DECL, int n, int d) {
/* Line 571, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_119(PCB_DECL, int n, int d) {
/* Line 574, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_120(PCB_DECL, int d) {
/* Line 577, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return d;
}

static inline int ag_rp_121(PCB_DECL, int n, int d) {
/* Line 578, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 16*n + d;
}

static inline int ag_rp_122(PCB_DECL, int d) {
/* Line 581, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return d-'0';
}

static inline int ag_rp_123(PCB_DECL, int d) {
/* Line 582, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_124(PCB_DECL, int c) {
/* Line 591, W:/agdev/agex/packages/xidek/pll/combine/dxi.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0,
  0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4, 5, 0, 5, 0, 0, 0, 0, 5, 0, 0,
  0, 5, 5, 0, 0, 0, 0, 4, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 5, 4, 0, 0, 0,
  5, 0, 5, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 8, 8, 0, 0, 5, 0, 0, 0,
  0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 8, 0,
  0, 8, 0, 5, 5, 0, 0, 0, 0, 8, 0, 0, 0, 5, 0, 0, 8, 0, 0, 0, 0, 4, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<Value> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<ForControl >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dxi_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  1,  1,  1,  2,  3,  4,  5,  0,  0,  0,  0,  6,  7,  8,  0,  9,
   10, 11,  0,  0,  0, 12,  0,  0, 13,  0,  0,  0, 14,  0,  0,  0, 15, 16,
   17, 18, 19, 20, 21, 22, 23,  0, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
   34, 35, 36, 37,  0,  0, 38, 39, 40, 41, 42, 43,  0, 44, 45, 46, 47,  0,
   48, 49, 50, 51, 52, 53, 54,  0, 55, 56, 57,  0, 58, 59,  0, 60,  0, 61,
   62, 63, 64, 65, 66, 67,  0, 68, 69, 70, 71,  0, 72,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0, 73, 74,  0, 75, 76, 77, 78, 79, 80, 81, 82,  0,
    0, 83, 84, 85, 86,  0,  0,  0, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96,
   97, 98,  0, 99,100,101,102,  0,  0,  0,  0,  0,103,104,105,106,107,108,
  109,110,111,112,113,  0,  0,  0,114,115,116,117,118,119,120,121
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,174,  0, 21,  0, 22,  0, 23,  0, 24,  0,179,  0,183,  0,184,
  0,185,  0,186,  0,187,  0,188,  0,189,  0,191,  0,193,  0,192,
  0,194,  0,196,  0,195,  0,197,  0,199,  0,202,  0,205,  0,214,
  0,215,  0,218,  0,219,  0,220,  0,221,  0,222,  0,223,  0,225,
  0,226,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 67, 84,255, 47, 82, 83, 87,255, 42,255, 42, 47,255, 61,
   62,255, 78, 82,255, 69, 89,255, 85,255, 73, 79, 85,255, 76, 78,255, 79,
   85,255, 67, 78,255, 79,255, 76,255, 84,255, 65,255, 69,255, 80, 84,255,
   69,255, 76, 82,255, 65, 82,255, 67, 72, 84,255, 72, 79,255, 76,255, 69,
  255, 76,255, 73,255, 72,255, 42, 47, 48, 58, 60, 62, 65, 66, 68, 69, 70,
   73, 76, 77, 78, 79, 80, 82, 83, 84, 85, 87, 88,255, 67, 84,255, 82, 83,
   87,255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70,
   73, 80, 82, 83, 87,255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47,
   65, 66, 68, 70, 73, 80, 82, 83, 85, 87,255, 42, 47,255, 47, 48, 78,255,
   42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73, 76,
   80, 82, 83, 87,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73,
   80, 82, 83, 87,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73,
   80, 82, 83, 85, 87,255, 48, 78,255, 79, 85,255, 80, 84,255, 69,255, 65,
   66, 68, 70, 73, 76, 80, 82, 83, 87,255, 42, 47,255, 61, 62,255, 42, 47,
   58, 60, 62,255, 42, 47,255, 47,255, 42, 47,255, 79, 85,255, 80, 84,255,
   69,255, 47, 65, 66, 68, 69, 70, 73, 76, 80, 82, 83, 87,255, 42, 47,255,
   76, 78,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73,
   80, 82, 83, 85, 87,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 69,
   70, 73, 80, 82, 83, 87,255, 58,255, 68,255, 34, 39, 63, 65, 66, 70, 78,
   82, 84, 86, 88, 92,255, 92,255, 42, 47,255, 61, 62,255, 73, 79,255, 76,
   82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 65, 66, 68, 77, 79, 83, 84,
   88,255, 42, 47,255, 61, 62,255, 76, 82,255, 72,255, 72, 79,255, 42, 47,
   60, 62, 77, 79, 83, 84, 88,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,
  255, 72, 79,255, 42, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,
  255, 47, 48, 68, 76, 78,255, 48, 68, 76, 78,255, 61, 62,255, 73, 79,255,
   76, 82,255, 72,255, 72, 79,255, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,
  255, 61, 62,255, 72, 79,255, 60, 62, 66, 68, 79, 84, 88,255, 84,255, 76,
   78,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 69, 70, 73, 80, 82,
   83, 85, 87,255, 42, 47,255, 61, 62,255, 69, 89,255, 73, 79,255, 76, 82,
  255, 72,255, 72, 79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83, 84,
   88,255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62, 65,
   66, 68, 77, 79, 83, 84, 88,255, 66, 84,255, 66,255, 84,255, 79, 85,255,
   80, 84,255, 69,255, 65, 66, 68, 69, 70, 73, 76, 80, 82, 83, 87,255, 69,
  255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,7,2,7,4,3,4,0,0,4,0,0,4,7,7,4,7,5,4,7,4,7,6,7,4,7,7,4,
  7,7,4,7,7,4,2,4,7,4,6,4,2,4,2,4,2,7,4,2,4,5,5,4,7,7,4,7,2,2,4,7,5,4,7,
  4,6,4,2,4,2,4,2,4,3,2,3,3,2,3,2,2,2,2,2,7,2,7,7,7,7,2,2,2,7,2,7,4,7,7,
  4,7,2,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,2,7,7,4,0,0,4,7,7,4,7,7,
  4,2,4,2,7,7,7,2,7,7,2,7,7,7,4,0,0,4,2,3,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,
  7,7,2,7,7,7,2,7,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,7,4,7,7,4,7,7,4,2,
  4,7,7,7,2,7,7,2,7,7,7,4,3,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,7,2,7,7,4,0,
  0,4,0,0,4,3,2,3,2,3,4,0,0,4,2,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,7,2,7,7,
  7,2,7,7,4,0,0,4,7,7,4,7,7,4,7,7,4,2,4,2,7,7,7,2,2,7,7,2,7,7,7,4,7,7,4,
  7,7,4,2,4,7,7,7,7,2,7,7,2,7,7,4,3,4,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,
  0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,2,7,4,0,0,4,0,0,
  4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,
  3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,7,5,4,5,5,4,2,4,
  7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,4,7,4,7,7,4,7,7,
  4,7,7,4,2,4,7,7,7,2,2,7,7,2,7,7,7,4,0,0,4,0,0,4,7,5,4,7,5,4,5,5,4,2,4,
  7,5,4,3,2,3,2,3,7,2,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,0,0,4,0,0,4,7,
  5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,7,7,4,7,4,7,4,7,7,4,7,
  7,4,2,4,7,7,7,7,2,7,7,7,2,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0,106,111,  0,  2,  8,  0,  0,  6,  0,  4,  0,110,  0,106,111,  0,207,
  211,  0, 54, 12,  0, 34, 26,  0, 64,  0, 50,  0, 38,  0, 32, 36,  0, 20,
   44,  0, 22, 62,  0,  0,  0,  6,  0, 14,  0,  0,  0,  0,  0,  0, 10,  0,
    0,  0, 56, 58,  0,  8, 42,  0,  2,  0,  0,  0, 30, 24,  0,  4,  0, 18,
    0,  0,  0,  0,  0,  0,  0,224,  0,135,190,  0,209,  0,  0,  0,  0,  0,
   28,  0, 52, 60, 46, 40,  0,  0,  0, 16,  0, 48,  0,  2,  8,  0,  6,  0,
    4,  0,106,111,  0, 20, 44,  0, 14, 10,  0,  0,  0,  0, 12, 34, 38,  0,
   28, 40,  0, 42, 18,  0,106,111,  0, 20, 44,  0, 14, 10,  0,  0,  0,  0,
   12, 34, 38,  0, 28, 40,  0, 42, 16, 18,  0,106,111,  0,  0,135, 60,  0,
  106,111,  0, 20, 44,  0, 14, 10,  0,  0,  0,  0, 12, 34, 38,  0, 28, 22,
   40,  0, 42, 18,  0, 20, 44,  0, 14, 10,  0,  0,  0, 12, 34, 38,  0, 28,
   40,  0, 42, 18,  0, 20, 44,  0, 14, 10,  0,  0,  0, 12, 34, 38,  0, 28,
   40,  0, 42, 16, 18,  0,135, 60,  0, 20, 44,  0, 14, 10,  0,  0,  0, 12,
   34, 38,  0, 28, 22, 40,  0, 42, 18,  0,106,111,  0,207,211,  0,224,  0,
  190,  0,209,  0,106,111,  0,  0,  0,106,111,  0, 20, 44,  0, 14, 10,  0,
    0,  0,  0, 12, 34, 38, 36,  0, 28, 22, 40,  0, 42, 18,  0,106,111,  0,
   32, 36,  0, 20, 44,  0, 14, 10,  0,  0,  0,  0, 12, 34, 38,  0,  0, 28,
   40,  0, 42, 16, 18,  0, 20, 44,  0, 14, 10,  0,  0,  0, 12, 34, 38, 36,
    0, 28, 40,  0, 42, 18,  0,190,  0,  0,  0,147,146,148,150,151,152,153,
  154,155,156,161,149,  0,  0,  0,106,111,  0,207,211,  0, 50,  0,  0, 56,
   58,  0,  0,  0, 30, 24,  0,224,  0,  0,209, 54, 26,  0, 52, 46,  0,  0,
   48,  0,106,111,  0,207,211,  0, 56, 58,  0,  0,  0, 30, 24,  0,224,  0,
    0,209, 52, 46,  0,  0, 48,  0,207,211,  0, 50,  0,  0, 56, 58,  0,  0,
    0, 30, 24,  0,224,  0,209, 54, 26,  0, 52, 46,  0,  0, 48,  0,106,111,
    0,  0,135, 64, 62, 60,  0,135, 64, 62, 60,  0,207,211,  0, 50,  0,  0,
   56, 58,  0,  0,  0, 30, 24,  0,  0,209, 54, 26,  0, 52, 46,  0,  0, 48,
    0,207,211,  0, 30, 24,  0,  0,209, 26,  0, 46,  0, 48,  0, 30,  0, 32,
   36,  0, 20, 44,  0, 14, 10,  0,  0,  0, 12, 34, 38,  0,  0, 28, 40,  0,
   42, 16, 18,  0,106,111,  0,207,211,  0, 34, 26,  0, 50,  0,  0, 56, 58,
    0,  0,  0, 30, 24,  0,224,  0,135,  0,209, 54,  0,  0, 52, 46,  0,  0,
   48,  0,135,  0, 30, 24,  0, 26,  0, 46,  0, 48,  0,106,111,  0,207,211,
    0, 50,  0,  0, 56, 58,  0,  0,  0, 30, 24,  0,224,  0,190,  0,209, 54,
   26,  0, 52, 46,  0,  0, 48,  0, 26, 24,  0, 34,  0, 24,  0, 20, 44,  0,
   14, 10,  0,  0,  0, 12, 34, 38, 36,  0, 28, 22, 40,  0, 42, 18,  0, 36,
    0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 10, 15,  0,  1,  0,  4, 23,  0, 32,  0,  0,  0,  0,  0,
    0,  0, 42, 44,  0, 48,  0,  0, 54,  0, 52, 26, 58,  0, 61, 64,  0, 66,
   68,  0, 77, 80,  0, 38,  0, 95,  0, 43,  0, 45,  0, 47,  0, 49, 99,  0,
   51,  0,  0,  0,  0,108,115,  0,103, 56, 59,  0,119,  0,  0,127,  0, 69,
    0, 71,  0, 73,  0, 75,  0, 34, 14, 36, 38, 17, 40, 20, 23, 28, 32, 35,
   75, 41, 82, 85, 88, 90, 54, 62, 66,122, 77,131,  0,144,149,  0,134,103,
  157,  0,  0,  0,  0,180,182,  0,196,200,  0,116,  0,110,166,171,176,113,
  189,191,119,204,210,  0,  0,  0,  0,229,231,  0,245,249,  0,138,  0,132,
  215,220,225,135,238,240,141,253,259,264,  0,  0,  0,  0,155,269,271,  0,
    0,  0,  0,288,290,  0,309,313,  0,168,  0,162,274,279,284,165,297,299,
  304,171,317,323,  0,342,344,  0,358,362,  0,188,  0,328,333,338,185,351,
  353,191,366,372,  0,391,393,  0,407,411,  0,206,  0,377,382,387,203,400,
  402,209,415,421,426,  0,431,433,  0,450,452,  0,471,475,  0,228,  0,436,
  441,446,225,459,461,466,231,479,485,  0,  0,  0,  0,  0,  0,  0,490,244,
  492,247,494,  0,  0,  0,  0,256,  0,  0,  0,  0,513,515,  0,534,538,  0,
  267,  0,261,496,501,506,510,264,522,524,529,270,542,548,  0,  0,  0,  0,
  567,570,  0,572,574,  0,588,592,  0,294,  0,285,553,558,563,288,291,581,
  583,297,596,602,607,  0,629,631,  0,645,649,  0,315,  0,612,617,622,626,
  312,638,640,318,653,659,  0,664,  0,666,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,335,  0,  0,  0,  0,  0,  0,  0,677,  0,  0,  0,
    0,  0,359,  0,684,  0,  0,668,350,353,670,672,675,356,679,682,362,364,
  687,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,386,  0,699,  0,  0,690,380,
  383,692,694,697,389,391,702,  0,  0,  0,  0,714,  0,  0,  0,  0,  0,410,
    0,721,  0,  0,705,404,707,709,712,407,716,719,413,415,724,  0,  0,  0,
    0,430,727,729,735,739,  0,742,744,750,754,  0,  0,  0,  0,764,  0,  0,
    0,  0,  0,450,  0,771,  0,  0,444,757,759,762,447,766,769,453,455,774,
    0,  0,  0,  0,785,  0,  0,469,777,779,781,783,472,788,  0,791,  0,809,
  812,  0,814,816,  0,830,834,  0,491,  0,795,800,805,485,488,823,825,494,
  838,844,849,  0,  0,  0,  0,  0,  0,  0,863,  0,  0,867,  0,  0,  0,  0,
    0,520,  0,874,  0,  0,854,508,856,511,858,860,514,517,869,872,523,525,
  877,  0,880,  0,888,  0,  0,882,884,886,544,891,  0,  0,  0,  0,  0,  0,
    0,905,  0,  0,  0,  0,  0,562,  0,912,  0,  0,894,553,896,556,898,900,
  903,559,907,910,565,567,915,  0,918,920,  0,922,  0,927,  0,946,948,  0,
  967,971,  0,594,  0,929,934,939,943,591,955,957,962,597,975,981,  0,986,
    0
};

static const unsigned short ag_key_index[] = {
    7,  0, 12, 79,106,  0,  0, 12, 12,121,143,158,173,193,211,222,233,250,
  259,259,259,158,259,272,299,299,  0,  0,259,222,  0,158,320,299,331,  0,
  158,  0,  0,  0,  0,222,  0,333,222,193,211,348,  0,  0,348,367,  0,367,
  394,367,367,367,367,367,418,418,433,418,439,158,158,158,418,222,222,222,
  458,475,333,259,  0,193,  0,259,  0,  0,320,259,  0,158,222,158,222,331,
    0,158,222,  0,  0,299,483,496,158,222,348,367,  0,  0,348,348,348,  0,
  367,  0,  0,222,259,  0,259,  0,  0,158,222,158,222,158,222,158,222,158,
  222,158,222,158,222,158,222,158,222,158,222,222,222,158,222,158,222,158,
  222,158,222,158,222,158,222,193,  0,193,  0,  0,  0,  0,299,  0,222,222,
  121,121,193,  0,  0,  0,  0,  0,528,542,542,458,458,458,458,547,547,547,
  547,547,547,  0,  0,259,  0,  0,570,584,  0,367,367,222,587,  0,  0,158,
  222,589,  0,587,  0,158,222,599,320,611,320
};

static const unsigned char ag_key_ends[] = {
69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 47,0, 42,0, 
88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 65,76,0, 71,0, 79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 
79,79,80,0, 85,82,78,0, 82,73,80,84,0, 84,69,77,69,78,84,0, 
85,67,84,0, 69,78,0, 78,84,73,76,0, 79,79,80,0, 79,82,0, 
69,80,69,65,84,76,79,79,80,0, 82,73,80,84,0, 
65,84,69,77,69,78,84,0, 72,73,76,69,76,79,79,80,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
72,73,76,69,0, 42,0, 61,0, 61,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 78,68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
79,67,65,76,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 83,69,0, 68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
78,84,73,76,0, 72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 78,68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
72,73,76,69,0, 61,0, 79,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 42,0, 88,0, 
61,0, 78,68,0, 71,73,78,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 88,0, 89,0, 79,0, 82,0, 69,78,0, 79,82,0, 42,0, 
61,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 89,0, 79,0, 69,71,73,78,0, 79,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 79,67,65,76,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 78,68,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   14,167,167,167,167,167,167,167,167,166,115,166,166,166,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,166,167,138,167,
  167,167,167,163,204,203,216,212,198,213,227,217,133,168,168,168,168,168,
  168,168,132,132,167,176,206,210,208,167,167,169,169,169,169,119,169,170,
  170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,
  170,181,160,180,167,170,167,169,169,169,169,119,169,170,170,170,170,170,
  170,170,170,170,170,170,170,170,170,170,170,170,170,170,170,201,167,200,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,167,
  167,167,167,167
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dxi_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dxi_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dxi_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
166,115,111,106,24,23,22,21,0,1,171,172,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,160,138,133,132,119,115,0,113,114,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,160,138,133,132,119,115,110,0,108,109,
166,115,111,106,0,1,
24,23,22,21,0,7,8,9,10,11,173,175,177,178,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,160,138,133,132,119,0,
115,0,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,160,138,133,132,119,115,0,
110,0,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,166,119,115,111,106,
  0,1,171,172,
205,202,199,197,196,193,187,186,185,184,183,179,176,170,169,166,119,115,111,
  106,0,1,171,172,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
205,202,199,197,196,193,188,187,186,184,183,179,176,170,169,166,119,115,111,
  106,14,0,1,171,172,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,0,4,17,20,25,26,
  27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,68,
  182,
205,202,199,197,196,193,187,186,185,184,183,179,176,170,169,119,0,13,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,15,41,
  52,69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
205,202,199,197,196,193,188,187,186,184,183,179,176,170,169,119,14,0,12,50,
227,224,217,216,213,212,211,210,209,208,207,206,204,203,201,200,198,190,181,
  180,176,170,169,168,166,133,132,119,115,111,106,0,1,171,172,
170,169,166,119,115,111,106,0,1,171,172,
170,169,166,119,115,111,106,0,1,171,172,
201,170,169,166,119,115,111,106,0,1,171,172,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
170,169,166,119,115,111,106,0,1,171,172,
205,202,199,197,196,195,193,188,187,186,184,183,179,176,170,169,166,119,115,
  111,106,0,1,171,172,
205,202,199,197,196,195,194,193,187,186,185,184,183,179,176,170,169,166,119,
  115,111,106,14,0,1,171,172,
205,202,199,197,196,195,194,193,187,186,185,184,183,179,176,170,169,166,119,
  115,111,106,14,0,1,171,172,
170,169,119,0,4,182,
201,170,169,119,0,4,66,182,
170,169,166,119,115,111,106,0,1,171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
170,169,119,0,4,182,
227,223,213,212,204,176,170,169,168,166,163,138,135,133,132,119,115,111,106,
  0,1,171,172,
205,202,199,197,196,195,193,187,186,184,183,179,176,170,169,119,0,13,
205,202,199,197,196,195,194,193,187,186,185,184,183,179,176,170,169,166,119,
  115,111,106,14,0,1,171,172,
227,190,181,0,40,53,101,
170,169,119,0,4,52,182,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
176,0,20,
170,169,119,0,4,182,
198,176,0,20,63,
198,176,0,20,63,
227,223,213,212,204,176,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,36,
  41,52,69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,
  228,229,230,231,
176,0,20,
174,0,16,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,0,4,17,20,25,26,
  27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,68,
  182,
205,202,199,197,196,193,187,186,185,184,183,179,176,170,169,119,0,4,17,18,
  20,25,26,27,29,30,31,32,33,34,35,37,38,39,43,44,45,46,47,48,49,52,57,60,
  62,64,65,68,182,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,161,160,156,155,154,153,152,151,150,149,148,147,146,138,133,
  132,119,0,139,
168,133,132,0,128,
168,133,132,0,122,128,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,161,160,156,155,154,153,152,151,150,149,148,147,146,138,133,132,
  119,0,142,143,144,145,157,158,159,164,
224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,203,
  198,192,191,189,180,176,174,166,138,115,111,106,0,1,171,172,
227,119,0,
119,0,
169,168,133,132,119,0,136,
227,224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,
  203,198,192,191,189,180,176,174,168,166,133,132,119,115,111,106,0,
227,224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,
  203,198,192,191,189,180,176,174,168,166,133,132,119,115,111,106,0,128,
224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,203,
  198,192,191,189,180,176,174,166,115,111,106,0,1,171,172,
224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,203,
  198,192,191,189,180,176,174,166,115,111,106,0,1,171,172,
224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,203,
  198,192,191,189,180,176,174,166,115,111,106,0,1,171,172,
204,0,69,
138,0,5,231,
227,226,225,223,213,212,204,203,170,169,168,166,163,138,135,133,132,119,115,
  111,106,0,1,171,172,
227,181,0,40,101,
227,226,225,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,
  41,52,69,73,80,81,82,85,93,94,95,97,98,99,100,104,118,121,123,124,125,
  129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
224,0,96,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
222,221,220,219,218,217,216,0,86,87,88,89,90,91,92,
215,214,213,212,211,210,209,208,207,206,0,74,75,76,77,78,79,81,82,83,84,
174,0,16,
170,169,166,119,115,111,106,0,1,171,172,
170,169,119,0,4,51,182,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,14,0,13,
204,0,69,
170,169,166,119,115,111,106,0,1,171,172,
170,169,119,0,4,51,182,
201,170,169,119,0,4,51,66,182,
205,202,199,197,196,195,193,187,186,184,183,179,176,170,169,119,0,4,17,20,
  25,26,27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,61,62,
  64,65,68,182,
170,169,166,119,115,111,106,0,1,171,172,
170,169,119,0,4,182,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
227,190,181,0,40,53,101,
181,0,40,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
170,169,119,0,4,182,
176,0,20,
205,202,199,197,196,195,194,193,187,186,185,184,183,179,176,170,169,166,119,
  115,111,106,14,0,1,171,172,
192,0,58,
194,0,28,59,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,19,41,
  52,69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
227,217,216,213,212,210,208,206,204,203,201,200,198,181,180,176,170,169,168,
  167,166,163,161,160,156,155,154,153,152,151,150,149,148,147,146,138,133,
  132,119,0,140,142,143,144,145,157,158,159,
168,133,132,0,122,128,
168,133,0,
169,168,133,132,119,0,162,
168,133,0,
168,133,0,
169,168,133,132,119,0,162,
163,0,
168,133,132,0,122,128,
213,212,168,133,132,0,120,126,
213,212,168,133,132,0,120,126,
227,223,213,212,204,203,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,
  52,69,73,80,81,82,85,93,94,95,97,98,102,103,104,118,121,123,124,125,129,
  182,228,229,230,231,
203,166,115,111,106,0,1,171,172,
203,0,70,
203,166,115,111,106,0,1,171,172,
203,0,70,
203,0,70,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,
  231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,
  231,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,
  231,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,230,
  231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,
  73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,229,
  230,231,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,0,4,17,20,25,26,
  27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,68,
  182,
198,176,0,20,63,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,14,0,4,17,20,25,
  26,27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,
  68,182,
170,169,119,0,4,51,182,
200,198,0,63,67,
198,0,63,
170,169,119,0,4,51,182,
205,202,199,197,196,195,194,193,187,186,185,184,183,179,176,170,169,166,119,
  115,111,106,14,0,1,171,172,
180,0,42,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,166,119,115,111,106,
  0,1,171,172,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,166,119,115,111,106,
  0,1,171,172,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,0,4,20,25,26,27,
  29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,68,
  182,
176,0,20,
168,133,132,0,127,
168,133,132,0,127,
198,0,63,
203,0,70,
227,224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,
  204,203,198,196,192,191,189,180,176,174,170,169,168,166,163,138,135,133,
  132,119,115,111,106,0,1,171,172,
227,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,95,97,98,104,
  118,121,123,124,125,129,182,228,229,230,231,
227,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,52,69,95,97,98,104,
  118,121,123,124,125,129,182,228,229,230,231,
222,221,220,219,218,217,216,0,86,87,88,89,90,91,92,
222,221,220,219,218,217,216,0,86,87,88,89,90,91,92,
222,221,220,219,218,217,216,0,86,87,88,89,90,91,92,
222,221,220,219,218,217,216,0,86,87,88,89,90,91,92,
215,214,213,212,0,81,82,83,84,
215,214,213,212,0,81,82,83,84,
215,214,213,212,0,81,82,83,84,
215,214,213,212,0,81,82,83,84,
215,214,213,212,0,81,82,83,84,
215,214,213,212,0,81,82,83,84,
170,169,119,0,4,182,
203,198,0,63,70,
176,170,169,166,119,115,111,106,0,1,171,172,
170,169,119,0,4,51,182,
200,198,0,63,67,
227,224,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208,207,206,
  203,198,192,191,190,189,181,180,176,174,166,115,111,106,0,1,171,172,
191,189,0,54,56,
180,0,42,
168,133,132,0,128,
168,133,132,0,128,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
196,0,71,
198,0,63,
170,169,119,0,4,51,182,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
189,0,55,
176,0,20,
196,0,60,
198,0,63,
227,223,213,212,204,170,169,168,166,163,138,135,133,132,119,115,111,106,0,1,
  171,172,
227,223,213,212,204,170,169,168,163,138,135,133,132,119,0,2,3,4,5,6,41,52,
  69,73,80,81,82,85,93,94,95,97,98,104,118,121,123,124,125,129,182,228,
  229,230,231,
205,202,199,197,196,195,193,188,187,186,184,183,179,176,170,169,119,0,12,50,
  72,
205,202,199,197,196,195,193,187,186,184,183,179,176,170,169,119,0,13,
195,0,61,
205,202,199,197,196,193,187,186,184,183,179,176,170,169,119,0,4,17,20,25,26,
  27,29,30,31,32,33,34,35,37,38,39,43,44,45,47,48,49,52,57,60,62,64,65,68,
  182,

};


static unsigned const char ag_astt[4307] = {
  1,1,1,1,8,8,8,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,
  1,9,9,1,1,5,3,2,2,2,2,7,1,1,1,1,0,1,1,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,5,3,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,7,2,2,2,2,2,1,2,2,2,1,1,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,
  2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,8,8,8,8,8,1,8,8,8,
  8,8,8,8,8,8,5,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,
  10,10,10,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  5,1,1,1,5,7,1,1,3,2,2,2,7,1,1,1,2,2,2,7,1,1,1,5,5,1,5,1,1,1,7,1,1,3,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  5,7,1,1,3,1,1,1,7,1,1,1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,7,3,2,2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,3,1,
  7,2,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,1,2,1,1,1,2,2,1,1,
  1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,
  3,1,3,3,3,1,3,2,2,1,1,3,1,1,1,1,1,3,2,2,2,1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,
  10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  2,2,2,2,2,2,2,2,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,4,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,5,5,5,7,4,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,5,5,5,7,2,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,1,5,2,1,5,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,4,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,
  1,1,1,1,1,1,1,1,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,7,1,1,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,1,2,2,2,7,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,3,3,3,3,1,3,2,2,1,1,3,1,1,1,1,1,3,
  2,2,1,1,1,1,1,3,1,1,1,1,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,1,7,1,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,7,2,1,5,1,2,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,7,2,2,2,
  2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,
  1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,7,
  2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,
  7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,
  2,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,2,1,2,2,2,1,1,2,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,3,3,3,3,1,3,
  2,2,1,1,3,1,1,1,1,1,3,2,2,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,1,1,7,1,1,1,4,
  1,2,2,2,7,2,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,7,
  2,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,
  1,2,2,2,7,2,2,2,2,1,2,2,2,1,1,2,1,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,7,2,
  2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,2,2,2,1,1,2,2,2,2,7,
  2,2,1,1,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,
  1,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,
  1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,
  1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,4,1,1,1,1,2,2,2,7,2,1,1,1,7,1,1,5,5,5,1,
  5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,7,1,1,1,7,1,10,10,10,4,2,10,10,
  10,4,2,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,4,7,1,1,4,1,2,2,2,4,2,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,7,2,1,7,1,1,4,1,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,8,8,8,8,5,8,1,8,8,8,8,8,8,8,
  8,8,7,1,1,1,8,8,8,8,8,5,8,8,8,8,8,8,8,8,8,8,7,1,1,7,2,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,4,2,3,3,3,3,1,3,2,2,1,1,3,1,1,1,1,1,3,2,2,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
3,3,1,2,4,4,4,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
307,307,1,2,309,307,
8,7,6,5,4,16,15,14,13,0,12,11,10,9,
110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,
  110,110,110,110,110,110,110,110,112,
113,6,
105,105,105,105,105,105,105,105,105,105,105,105,105,105,105,105,105,105,105,
  105,105,105,105,105,105,105,105,105,107,
108,8,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,9,3,3,
  315,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,10,
  3,3,314,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,11,3,3,312,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,308,
  12,3,3,310,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,13,91,4,4,4,4,45,4,17,18,43,
  42,4,41,40,39,38,37,4,17,18,35,35,34,44,32,30,29,27,26,17,
46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,14,46,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,15,99,100,60,61,102,74,35,
  63,64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
77,77,77,77,77,77,75,77,77,77,77,77,77,77,77,77,33,16,77,76,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,115,115,115,3,115,115,115,3,1,2,17,3,3,319,
308,308,3,308,3,1,2,18,3,3,324,
308,308,3,308,3,1,2,19,3,3,342,
308,308,308,3,308,3,1,2,20,3,3,339,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,21,3,3,336,
308,308,3,308,3,1,2,22,3,3,334,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,
  23,3,3,333,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,
  1,2,308,24,3,3,323,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,
  1,2,308,25,3,3,321,
114,114,114,26,78,17,
79,114,114,114,27,81,80,17,
308,308,3,308,3,1,2,28,3,3,320,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,29,99,100,60,61,102,46,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
114,114,114,30,44,17,
308,308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,31,3,3,
  316,
82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,32,82,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,
  1,2,308,33,3,3,313,
83,87,85,34,86,88,84,
114,114,114,35,91,89,17,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,36,3,3,330,
33,37,27,
114,114,114,38,90,17,
91,33,39,25,92,
91,33,40,24,93,
49,65,66,67,62,57,114,114,134,50,47,142,140,134,114,41,99,100,60,61,102,94,
  94,63,64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,
  51,
33,42,20,
95,43,19,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,44,99,100,60,61,102,96,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,45,91,13,97,13,97,45,97,17,
  18,43,42,97,41,40,39,38,37,97,17,18,35,35,34,44,32,30,29,27,26,17,
19,20,21,22,23,36,18,24,98,25,28,31,33,114,114,114,46,91,10,99,10,10,10,45,
  10,17,18,43,42,10,41,40,39,38,37,10,17,30,18,35,35,34,44,32,30,29,27,26,
  17,
149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
  149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,149,
  149,149,47,100,
139,139,139,123,139,
101,101,101,49,121,101,
107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,
  107,107,103,102,166,165,164,163,162,161,160,159,158,157,156,107,107,107,
  107,50,107,107,107,106,105,104,107,107,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,3,308,3,1,2,51,3,3,368,
108,109,52,
110,116,
145,144,144,144,145,133,143,
124,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,132,
  132,132,132,132,132,132,132,132,141,132,141,138,124,132,132,132,55,
122,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
  131,131,131,131,131,131,131,131,135,131,135,135,122,131,131,131,56,135,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,3,3,1,2,57,3,3,367,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,3,3,1,2,58,3,3,366,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,3,3,1,2,59,3,3,365,
62,91,111,
47,101,147,51,
308,308,308,308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,
  62,3,3,341,
83,85,87,86,84,
49,112,114,65,66,67,62,114,114,134,50,47,142,140,134,114,64,99,100,60,61,
  102,116,63,64,73,72,71,70,72,72,69,68,68,68,115,113,61,53,52,56,48,55,
  54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,65,3,3,360,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,66,3,3,350,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,67,3,3,349,
117,83,118,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,69,99,100,60,61,102,63,64,
  71,70,82,82,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,70,99,100,60,61,102,63,64,
  71,70,81,81,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,71,99,100,60,61,102,63,64,
  71,70,80,80,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
119,121,123,125,127,129,131,66,132,130,128,126,124,122,120,
133,135,66,67,139,141,143,145,147,149,59,150,148,146,144,142,140,138,137,
  136,134,
95,74,151,
308,308,3,308,3,1,2,75,3,3,325,
114,114,114,76,55,152,17,
153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,77,153,
62,78,154,
308,308,3,308,3,1,2,79,3,3,338,
114,114,114,80,55,155,17,
79,114,114,114,81,55,156,157,17,
19,20,21,22,23,158,36,18,24,25,28,31,33,114,114,114,82,91,10,10,10,10,45,10,
  17,18,43,42,10,41,40,39,38,37,10,17,18,35,35,34,44,32,43,30,29,27,26,17,
308,308,3,308,3,1,2,83,3,3,364,
114,114,114,84,93,17,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,85,3,3,318,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,86,99,100,60,61,102,159,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,87,3,3,327,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,88,99,100,60,61,102,42,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
83,87,85,89,86,160,84,
85,90,161,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,91,3,3,335,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,92,99,100,60,61,102,47,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
114,114,114,93,45,17,
33,94,23,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,
  1,2,308,95,3,3,311,
162,96,40,
163,12,164,41,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,98,3,3,322,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,99,99,100,60,61,102,165,36,
  63,64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,103,102,166,165,164,163,162,161,160,159,158,157,156,148,150,
  150,150,100,150,150,150,150,106,105,104,150,
101,101,101,129,130,101,
170,170,102,
176,175,175,175,176,103,173,
172,172,168,
171,171,167,
176,175,175,175,176,155,174,
177,107,
101,101,101,120,119,101,
166,167,167,167,167,109,118,167,
166,167,167,167,167,110,117,167,
49,65,66,67,62,95,114,114,134,50,47,142,140,134,114,111,99,100,60,61,102,97,
  63,64,73,72,71,70,72,72,69,68,68,68,169,168,61,53,52,56,48,55,54,17,59,
  58,57,51,
308,3,3,1,2,112,3,3,363,
170,113,171,
308,3,3,1,2,114,3,3,362,
170,115,172,
170,116,85,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,117,3,3,361,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,118,99,100,60,61,102,63,64,
  71,70,84,84,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,119,3,3,359,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,120,99,100,60,61,102,63,64,
  71,70,78,78,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,121,3,3,358,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,122,99,100,60,61,102,63,64,
  71,70,77,77,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,123,3,3,357,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,124,99,100,60,61,102,63,64,
  71,70,76,76,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,125,3,3,356,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,126,99,100,60,61,102,63,64,
  71,70,75,75,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,127,3,3,355,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,128,99,100,60,61,102,63,64,
  71,70,74,74,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,129,3,3,354,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,130,99,100,60,61,102,63,64,
  71,70,73,73,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,131,3,3,353,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,132,99,100,60,61,102,63,64,
  71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,133,3,3,352,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,134,99,100,60,61,102,63,64,
  173,71,70,173,173,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,135,3,3,351,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,136,99,100,60,61,102,63,64,
  174,71,70,174,174,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,137,99,100,60,61,102,63,64,
  175,71,70,175,175,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,138,99,100,60,61,102,63,64,
  176,71,70,176,176,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,139,3,3,348,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,140,99,100,60,61,102,63,64,
  177,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,141,3,3,347,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,142,99,100,60,61,102,63,64,
  178,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,143,3,3,346,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,144,99,100,60,61,102,63,64,
  179,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,145,3,3,345,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,146,99,100,60,61,102,63,64,
  180,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,147,3,3,344,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,148,99,100,60,61,102,63,64,
  181,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,149,3,3,343,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,150,99,100,60,61,102,63,64,
  182,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,151,91,2,2,2,2,45,2,17,18,
  43,42,2,41,40,39,38,37,2,17,18,35,35,34,44,32,30,29,27,26,17,
91,33,152,34,183,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,1,153,91,10,10,10,10,45,10,
  17,18,43,42,10,41,40,39,38,37,10,17,18,35,35,34,44,32,30,29,27,26,17,
114,114,114,154,55,184,17,
185,91,155,183,186,
91,51,183,
114,114,114,157,55,187,17,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,
  1,2,308,158,3,3,332,
188,159,92,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,160,99,100,60,61,102,189,
  63,64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,161,99,100,60,61,102,190,
  63,64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,162,3,3,
  329,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,3,1,2,163,3,3,
  331,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,164,91,15,14,15,45,15,17,18,
  43,42,15,41,40,39,38,37,15,17,18,35,35,34,44,32,30,29,27,26,17,
33,165,3,
136,136,136,166,191,
136,136,136,167,192,
91,96,193,
170,169,94,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,308,308,308,308,308,308,3,308,308,308,308,
  308,308,3,1,2,170,3,3,340,
49,62,114,114,134,50,47,142,140,134,114,171,99,100,60,61,102,63,64,90,90,90,
  61,53,52,56,48,55,54,17,59,58,57,51,
49,62,114,114,134,50,47,142,140,134,114,172,99,100,60,61,102,63,64,89,89,89,
  61,53,52,56,48,55,54,17,59,58,57,51,
119,121,123,125,127,129,131,70,132,130,128,126,124,122,120,
119,121,123,125,127,129,131,69,132,130,128,126,124,122,120,
119,121,123,125,127,129,131,68,132,130,128,126,124,122,120,
119,121,123,125,127,129,131,67,132,130,128,126,124,122,120,
133,135,66,67,65,138,137,136,134,
133,135,66,67,64,138,137,136,134,
133,135,66,67,63,138,137,136,134,
133,135,66,67,62,138,137,136,134,
133,135,66,67,61,138,137,136,134,
133,135,66,67,60,138,137,136,134,
114,114,114,183,56,17,
170,91,184,183,194,
308,308,308,3,308,3,1,2,185,3,3,337,
114,114,114,186,55,195,17,
185,91,187,183,196,
308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,308,
  308,308,308,308,308,308,308,308,308,308,3,3,1,2,188,3,3,317,
197,38,189,199,198,
188,190,200,
137,137,137,128,137,
137,137,137,127,137,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,193,99,100,60,61,102,98,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
52,194,201,
91,50,183,
114,114,114,49,55,202,17,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,197,3,3,328,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,198,99,100,60,61,102,39,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
203,199,204,
33,200,26,
23,201,205,
91,48,183,
308,308,308,308,308,308,308,308,3,308,308,308,308,308,308,3,1,2,203,3,3,326,
49,65,66,67,62,114,114,134,50,47,142,140,134,114,204,99,100,60,61,102,37,63,
  64,73,72,71,70,72,72,69,68,68,68,61,53,52,56,48,55,54,17,59,58,57,51,
206,206,206,206,206,33,206,75,206,206,206,206,206,206,206,206,206,205,206,
  76,207,
208,208,208,208,208,9,208,208,208,208,208,208,208,208,208,208,206,208,
158,207,53,
19,20,21,22,23,36,18,24,25,28,31,33,114,114,114,54,91,10,10,10,10,45,10,17,
  18,43,42,10,41,40,39,38,37,10,17,18,35,35,34,44,32,30,29,27,26,17,

};


static const unsigned short ag_sbt[] = {
     0,  12,  43,  75,  81,  95, 123, 125, 154, 156, 179, 203, 225, 250,
   296, 314, 360, 380, 415, 426, 437, 449, 471, 482, 507, 534, 561, 567,
   575, 586, 631, 637, 660, 678, 705, 712, 719, 741, 744, 750, 755, 760,
   807, 810, 813, 858, 904, 953, 994, 999,1005,1052,1087,1090,1092,1099,
  1135,1172,1206,1240,1274,1277,1281,1306,1311,1360,1382,1404,1426,1429,
  1471,1513,1555,1570,1591,1594,1605,1612,1630,1633,1644,1651,1660,1708,
  1719,1725,1747,1792,1814,1859,1866,1869,1891,1936,1942,1945,1972,1975,
  1979,2001,2047,2095,2101,2104,2111,2114,2117,2124,2126,2132,2140,2148,
  2196,2205,2208,2217,2220,2223,2245,2287,2309,2351,2373,2415,2437,2479,
  2501,2543,2565,2607,2629,2671,2693,2735,2757,2800,2822,2865,2908,2951,
  2973,3017,3039,3083,3105,3149,3171,3215,3237,3281,3303,3347,3393,3398,
  3445,3452,3457,3460,3467,3494,3497,3542,3587,3610,3633,3678,3681,3686,
  3691,3694,3697,3743,3777,3811,3826,3841,3856,3871,3880,3889,3898,3907,
  3916,3925,3931,3936,3948,3955,3960,3997,4002,4005,4010,4015,4060,4063,
  4066,4073,4095,4140,4143,4146,4149,4152,4174,4219,4240,4258,4261,4307
};


static const unsigned short ag_sbe[] = {
     8,  40,  72,  79,  85, 122, 124, 153, 155, 175, 199, 221, 246, 265,
   312, 328, 377, 411, 422, 433, 445, 467, 478, 503, 530, 557, 564, 571,
   582, 600, 634, 656, 676, 701, 708, 715, 737, 742, 747, 752, 757, 775,
   808, 811, 827, 873, 920, 992, 997,1002,1043,1083,1089,1091,1097,1134,
  1170,1202,1236,1270,1275,1278,1302,1308,1327,1378,1400,1422,1427,1443,
  1485,1527,1562,1580,1592,1601,1608,1628,1631,1640,1647,1655,1676,1715,
  1722,1743,1761,1810,1828,1862,1867,1887,1905,1939,1943,1968,1973,1976,
  1997,2015,2086,2098,2103,2109,2113,2116,2122,2125,2129,2137,2145,2163,
  2201,2206,2213,2218,2221,2241,2259,2305,2323,2369,2387,2433,2451,2497,
  2515,2561,2579,2625,2643,2689,2707,2753,2771,2818,2836,2879,2922,2969,
  2987,3035,3053,3101,3119,3167,3185,3233,3251,3299,3317,3362,3395,3414,
  3448,3454,3458,3463,3490,3495,3511,3556,3606,3629,3648,3679,3684,3689,
  3692,3695,3739,3754,3788,3818,3833,3848,3863,3875,3884,3893,3902,3911,
  3920,3928,3933,3944,3951,3957,3993,3999,4003,4008,4013,4029,4061,4064,
  4069,4091,4109,4141,4144,4147,4150,4170,4188,4236,4256,4259,4276,4307
};


static const unsigned char ag_fl[] = {
  2,4,4,5,2,1,1,1,1,0,2,1,1,2,4,4,1,1,1,2,2,1,1,3,2,2,6,2,1,1,1,1,1,0,3,
  1,1,7,0,2,3,1,3,3,2,3,2,3,6,5,5,3,0,9,2,1,3,0,1,1,3,3,3,3,3,3,1,3,3,3,
  3,1,3,3,3,3,3,3,3,1,2,2,2,1,3,3,1,1,1,4,4,1,4,3,4,0,1,1,3,1,1,1,1,1,1,
  2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,
  1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,
  1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0, 11, 11, 11, 11,173,175,177,178, 13, 13, 17, 17, 25, 25, 26, 26, 29,
   29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 18, 31, 48, 12, 12, 15,
   19, 32, 54, 54, 27, 28, 33, 34, 37, 37, 38, 38, 43, 43, 43, 43, 71, 44,
   72, 51, 51, 36, 36, 41, 41, 41, 41, 41, 41, 41, 73, 73, 73, 73, 73, 80,
   80, 80, 80, 80, 80, 80, 80, 85, 85, 85, 85, 93, 93, 95, 95, 95, 95, 95,
   95, 52, 52, 52, 98,102,102,103,103, 97, 97, 97, 97,  1,108,108,109,109,
    1,113,113,114,114,  1,182,182,229,229,229,118,118,118,121,121,121,126,
  126,120,120,122,122,228,228,228,123,123,127,127,124,124,125,125,129,129,
  136,136,104,104,231,139,139,140,140,142,142,142,143,143,143,143,143,143,
  143,143,143,143,143,144,144,144,157,158,159,145,145,162,162,230,164,164,
  128,105,105,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,
  107,107,107,107,107,107,107,107,107,107,107,107,107,112,112,112,112,112,
  112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,
  112,112,112,112,116,116,116,117,117,117,117,117,117,130,130,131,131,131,
  134,134,137,137,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
  141,141,141,141,141,141,141,141,141,141,141,165,165,165,165,165,165,165,
  165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,
  171,171,172,172,  7, 16,  8, 20,  9, 10, 35, 42, 40,  4, 39, 45, 46, 47,
   49, 50, 55, 53, 56, 58, 57, 59, 61, 60, 62, 63, 64, 67, 66, 65, 70, 69,
   68, 74, 75, 76, 77, 78, 79, 81, 82, 83, 84, 86, 87, 88, 89, 90, 91, 92,
   94, 96, 99,100,101,  2,  3,  6,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VNO AG_WRAP_5(ag_rp_1(PCB_POINTER)); break;
    case 2: ag_rp_5(PCB_POINTER); break;
    case 3: ag_rp_6(PCB_POINTER); break;
    case 4: ag_rp_7(PCB_POINTER); break;
    case 5: ag_rp_8(PCB_POINTER); break;
    case 6: ag_rp_9(PCB_POINTER); break;
    case 7: ag_rp_10(PCB_POINTER); break;
    case 8: ag_rp_11(PCB_POINTER); break;
    case 9: ag_rp_12(PCB_POINTER); break;
    case 10: ag_rp_13(PCB_POINTER); break;
    case 11: ag_rp_14(PCB_POINTER, VW(0, AG_WRAP_7 *)); 
            VWD(0, AG_WRAP_7 *); break;
    case 12: ag_rp_15(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 13: ag_rp_16(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_5 *); break;
    case 14: ag_rp_17(PCB_POINTER); break;
    case 15: ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_8 *)); 
            VWD(1, AG_WRAP_8 *); break;
    case 16: ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 17: ag_rp_20(PCB_POINTER, VW(0, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); break;
    case 18: VNO AG_WRAP_7(ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_5 *), VW(3, AG_WRAP_5 *), VW(4, AG_WRAP_5 *), VW(6, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); VWD(3, AG_WRAP_5 *); VWD(4, AG_WRAP_5 *); VWD(6, AG_WRAP_5 *); break;
    case 19: VNO AG_WRAP_5(ag_rp_22(PCB_POINTER)); break;
    case 20: VNO AG_WRAP_5(ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 21: ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 22: ag_rp_25(PCB_POINTER); break;
    case 23: ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *)); 
            VWD(0, AG_WRAP_5 *); VWD(2, AG_WRAP_5 *); break;
    case 24: ag_rp_27(PCB_POINTER, VW(1, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: ag_rp_28(PCB_POINTER, VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_5 *)); 
            VWD(1, AG_WRAP_5 *); break;
    case 27: ag_rp_30(PCB_POINTER, VW(2, AG_WRAP_5 *)); 
            VWD(2, AG_WRAP_5 *); break;
    case 28: ag_rp_31(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_8 *), VW(5, AG_WRAP_8 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_8 *); VWD(5, AG_WRAP_8 *); break;
    case 29: ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_8 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_8 *); break;
    case 30: ag_rp_33(PCB_POINTER, VW(2, AG_WRAP_8 *), VW(4, AG_WRAP_8 *)); 
            VWD(2, AG_WRAP_8 *); VWD(4, AG_WRAP_8 *); break;
    case 31: ag_rp_34(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_8 *)); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_8 *); break;
    case 32: ag_rp_35(PCB_POINTER, VW(-4, AG_WRAP_4 *), VW(-2, AG_WRAP_8 *)); break;
    case 33: ag_rp_36(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_8 *), VW(7, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_8 *); VWD(7, AG_WRAP_4 *); break;
    case 34: VNO AG_WRAP_4(ag_rp_37(PCB_POINTER)); break;
    case 35: VRO(AG_WRAP_4 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 36: VRO(AG_WRAP_8 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 37: VNO AG_WRAP_5(ag_rp_40(PCB_POINTER)); break;
    case 38: VRO(AG_WRAP_5 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 39: VRO(AG_WRAP_5 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 40: VRO(AG_WRAP_5 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 41: VRO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 42: VRO(AG_WRAP_5 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 43: VRO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 44: VRO(AG_WRAP_5 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 45: VRO(AG_WRAP_5 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 46: VRO(AG_WRAP_5 *, ag_rp_49(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 47: VRO(AG_WRAP_5 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 48: VRO(AG_WRAP_5 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 49: VRO(AG_WRAP_5 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 50: VRO(AG_WRAP_5 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 51: VRO(AG_WRAP_5 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 52: VRO(AG_WRAP_5 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 53: VRO(AG_WRAP_5 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 54: VRO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 55: VNO AG_WRAP_5(ag_rp_58(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 56: VNO AG_WRAP_5(ag_rp_59(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 57: VNO AG_WRAP_5(ag_rp_60(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 58: VRO(AG_WRAP_5 *, ag_rp_61(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 59: VNO AG_WRAP_5(ag_rp_62(PCB_POINTER, VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 60: VRO(AG_WRAP_5 *, ag_rp_63(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 61: VNO AG_WRAP_5(ag_rp_64(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 62: VNO AG_WRAP_5(ag_rp_65(PCB_POINTER, VW(3, AG_WRAP_5 *))); 
            VWD(3, AG_WRAP_5 *); break;
    case 63: VRO(AG_WRAP_4 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 64: VRO(AG_WRAP_5 *, ag_rp_67(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 65: VRO(AG_WRAP_5 *, ag_rp_68(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 66: VRO(AG_WRAP_4 *, ag_rp_69(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 67: VNO AG_WRAP_6(ag_rp_70(PCB_POINTER)); break;
    case 68: VRO(AG_WRAP_5 *, ag_rp_71(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 69: VRO(AG_WRAP_6 *, ag_rp_72(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_5 *))); 
            VWD(2, AG_WRAP_5 *); break;
    case 70: VNO AG_WRAP_5(ag_rp_73(PCB_POINTER, V(0,(long *)))); break;
    case 71: VNO AG_WRAP_5(ag_rp_74(PCB_POINTER, V(0,(double *)))); break;
    case 72: VNO AG_WRAP_5(ag_rp_75(PCB_POINTER, V(0,(int *)))); break;
    case 73: VNO AG_WRAP_4(ag_rp_76(PCB_POINTER, V(0,(int *)))); break;
    case 74: VRO(AG_WRAP_4 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 75: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 76: V(0,(double *)) = ag_rp_79(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 77: V(0,(double *)) = ag_rp_80(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 78: V(0,(double *)) = ag_rp_81(PCB_POINTER, V(0,(double *))); break;
    case 79: V(0,(double *)) = ag_rp_82(PCB_POINTER, V(1,(double *))); break;
    case 80: V(0,(double *)) = ag_rp_83(PCB_POINTER, V(0,(long *))); break;
    case 81: V(0,(double *)) = ag_rp_84(PCB_POINTER, V(0,(long *))); break;
    case 82: V(0,(double *)) = ag_rp_85(PCB_POINTER, V(0,(long *))); break;
    case 83: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(1,(long *))); break;
    case 84: V(0,(long *)) = ag_rp_87(PCB_POINTER, V(1,(long *))); break;
    case 85: V(0,(double *)) = ag_rp_88(PCB_POINTER, V(0,(int *))); break;
    case 86: V(0,(double *)) = ag_rp_89(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 87: V(0,(long *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 88: V(0,(long *)) = ag_rp_91(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 89: V(0,(long *)) = ag_rp_92(PCB_POINTER, V(0,(int *))); break;
    case 90: V(0,(long *)) = ag_rp_93(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 91: V(0,(long *)) = ag_rp_94(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 92: V(0,(long *)) = ag_rp_95(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 93: V(0,(long *)) = ag_rp_96(PCB_POINTER); break;
    case 94: V(0,(long *)) = ag_rp_97(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 95: V(0,(long *)) = ag_rp_98(PCB_POINTER); break;
    case 96: V(0,(long *)) = ag_rp_99(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 97: V(0,(int *)) = ag_rp_100(PCB_POINTER, V(0,(int *))); break;
    case 98: V(0,(int *)) = ag_rp_101(PCB_POINTER, V(0,(int *))); break;
    case 99: VRO(AG_WRAP_5 *, ag_rp_102(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 100: VNO AG_WRAP_5(ag_rp_103(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 101: VNO AG_WRAP_4(ag_rp_104(PCB_POINTER)); break;
    case 102: VRO(AG_WRAP_4 *, ag_rp_105(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 103: V(0,(int *)) = ag_rp_106(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_107(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_108(PCB_POINTER); break;
    case 106: V(0,(int *)) = ag_rp_109(PCB_POINTER); break;
    case 107: V(0,(int *)) = ag_rp_110(PCB_POINTER); break;
    case 108: V(0,(int *)) = ag_rp_111(PCB_POINTER); break;
    case 109: V(0,(int *)) = ag_rp_112(PCB_POINTER); break;
    case 110: V(0,(int *)) = ag_rp_113(PCB_POINTER); break;
    case 111: V(0,(int *)) = ag_rp_114(PCB_POINTER); break;
    case 112: V(0,(int *)) = ag_rp_115(PCB_POINTER); break;
    case 113: V(0,(int *)) = ag_rp_116(PCB_POINTER); break;
    case 114: V(0,(int *)) = ag_rp_117(PCB_POINTER, V(1,(int *))); break;
    case 115: V(0,(int *)) = ag_rp_118(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 116: V(0,(int *)) = ag_rp_119(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 117: V(0,(int *)) = ag_rp_120(PCB_POINTER, V(1,(int *))); break;
    case 118: V(0,(int *)) = ag_rp_121(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 119: V(0,(int *)) = ag_rp_122(PCB_POINTER, V(0,(int *))); break;
    case 120: V(0,(int *)) = ag_rp_123(PCB_POINTER, V(0,(int *))); break;
    case 121: V(0,(int *)) = ag_rp_124(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dxi_token_names
const char *const dxi_token_names[232] = {
  "syntax selection",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "SCRIPT",
  "WHILE LOOP",
  "REPEAT LOOP",
  "STATEMENT",
  "syntax selection",
  "local statement",
  "statement list",
  "eof",
  "loop condition",
  "\"DO\"",
  "statement",
  "UNTIL",
  "until condition",
  "';'",
  "\"SCRIPT\"",
  "\"WHILELOOP\"",
  "\"REPEATLOOP\"",
  "\"STATEMENT\"",
  "open statement",
  "closed statement",
  "if condition",
  "ELSE",
  "simple statement",
  "REPEAT",
  "WHILE",
  "for initialization",
  "assignment statement",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "expression",
  "']'",
  "structure declaration",
  "function definition",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "FOR",
  "\"FOR\"",
  "\"LOCAL\"",
  "name list",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "\"BY\"",
  "\"IF\"",
  "\"THEN\"",
  "\"ELSE\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "\"STRUCT\"",
  "'{'",
  "'}'",
  "\"FUNCTION\"",
  "'('",
  "')'",
  "",
  "function body",
  "simple expression",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "term",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "unary expression",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "factor",
  "\"NOT\"",
  "primary",
  "\"**\"",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "SCRIPT",
  "\"DO\"",
  "WHILE LOOP",
  "';'",
  "REPEAT LOOP",
  "STATEMENT",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"LOCAL\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "\"THEN\"",
  "\"IF\"",
  "\"ELSE\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'}'",
  "'{'",
  "\"STRUCT\"",
  "')'",
  "'('",
  "\"FUNCTION\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'='",
  "\"<>\"",
  "'+'",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "\"**\"",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dxi_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dxi_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dxi(dxi_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dxi(dxi_pcb_type *PCB_POINTER) {
  init_dxi(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dxi_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


